Package: CatsCradle
Title: This package provides methods for analysing spatial
        transcriptomics data and for discovering gene clusters
Version: 1.6.1
Authors@R: 
    c(person(given = "Anna",
        family = "Laddach",
        role = c("aut"),
        email = "anna.laddach@crick.ac.uk",
        comment = c(ORCID = "0000-0001-5552-6534")),
    person(given = "Michael",
        family = "Shapiro",
        role = c("aut","cre"),
        email = "michael.shapiro@crick.ac.uk",
        comment = c(ORCID = "0000-0002-2769-9320")))
Description: This package addresses two broad areas.  It allows for in-depth analysis of spatial transcriptomic data by identifying tissue neighbourhoods.  These are contiguous regions of tissue surrounding individual cells.  'CatsCradle' allows for the categorisation of neighbourhoods by the cell types contained in them and the genes expressed in them.  In particular, it produces Seurat objects whose individual elements are neighbourhoods rather than cells.  In addition, it enables the categorisation and annotation of genes by producing Seurat objects whose elements are genes.
License: MIT + file LICENSE
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Imports: Seurat (>= 5.0.1), ggplot2, networkD3, stringr, pracma,
        reshape2, rdist, igraph, geometry, Rfast, data.table, abind,
        pheatmap, EBImage, S4Vectors, SeuratObject,
        SingleCellExperiment, SpatialExperiment, Matrix, methods,
        SummarizedExperiment, msigdbr
Suggests: fossil, interp, knitr, BiocStyle, tictoc
Depends: R (>= 4.4.0)
LazyData: false
VignetteBuilder: knitr
BugReports: https://github.com/AnnaLaddach/CatsCradle/issues
URL: https://github.com/AnnaLaddach/CatsCradle
biocViews: BiologicalQuestion, StatisticalMethod, GeneExpression,
        SingleCell, Transcriptomics, Spatial
NeedsCompilation: no
Packaged: 2026-07-05 20:48:57 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/CatsCradle
git_branch: RELEASE_3_23
git_last_commit: c07e3c1
git_last_commit_date: 2026-07-02
Repository: Bioconductor 3.23
Date/Publication: 2026-07-05
Author: Anna Laddach [aut] (ORCID: <https://orcid.org/0000-0001-5552-6534>),
  Michael Shapiro [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-2769-9320>)
Maintainer: Michael Shapiro <michael.shapiro@crick.ac.uk>

# Generated by roxygen2: do not edit by hand

export(aggregateFeatureMatrix)
export(aggregateGeneExpression)
export(annotateGeneAsVector)
export(annotateGenesByGeneSet)
export(annotateLRInteractionCounts)
export(cellTypesPerCellTypeGraphFromCellMatrix)
export(cellTypesPerCellTypeGraphFromNbhdMatrix)
export(collapseExtendedNBHDs)
export(combinatorialSpheres)
export(computeCellTypesPerCellTypeMatrix)
export(computeEdgeGraph)
export(computeEdgeObject)
export(computeGraphEmbedding)
export(computeMoransI)
export(computeNBHDByCTMatrix)
export(computeNBHDVsCTObject)
export(computeNeighbourEnrichment)
export(computeNeighboursDelaunay)
export(computeNeighboursEuclidean)
export(countLRInteractionsPerCell)
export(cullEdges)
export(desymmetriseNN)
export(directedHausdorfDistance)
export(edgeCutoffsByClustering)
export(edgeCutoffsByPercentile)
export(edgeCutoffsByWatershed)
export(edgeCutoffsByZScore)
export(edgeLengthPlot)
export(edgeLengthsAndCellTypePairs)
export(exampleObjects)
export(geneSetsVsGeneClustersPValueMatrix)
export(getAverageExpressionDF)
export(getAverageExpressionMatrix)
export(getClusterOrder)
export(getExtendedNBHDs)
export(getFeatureZScores)
export(getGeneClusterAveragesPerCell)
export(getGeneNeighbors)
export(getLigandReceptorNetwork)
export(getLigandReceptorPairsInPanel)
export(getNearbyGenes)
export(getNearestNeighbourLists)
export(getObjectSubsetClusteringPValue)
export(getObjectSubsetClusteringStatistics)
export(getSubsetComponents)
export(make.getExample)
export(makeLRInteractionHeatmap)
export(makeSummedLRInteractionHeatmap)
export(meanGeneClusterOnCellUMAP)
export(meanZPerCluster)
export(meanZPerClusterOnUMAP)
export(medianComplementDistance)
export(medianComplementPValue)
export(nbhdsAsEdgesToNbhdsAsList)
export(neighbourhoodDiameter)
export(orderGeneSetPValues)
export(performLigandReceptorAnalysis)
export(plotLRDotplot)
export(predictAnnotation)
export(predictAnnotationAllGenes)
export(predictGeneAnnotationImpl)
export(randomiseGraph)
export(randomiseNodeIndices)
export(readGmt)
export(runGeometricClusteringTrials)
export(runMoransI)
export(sankeyFromMatrix)
export(stripGeneSet)
export(symmetriseNN)
export(symmetryCheckNN)
export(tagRowAndColNames)
export(transposeObject)
import(Seurat)
import(SeuratObject)
import(SingleCellExperiment)
import(SpatialExperiment)
import(ggplot2)
import(pheatmap)
importFrom(EBImage,watershed)
importFrom(Matrix,sparseMatrix)
importFrom(Rfast,colMins)
importFrom(Rfast,rowMins)
importFrom(Rfast,rowSort)
importFrom(S4Vectors,SelfHits)
importFrom(SummarizedExperiment,assays)
importFrom(abind,abind)
importFrom(data.table,data.table)
importFrom(data.table,merge.data.table)
importFrom(geometry,delaunayn)
importFrom(ggplot2,aes)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,theme)
importFrom(ggplot2,xlim)
importFrom(graphics,hist)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,layout_with_fr)
importFrom(methods,as)
importFrom(msigdbr,msigdbr)
importFrom(networkD3,sankeyNetwork)
importFrom(pracma,Norm)
importFrom(pracma,ceil)
importFrom(pracma,distmat)
importFrom(pracma,std)
importFrom(rdist,pdist)
importFrom(reshape2,melt)
importFrom(stats,aggregate)
importFrom(stats,fisher.test)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,pbinom)
importFrom(stats,phyper)
importFrom(stats,sd)
importFrom(stringr,str_replace)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_split_1)
importFrom(stringr,str_split_fixed)
importFrom(utils,data)

# CatsCradle 

<img src="vignettes/CatsCradleLogo.png" alt="" width="200"/>

CatsCradle is an R package which provides tools for exploring duality
present in single cell datasets. It interfaces with analyses performed
with the Seurat Package, although many functions take simple lists,
dataframes or matrices as input.

In a typical scRNAseq dataset duality exists between cells and the
genes they express. Cells are typically assigned cell types based on
gene expression; however, programmes of gene expression can cut across
cell types. Furthermore, information on how genes are coexpressed can
be leveraged to infer functionality. Please see vignette
[CatsCradle](vignettes/CatsCradle.Rmd) to find out how you can apply
CatsCradle to explore this duality.

Even more relationships exist in high resolution spatial
transcriptomics datasets, between genes, cells and their position in
space. Here we use graph-based representations of cell position, and
provide functionality for extending these graphs to account for
longer-range interactions.  We also provide functions for identifying
and exploring neighbourhoods; in terms of gene expression and cell
type composition.  Finally, we provide tools for analysing
ligand-receptor interactions, taking spatial information into
account. Please see vignette
[CatsCradleSpatial](vignettes/CatsCradleSpatial.Rmd) for details on
how to use CatsCradle to analyse spatial data.

To dive straight into CatsCradle please see our quickstart vignette
[CatsCradleQuickStart](vignettes/CatsCradleQuickStart.Rmd)!

## Installation

```
# Install devtools
if (!requireNamespace("devtools", quietly = TRUE)) {
  install.packages("devtools")
}

devtools::install_github("AnnaLaddach/CatsCradle")
```


#### Developed by Anna Laddach and Michael Shapiro in the Pachnis lab at the Francis Crick Institute.
     

## ####################################################
## These functions focus on detecting geometric clustering
## of points, e.g., gene sets on a gene UMAP.
## ####################################################


## ####################################################
#' This function computes a p-value for the geometric
#' clustering of a gene set (in UMAP or PCA reduction)
#' based on the median distance from its complement to
#' the set.
#'
#' @param fPrime - a transposed Seurat object, i.e. a
#' Seurat object of genes or SingleCellExperiment to
#' be converted to a Seurat object
#' @param geneSubset - a subset of the genes which can
#' be given as a character vector as a logical vector
#' @param numTrials - the number of random trials to be
#' carried out for randomised testing. Defaults to 1000.
#' @param reduction - can be 'UMAP' or 'PCA', defaults
#' to 'UMAP'
#' @param numPCs - number of PCs to use if reduction is
#' 'PCA'
#' @return A p-value reporting how often a random subset
#' of the same size is sufficiently clustered to produce
#' an equally large distance from its complement.
#' @importFrom pracma distmat Norm ceil std
#' @importFrom stats median aggregate fisher.test kmeans p.adjust sd
#' @importFrom graphics hist
#' @importFrom methods as
#' @importFrom utils data
#' @export
#' @examples
#' getExample = make.getExample()
#' STranspose = getExample('STranspose')
#' hallmark = getExample('hallmark',toy=TRUE)
#' geneSubset = intersect(colnames(STranspose),hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]])
#' p = getObjectSubsetClusteringPValue(STranspose,geneSubset,100)
getObjectSubsetClusteringPValue = function(fPrime,
                                           geneSubset,
                                           numTrials=1000,
                                           reduction='UMAP',
                                           numPCs=10)
{
    fPrime = acceptor(fPrime)
    
    ## Test for non-empty subset:
    if(isa(geneSubset,'logical'))
        numInSubset = sum(geneSubset)
    if(isa(geneSubset,'character'))
        numInSubset = length(intersect(geneSubset,
                                       colnames(fPrime)))
    stopifnot(numInSubset > 0)
    
    stats = getObjectSubsetClusteringStatistics(fPrime,
                                                geneSubset,
                                                numTrials,
                                                reduction,
                                                numPCs)
    
    return(stats$pValue)
}

## ####################################################
#' This function computes statistics for the geometric
#' clustering of a gene set (in UMAP or PCA reduction)
#' based on the median distance from its complement to
#' the set.
#'
#' @param fPrime - a transposed Seurat object, i.e. a
#' Seurat object of genes or SingleCellExperiment to
#' be converted to a Seurat object
#' @param geneSubset - a subset of the genes which can
#' be given as a character vector or as a logical vector
#' @param numTrials - the number of random trials to be
#' carried out for randomised testing. Defaults to 1000.
#' @param reduction - can be 'UMAP' or 'PCA', defaults
#' to 'UMAP'
#' @param numPCs - number of PCs to use if reduction is
#' 'PCA'
#' @return A list of statistics resulting from the
#' testing of randomised subsets of the same size as the
#' given gene subset.  These include subsetDistance, the 
#' actual median complement distance; randomSubsetDistance,
#' the median complement distances for randomised subsets;
#' pValue, computed by comparing the real and randomised
#' distances; and zScore, the z-distance of the actual
#' median distance from the mean of the randomised distances.
#' @export
#' @examples
#' getExample = make.getExample()
#' STranspose = getExample('STranspose',toy=TRUE)
#' hallmark = getExample('hallmark')
#' geneSubset = intersect(colnames(STranspose),hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]])
#' stats = getObjectSubsetClusteringStatistics(STranspose,geneSubset,100)
getObjectSubsetClusteringStatistics = function(fPrime,
                                               geneSubset,
                                               numTrials=1000,
                                               reduction='UMAP',
                                               numPCs=10)
{
    fPrime = acceptor(fPrime)
    
    if(reduction == 'UMAP')
        S = fetchUMAP(fPrime)

    if(reduction == 'PCA')
    {
        pcs = paste0('PC_',seq_len(numPCs))
        S = FetchData(fPrime,pcs)
    }

    ## The coordinates of all the genes in the
    ## chosen reduction:
    S = as.matrix(S)
    rownames(S) = colnames(fPrime)

    answer = runGeometricClusteringTrials(S,
                                          geneSubset,
                                          numTrials)

    return(answer)
}


## ####################################################
#' This runs random trials to determine the statistical
#' significance of the clustering of a set of points
#' within a larger set.
#' 
#' This function takes a matrix whose rows are geometric
#' coordinates and a subset of these points either given
#' as a character vector which is a subset of the rownames
#' or as a logical vector.  It returns statistics on the
#' mean distance of the complement to the subset.
#'
#' @param S - a set of points given as a matrix. The rows
#' are the coordinates of these points
#' @param geneSubset - this is either a subset of the rownames of
#' S or a logical whose length is nrow(S)
#' @param numTrials - the number or random trials to perform
#' @return This returns a list. subsetDistance gives the
#' median complement distance for the actual set,
#' randomSubsetDistance gives the complement distances for
#' the numTrials random sets, pValue gives a p-value based
#' on the rank of the actual distance among the random
#' distances and zScore gives its z-score.
#' @export
#' @examples
#' library(Seurat)
#' getExample = make.getExample()
#' STranspose = getExample('STranspose',toy=TRUE)
#' hallmark = getExample('hallmark')
#' S = data.matrix(FetchData(STranspose,c('umap_1','umap_2')))
#' geneSubset = rownames(S) %in% hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]]
#' geneClustering = runGeometricClusteringTrials(S,geneSubset,100)
runGeometricClusteringTrials = function(S,
                                        geneSubset,
                                        numTrials)
{
    if(inherits(geneSubset,'character'))
        geneSubset = rownames(S) %in% geneSubset
    
    answer = list()
    answer$subsetDistance = medianComplementDistance(S,geneSubset)

    randomSubsetDistance = c()
    for(i in seq_len(numTrials))
    {
        randomSubset = sample(rownames(S),sum(geneSubset))
        randomSubset = rownames(S) %in% randomSubset
        randomSubsetDistance[i] = medianComplementDistance(S,randomSubset)
    }
    answer$randomSubsetDistance = randomSubsetDistance

    count = sum(randomSubsetDistance > answer$subsetDistance)
    count = max(count,1)

    answer$pValue = count / numTrials

    mu = mean(randomSubsetDistance)
    std = sd(randomSubsetDistance)
    answer$zScore = (answer$subsetDistance - mu) / std

     return(answer)
}


## ####################################################
## Used internally
medianComplementDistance = function(S,geneSubset)
{
    if(inherits(geneSubset,'character'))
        geneSubset = colnames(S) %in% geneSubset
    
    ## The complement:
    A = S[!geneSubset,]
    
    ## The subset:
    B = S[geneSubset,]
    
    D = distmat(A,B)
    
    rowMin = c()
    for(i in seq_len(nrow(D)))
        rowMin[i] = min(D[i,])
    d = median(rowMin)
    
    return(d)
}


## ####################################################
#' This finds the directed Hausdorf distance from A to B
#'
#' @param A - an m x d matrix representing m points in
#' dimension d
#' @param B - an n x d matrix representing n points in
#' dimension d
#' @return This returns the distance of the furthest point
#' in A from its nearest point in B.
#' @export
#' @examples
#' A = matrix(seq_len(8),ncol=2)
#' B = matrix(seq(from=3,to=16),ncol=2)
#' d_hausdorf = directedHausdorfDistance(A,B)
directedHausdorfDistance = function(A,B)
{
    D = distmat(A,B)
    
    rowMin = c()
    for(i in seq_len(nrow(D)))
        rowMin[i] = min(D[i,])
    d = max(rowMin)
    
    return(d)
}

## ####################################################
directedMedianDistance =function(A,B)
{
    D = distmat(A,B)
    
    rowMin = c()
    for(i in seq_len(nrow(D)))
        rowMin[i] = min(D[i,])
    d = median(rowMin)
    
    return(d)
}

## ####################################################
#' This takes a set S of n points in dimension d given
#' by an n x d matrix and a subset A given by a logical
#' and returns the median distance from the complement to
#' the given subset.
#'
#' @param S - an n x d matrix representing a set of n points
#' in dimension d
#' @param idx - a logical of length n representing a subset of
#' S.  This should not be the empty set or all of S.
#' @return This returns the median distance from the complement
#' to the subset
#' @export
#' @examples
#' S = matrix(seq_len(12),ncol=2)
#' idx = c(rep(FALSE,3),rep(TRUE,3))
#' compDist = medianComplementDistance(S,idx)
medianComplementDistance = function(S,idx)
{
    ## The complement:
    A = S[!idx,]
    
    ## The subset:
    B = S[idx,]
    
    return(directedMedianDistance(A,B))
}

## ####################################################
#' This takes a set S of n points in dimension d and a subset
#' A and computes a p-value for the co-localization of the subset
#' by comparing the median complement distance for the given set
#' to values of the median complement distance computed for random
#' subsets of the same size.
#'
#' @param  S - an n x d matrix representing a set of n points
#' in dimension d
#' @param idx - a logical of length n representing a subset of
#' S.  This should not be the empty set or all of S.
#' @param numTrials - the number of random trials to perform,
#' defaults to 1000
#' @param returnTrials - whether to report the real and random median
#' complement distances.
#' @return By default this reports a p-value.  If returnTrials is set,
#' this returns a list giving the p-value, the actual complement distance
#' and the random complement distances.
#' @export
#' @examples
#' library(Seurat)
#' getExample = make.getExample()
#' STranspose = getExample('STranspose',toy=TRUE)
#' hallmark = getExample('hallmark')
#' S = data.matrix(FetchData(STranspose,c('umap_1','umap_2')))
#' idx = colnames(STranspose) %in% hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]]
#' mcpv = medianComplementPValue(S,idx,numTrials=100)
medianComplementPValue = function(S,idx,numTrials=1000,returnTrials=FALSE)
{
    actual = medianComplementDistance(S,idx)
    
    numSuper = nrow(S)
    numSub = sum(idx)
    
    random = c()

    for(i in seq_len(numTrials))
    {
        IDX = rep(FALSE,numSuper)
        r = sample(numSuper,numSub)
        IDX[r] = TRUE
        random[i] = medianComplementDistance(S,IDX)
    }
    n = sum(random >= actual)
    n = max(1,n)
    
    answer = list()
    answer$pValue = n / numTrials    
    answer$medianDist = actual
    answer$random = random

    if(!returnTrials)
        return(answer$pValue)
    
    return(answer)
}

 
## ####################################################
#' This is designed to dectect the components of a
#' gene subset in the case where median complement
#' distance detects clustering.
#'
#' @param fPrime - a gene Seurat object or
#' SingleCellExperiment
#' @param theSubset - a subset of the genes
#' @param alpha - a parameter typically less than
#' one controling the granularity of the components.
#' Defaults to .5
#' @param edgeCut - the maximum length of edges included in
#' the subgraph whose components are returned.  If it
#' is NA (the default) it is computed using alpha.
#' Otherwise, it can be supplied directly.
#' @return A list of the components of the subset
#' treated as a graph whose edges are determined by
#' their distance in UMAP coordinates.
#' @export
getSubsetComponents = function(fPrime,
                               theSubset,
                               alpha=.5,
                               edgeCut=NA)
{
    fPrime = acceptor(fPrime)
    
    ## Subset to the gene subset in question:
    fSubset = fPrime[,theSubset]
    centroids = FetchData(fSubset,c('umap_1','umap_2'))

    ## Triangulate the subset:
    triangulation = computeNeighboursDelaunay(centroids)
    edgeLengths = c()

    ## Get the edge lengths:
    for(i in 1:nrow(triangulation))
    {
        a = as.numeric(centroids[triangulation$nodeA[i],])
        b = as.numeric(centroids[triangulation$nodeB[i],])
        delta = a - b
                       
        edgeLengths[i] = Norm(delta)
    }


    if(is.na(edgeCut))
    {
        ## Find the cut by 2-means clustering of edge lengths:
        k = kmeans(edgeLengths,centers=c(min(edgeLengths),max(edgeLengths)))
        M1 = max(edgeLengths[k$cluster==1])
        m2 = min(edgeLengths[k$cluster==2])

        cut = mean(M1,m2)
        edgeCut = alpha * cut
    }

    ## Subset triangulation to shorter edges:
    idx = edgeLengths <= edgeCut
    subgraph = triangulation[idx,]
    vertices = unique(c(subgraph$nodeA,subgraph$nodeB))
    vertices = vertices[order(vertices)]
    numVertices = length(vertices)

    ## Get adjacency matrix:
    M = matrix(0,nrow=numVertices,ncol=numVertices)
    rownames(M) = vertices
    colnames(M) = vertices
    for(i in 1:nrow(subgraph))
    {
        a = subgraph$nodeA[i]
        b = subgraph$nodeB[i]
        M[a,b] = 1
        M[b,a] = 1
    }

    ## Get igraph:
    subsetAsIgraph = igraph::graph_from_adjacency_matrix(M)
    components = igraph::components(subsetAsIgraph)

    theComponents = list()
    for(i in 1:components$no)
    {
        idx = components$membership == i
        theComponents[[i]] = vertices[idx]
    }

    return(theComponents)
}

## ----[Sp1] setup, include = FALSE, warning = FALSE----------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    fig.dim = c(6,6),
    comment = "#>"
)

## ----[Sp2], message=FALSE-----------------------------------------------------
library(Seurat,quietly=TRUE)
library(CatsCradle,quietly=TRUE)
getExample = make.getExample()
smallXenium = getExample('smallXenium')
ImageDimPlot(smallXenium, cols = "polychrome", size = 1)

## ----[Sp3]--------------------------------------------------------------------
centroids = GetTissueCoordinates(smallXenium)
rownames(centroids) = centroids$cell
clusters = smallXenium@active.ident
delaunayNeighbours = computeNeighboursDelaunay(centroids)
head(delaunayNeighbours)

## ----[Sp4]--------------------------------------------------------------------
idx = (delaunayNeighbours$nodeA == 16307 |
       delaunayNeighbours$nodeB == 16307)
nbhd = unique(c(delaunayNeighbours$nodeA[idx],
                delaunayNeighbours$nodeB[idx]))
nbhd		

## ----[Sp5]--------------------------------------------------------------------
extendedNeighboursList = getExtendedNBHDs(delaunayNeighbours, 4)
extendedNeighbours = collapseExtendedNBHDs(extendedNeighboursList, 4)

## ----[Sp6]--------------------------------------------------------------------
idx = (extendedNeighbours$nodeA == 16307 |
       extendedNeighbours$nodeB == 16307)
nbhd = unique(c(extendedNeighbours$nodeA[idx],
                extendedNeighbours$nodeB[idx]))
length(nbhd)		

## ----[Sp7]--------------------------------------------------------------------
euclideanNeighbours = computeNeighboursEuclidean(centroids,
threshold=20)

## ----[Sp8]--------------------------------------------------------------------
agg = aggregateGeneExpression(smallXenium,extendedNeighbours,
                                    verbose=FALSE)
smallXenium$aggregateNBHDClusters = agg@active.ident
ImageDimPlot(smallXenium,group.by='aggregateNBHDClusters',cols='polychrome')

## ----[Sp9]--------------------------------------------------------------------
NBHDByCTMatrix = computeNBHDByCTMatrix(delaunayNeighbours, clusters)

## ----[Sp10]-------------------------------------------------------------------
NBHDByCTMatrixExtended = 
  computeNBHDByCTMatrix(extendedNeighbours, clusters)

## ----[Sp11]-------------------------------------------------------------------
cellTypesPerCellTypeMatrix = 
  computeCellTypesPerCellTypeMatrix(NBHDByCTMatrix,clusters)

## ----[Sp12]-------------------------------------------------------------------
colours = DiscretePalette(length(levels(clusters)), palette = "polychrome")
names(colours) = levels(clusters)

cellTypesPerCellTypeGraphFromCellMatrix(cellTypesPerCellTypeMatrix, 
                                    minWeight = 0.05, colours = colours)

## ----[Sp13], message=FALSE----------------------------------------------------
library(pheatmap,quietly=TRUE)
pheatmap(cellTypesPerCellTypeMatrix)

## ----[Sp14]-------------------------------------------------------------------
cellTypesPerCellTypeMatrixExtended = computeCellTypesPerCellTypeMatrix(NBHDByCTMatrixExtended,clusters)

cellTypesPerCellTypeGraphFromCellMatrix(cellTypesPerCellTypeMatrixExtended,
minWeight = 0.05, colours = colours)

## ----[Sp15]-------------------------------------------------------------------
cellTypesPerCellTypePValues = computeNeighbourEnrichment(delaunayNeighbours, 
                                          clusters)

## ----[Sp16]-------------------------------------------------------------------
cellTypesPerCellTypePValuesNegLog = -log10(cellTypesPerCellTypePValues + 0.001)
pheatmap(cellTypesPerCellTypePValuesNegLog)

## ----[Sp17]-------------------------------------------------------------------
NBHDByCTSeurat = computeNBHDVsCTObject(NBHDByCTMatrix,verbose=FALSE)

## ----[Sp18]-------------------------------------------------------------------
NBHDByCTSeurat$cellType = clusters

## ----[Sp19]-------------------------------------------------------------------
DimPlot(NBHDByCTSeurat, group.by = c("cellType"), cols = "polychrome", reduction = "umap")
DimPlot(NBHDByCTSeurat, group.by = c("neighbourhood_clusters"), cols = "polychrome", reduction = "umap")

## ----[Sp20]-------------------------------------------------------------------
smallXenium$NBHDCluster = NBHDByCTSeurat@active.ident
ImageDimPlot(smallXenium, group.by = "NBHDCluster", size = 1, cols = "polychrome")

## ----[Sp21]-------------------------------------------------------------------
NBHDByCTSeuratExtended = computeNBHDVsCTObject(NBHDByCTMatrixExtended,
                                               verbose=FALSE)

## ----[Sp22]-------------------------------------------------------------------
NBHDByCTSeuratExtended$cellType = clusters

## ----[Sp23]-------------------------------------------------------------------
DimPlot(NBHDByCTSeuratExtended, group.by = c("cellType"), cols = "polychrome", reduction = "umap")
DimPlot(NBHDByCTSeuratExtended, group.by = c("neighbourhood_clusters"), cols = "polychrome", reduction = "umap")

## ----[Sp24]-------------------------------------------------------------------
smallXenium$NBHDClusterExtended= 
  NBHDByCTSeuratExtended@active.ident
ImageDimPlot(smallXenium, group.by = c("NBHDClusterExtended"), 
             size = 1, cols = "polychrome")

## ----[Sp25]-------------------------------------------------------------------
CTByNBHDCluster = table(NBHDByCTSeurat$cellType,NBHDByCTSeurat@active.ident)
CTByNBHDCluster = CTByNBHDCluster/rowSums(CTByNBHDCluster)

rownames(CTByNBHDCluster) = paste0("CellType",rownames(CTByNBHDCluster))
colnames(CTByNBHDCluster) = paste0("NBHDCluster",colnames(CTByNBHDCluster))

pheatmap(CTByNBHDCluster,
      fontsize_row=8,
      fontsize_col=8,
      cellheight=10,
      cellwidth=10)

sankeyFromMatrix(CTByNBHDCluster)

## ----[Sp26]-------------------------------------------------------------------
CTByNBHDClusterExtended = table(NBHDByCTSeuratExtended$cellType,NBHDByCTSeuratExtended@active.ident)
CTByNBHDClusterExtended = CTByNBHDClusterExtended/rowSums(CTByNBHDClusterExtended)

rownames(CTByNBHDClusterExtended) = paste0("CellType",rownames(CTByNBHDClusterExtended))
colnames(CTByNBHDClusterExtended) = paste0("NBHDCluster",colnames(CTByNBHDClusterExtended))

pheatmap(CTByNBHDClusterExtended,
      fontsize_row=8,
      fontsize_col=8,
      cellheight=10,
      cellwidth=10)

sankeyFromMatrix(CTByNBHDClusterExtended)

## ----[Sp27]-------------------------------------------------------------------
CTByNBHDSeurat = 
  computeNBHDVsCTObject(t(NBHDByCTMatrix), npcs = 10, 
                        transpose = TRUE, resolution = 1, n.neighbors = 5,
			verbose=FALSE)

CTByNBHDSeurat$cellType = colnames(CTByNBHDSeurat)

DimPlot(CTByNBHDSeurat, group.by = "cellType", cols = "polychrome", 
        reduction = "umap", label = TRUE)

## ----[Sp28]-------------------------------------------------------------------
CTByNBHDSeurat= computeGraphEmbedding(CTByNBHDSeurat)

DimPlot(CTByNBHDSeurat,group.by = "cellType", cols = "alphabet", reduction = "graph", label = TRUE)

## ----[Sp29]-------------------------------------------------------------------
ImageDimPlot(smallXenium, cols = "polychrome", size = 1)

## ----[Sp30]-------------------------------------------------------------------
pca = Embeddings(CTByNBHDSeurat, reduction = "pca")
res = pheatmap(pca)

## ----[Sp31]-------------------------------------------------------------------
CTClust = cutree(res$tree_row, k = 11)
CTByNBHDSeurat$neighbourhood_clusters = factor(CTClust)

## ----[Sp32]-------------------------------------------------------------------
CTComposition = table(CTByNBHDSeurat$cellType, CTByNBHDSeurat$neighbourhood_clusters)
pheatmap(CTComposition)

## ----[Sp33]-------------------------------------------------------------------
averageExpMatrix = getAverageExpressionMatrix(NBHDByCTSeurat,
                           CTByNBHDSeurat,
			    clusteringName='neighbourhood_clusters')
averageExpMatrix = tagRowAndColNames(averageExpMatrix,
                                     ccTag='neighbourhoodClusters_',
                                     gcTag='cellTypeClusters_')
pheatmap(averageExpMatrix,
      fontsize_row=8,
      fontsize_col=8,
      cellheight=10,
      cellwidth=10)


sankeyFromMatrix(averageExpMatrix)

## ----[Sp34]-------------------------------------------------------------------
moransI = runMoransI(smallXenium, delaunayNeighbours, assay = "SCT", 
                     layer = "data", nSim = 20, verbose = FALSE)

## ----[Sp35]-------------------------------------------------------------------
head(moransI)

## ----[Sp36]-------------------------------------------------------------------
tail(moransI)

## ----[Sp37]-------------------------------------------------------------------
ImageFeaturePlot(smallXenium, "Nwd2")

## ----[Sp38]-------------------------------------------------------------------
ImageFeaturePlot(smallXenium, "Trbc2")

## ----[Sp39]-------------------------------------------------------------------
ligandReceptorResults = performLigandReceptorAnalysis(smallXenium, delaunayNeighbours, 
                                                "mouse", clusters, 
                                                method = "analytical", 
                                                conditional = FALSE,
                                                minEdgesPos = 10)

## ----[Sp40]-------------------------------------------------------------------
head(as.matrix(ligandReceptorResults$interactionsOnEdges[,1:10]),10)

## ----[Sp41]-------------------------------------------------------------------
head(ligandReceptorResults$totalInteractionsByCluster[,1:10],10)

## ----[Sp42]-------------------------------------------------------------------
head(ligandReceptorResults$meanInteractionsByCluster[,1:10],10)

## ----[Sp43]-------------------------------------------------------------------
head(ligandReceptorResults$pValues,10) 

## ----[Sp44]-------------------------------------------------------------------
ligRecMatrix = makeLRInteractionHeatmap(ligandReceptorResults, clusters, colours = colours, labelClusterPairs = FALSE)

## ----[Sp45]-------------------------------------------------------------------
cellTypePerCellTypeLigRecMatrix = makeSummedLRInteractionHeatmap(ligandReceptorResults, clusters, "total")

## ----[Sp46]-------------------------------------------------------------------
cellTypePerCellTypeLigRecMatrix = makeSummedLRInteractionHeatmap(ligandReceptorResults, clusters, "mean", logScale = TRUE)

## ----[Sp47]-------------------------------------------------------------------
p = plotLRDotplot(ligandReceptorResults, padjCutoff = 0.05, pvalCutoff = F, splitBy = "sender")

## ----[Sp48]-------------------------------------------------------------------
plotLRDotplot(ligandReceptorResults, senderClusters = c("6_VLMC","7_Granule"), 
              padjCutoff = 0.05, pvalCutoff = F, splitBy = "sender")

## ----[Sp49]-------------------------------------------------------------------
hist(cellTypePerCellTypeLigRecMatrix)

## ----[Sp50]-------------------------------------------------------------------
cellTypesPerCellTypeGraphFromCellMatrix(cellTypePerCellTypeLigRecMatrix, 
                                    minWeight = 0.4, colours = colours)

## ----[Sp51]-------------------------------------------------------------------
scaleFactor = 3
cellTypesPerCellTypeGraphFromCellMatrix(cellTypePerCellTypeLigRecMatrix/scaleFactor, 
                                    minWeight = 0.4/scaleFactor, colours = colours)

## ----[Sp52]-------------------------------------------------------------------
edgeSeurat = computeEdgeObject(ligandReceptorResults, centroids)

## ----[Sp53]-------------------------------------------------------------------
ImageFeaturePlot(edgeSeurat, features = "Pdyn-Npy2r")

## ----[Sp54]-------------------------------------------------------------------
edgeNeighbours = computeEdgeGraph(delaunayNeighbours)

## ----[Sp55] , eval=FALSE------------------------------------------------------
# moransILigandReceptor = runMoransI(edgeSeurat, edgeNeighbours, assay = "RNA",
#                      layer = "counts", nSim = 100)

## ----[Sp56]-------------------------------------------------------------------
moransILigandReceptor = getExample('moransILigandReceptor')
head(moransILigandReceptor)

## ----[Sp57]-------------------------------------------------------------------
tail(moransILigandReceptor)

## ----[Sp58]-------------------------------------------------------------------
ImageFeaturePlot(edgeSeurat, "Penk-Htr1f")

## ----[Sp59]-------------------------------------------------------------------
ImageFeaturePlot(edgeSeurat, "Sst-Gpr17")

## ----[Sp60]-------------------------------------------------------------------
annEdges =
edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
head(annEdges)

## ----[Sp61]-------------------------------------------------------------------
cutoffDF = edgeCutoffsByPercentile(annEdges,percentileCutof=95)
g = edgeLengthPlot(annEdges,cutoffDF,whichPairs=60)
print(g)

## ----[Sp62]-------------------------------------------------------------------
cutoffDF = edgeCutoffsByClustering(annEdges)

## ----[Sp63]-------------------------------------------------------------------
cutoffDF = edgeCutoffsByPercentile(annEdges,percentileCutoff=95)

## ----[Sp65]-------------------------------------------------------------------
cutoffDF = edgeCutoffsByZScore(annEdges,zCutoff=1.5)

## ----[Sp66]-------------------------------------------------------------------
cutoffDF = edgeCutoffsByWatershed(annEdges,nbins=15,tolerance=10)

## ----[Sp67]-------------------------------------------------------------------
cutoffDF = edgeCutoffsByWatershed(annEdges,nbins=15,tolerance=10)
culledEdges = cullEdges(annEdges,cutoffDF)
nrow(annEdges)
nrow(culledEdges)

## ----[Sp68]-------------------------------------------------------------------
sessionInfo()

## ####################################################
## These functions are used internally and are not
## exported.
## ####################################################

## ####################################################
## Extracts a p-value for gene set overlap from their
## sizes and the size of the background.
geneListPValue = function(A,B,C,background=25000)
{
  M = matrix(0,nrow=2,ncol=2)
  
  M[1,1] = background - A
  M[1,2] = A
  M[2,1] = B - C
  M[2,2] = C
  
  f = fisher.test(M,alternative='greater')
  
  return(f$p.value)
}


## ####################################################
## Extracts a default graph from a Seurat object.
defaultGraph = function(f)
{
    graph = paste0(f@active.assay,'_snn')
    return(graph)
}

## ####################################################
## Used to fetch umap agnostically as to upper or lower case:
fetchUMAP = function(f)
{
    umapIsCalled = names(f@reductions$umap)
    if(is.null(umapIsCalled))
        umapIsCalled = names(f@reductions$UMAP)
    df = FetchData(f,umapIsCalled)

    return(df)
}

## ####################################################
## This extracts the cells from a nearest neighbour graph.
extractCells = function(NN)
{
    cells = unique(c(NN$nodeA,NN$nodeB))
    cells = cells[order(cells)]
    return(cells)
}


## ####################################################
## The following functions are used internally to manage
## conversions between Seurat objects on the one hand and
## SingleCellExperiments and SpatialExperiments on the
## other.  These are not necessarily lossless conversions.
## ####################################################
acceptor = function(obj)
{
    ## We only convert if we need to:
    if(isa(obj,'SingleCellExperiment'))
        return(SCEtoSeurat(obj))

    return(obj)
}

## ####################################################
returnAs = function(obj,returnType,spatial=FALSE)
{
    if(returnType %in% c('SCE',
                         'sce',
                         'SingleCellExperiment',
                         'singlecellexperiment'))
        return(SeuratToSCE(obj,spatial))

    return(obj)
}

## ####################################################
SCEtoSeurat = function(sce)
{
    if('counts' %in% names(assays(sce))) {
        f = as.Seurat(sce)
    } else {
        f = as.Seurat(sce,counts=NULL) }
    
    numCells = ncol(sce)
    for(n in colPairNames(sce))
    {
        df = colPair(sce,n)
        df = as.data.frame(df)

        ## Factorize:
        df[,1] = factor(df[,1])
        df[,2] = factor(df[,2])

        ## Make sparse matrix:
        M = sparseMatrix(i=as.numeric(df[,1]),
                         j=as.numeric(df[,2]),
                         x = as.numeric(df[,3]),
                         dimnames = list(levels(df[,1]), levels(df[,2])))

        ## Make graph and install:
        graph = as.Graph(M)
        f@graphs[[n]] = graph
    }

    return(f)
}

## ####################################################
SeuratToSCE = function(f,spatial=FALSE)
{
    sce = as.SingleCellExperiment(f)
    
    for(n in names(f@graphs))
    {
        NN = getNearestNeighbourLists(f,n)
        numPairs = nrow(NN)

        a = seq_len(ncol(sce))
        names(a) = colnames(sce)
        cell1 = a[NN$nodeA]
        cell2 = a[NN$nodeB]

        names(cell1) = NULL
        names(cell2) = NULL
        
        colPair(sce,n) = SelfHits(cell1,
                                  cell2,
                                  nnode=ncol(sce),
                                  value=NN$weight)
    }

    if(spatial)
    {
        sce = SpatialExperiment(sce)

        for(n in names(f@meta.data))
            colData(sce)[,n] = f@meta.data[,n]

        ## Copy in the centroids:
        centroids = GetTissueCoordinates(f)
        rownames(centroids) = centroids$cell
        centroids = centroids[,seq_len(2)]
        centroids = data.matrix(centroids)

        spatialCoords(sce) = centroids
    }
    return(sce)
}

## ####################################################
## Functions for quality control of neighbouring cells
## ####################################################

## ####################################################
#' This function annotates edges with their distance and
#' the types of cells they connect
#'
#' @param edges - A data frame with columns nodeA and nodeB giving the
#'     cells of each edge
#' @param clusters - the clusters of each cell
#' @param centroids - the centroids of each cell
#' @return a data frame giving the edges (as nodeA and nodeB), their
#'     lengths and the cell type pair.
#' @export
#' @examples
#' getExample = make.getExample()
#' centroids = getExample('centroids')
#' clusters = getExample('clusters')
#' delaunayNeighbours = getExample('delaunayNeighbours')
#' annEdges = edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
edgeLengthsAndCellTypePairs = function(edges,clusters,centroids)
{
  coords  = cbind(centroids[edges$nodeA,c("x","y")],centroids[edges$nodeB,c("x","y")])
  colnames(coords)= c("x1","y1","x2","y2")
  edges$length = sqrt((coords$x2 - coords$x1)^2 + (coords$y2 - coords$y1)^2)
  sortedClusters = t(apply(cbind(clusters[edges$nodeA], clusters[edges$nodeB]), 1, sort))
  edges$cellTypePair = paste(sortedClusters[,1], sortedClusters[,2], sep = "_")
  return(edges)    
}

## ####################################################
#' This finds proposed cutoffs for edge lengths by clustering
#' the lengths of the edges for each cell type pair using k-means
#' clustering with k  = 2
#'
#' @param annEdges - a data frame with columns nodeA, nodeB, length
#'     and cellTypePair as produced by edgeLengthsAndCellTypePairs.
#' @return This returns a data frame with columns cellTypePair and
#'     cutoff. 
#' @export
#' @examples
#' getExample = make.getExample()
#' centroids = getExample('centroids')
#' clusters = getExample('clusters')
#' delaunayNeighbours = getExample('delaunayNeighbours')
#' annEdges = 
#'     edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
#' cutoffDF = edgeCutoffsByClustering(annEdges)
edgeCutoffsByClustering = function(annEdges)
{
    cellTypePairs = unique(annEdges$cellTypePair)
    cutoff = c()
    for(ctp in cellTypePairs)
    {
        idx = annEdges$cellTypePair == ctp
        lengths = annEdges$length[idx]
        centers = c(min(lengths),max(lengths))
        if(length(lengths) <= 2)
        {
            cutoff[ctp] = mean(lengths)
            next
        }
        km = kmeans(lengths,2)

        A = lengths[km$cluster == 1]
        B = lengths[km$cluster == 2]

        if(max(A) < min(B))
            cutoff[ctp] = (max(A) + min(B)) / 2
        else
            cutoff[ctp] = (max(B) + min(A)) / 2
    }
    df = data.frame(cellTypePair=names(cutoff),cutoff)

    return(df)
}

## ####################################################
#' This finds edge cutoffs by percentile
#'
#'@param annEdges - a data frame with columns nodeA, nodeB, length
#'     and cellTypePair as produced by edgeLengthsAndCellTypePairs.
#' @param percentileCutoff - a numeric
#' @return This returns a data frame with columns cellTypePair and
#'     cutoff.
#' @export
#' @examples
#' getExample = make.getExample()
#' centroids = getExample('centroids')
#' clusters = getExample('clusters')
#' delaunayNeighbours = getExample('delaunayNeighbours')
#' annEdges =
#'     edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
#' cutoffDF = edgeCutoffsByPercentile(annEdges,percentileCutoff=95)
edgeCutoffsByPercentile = function(annEdges,
                                   percentileCutoff)
{
    cellTypePairs = unique(annEdges$cellTypePair)

    cutoff = c()
    for(ctp in cellTypePairs)
    {
        idx = annEdges$cellTypePair == ctp
        lengths = annEdges$length[idx]
        lengths = lengths[order(lengths)]
        num = length(lengths)
        n = ceil(percentileCutoff * num / 100)
        n = max(1,n)
        n = min(n,num)
        cutoff[ctp] = lengths[n]
    }
    cutoffDF = data.frame(cellTypePair=names(cutoff),
                          cutoff)

    return(cutoffDF)
}


## ####################################################
#' This finds edge cutoffs by z-score
#'
#'@param annEdges - a data frame with columns nodeA, nodeB, length
#'     and cellTypePair as produced by edgeLengthsAndCellTypePairs.
#' @param zCutoff - a numeric
#' @return This returns a data frame with columns cellTypePair and
#'     cutoff.
#' @export
#' @examples
#' getExample = make.getExample()
#' centroids = getExample('centroids')
#' clusters = getExample('clusters')
#' delaunayNeighbours = getExample('delaunayNeighbours') 
#' annEdges =
#'     edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
#' cutoffDF = edgeCutoffsByZScore(annEdges,zCutoff=1.5)
edgeCutoffsByZScore = function(annEdges,zCutoff)
{
    cellTypePairs = unique(annEdges$cellTypePair)

    cutoff = c()
    for(ctp in cellTypePairs)
    {
        idx = annEdges$cellTypePair == ctp
        lengths = annEdges$length[idx]

        cutoff[ctp] = mean(lengths) + zCutoff * std(lengths)
    }
    cutoffDF = data.frame(cellTypePair=names(cutoff),
                          cutoff)

    return(cutoffDF)       
}


## ####################################################
#' This finds proposed cutoffs for edge lengths by computing
#' the histogram of edge lengths for each cell type pair and
#' then using the watershed algorithm to find the hump of the
#' histogram containing the median.
#'
#' @param annEdges - a data frame with columns nodeA, nodeB, length
#'     and cellTypePair as produced by edgeLengthsAndCellTypePairs.
#' @param tolerance - the tolerance parameter for the watershed
#'     algorithm.
#' @param nbins - the number of bins for the histogram
#' @return This returns a data frame with columns cellTypePair and
#'     cutoff.
#' @importFrom EBImage watershed
#' @export
#' @examples
#' getExample = make.getExample()
#' centroids = getExample('centroids')
#' clusters = getExample('clusters')
#' delaunayNeighbours = getExample('delaunayNeighbours') 
#' annEdges =
#'     edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
#' cutoffDF = edgeCutoffsByWatershed(annEdges)
edgeCutoffsByWatershed = function(annEdges,nbins=15,tolerance=10)
{
    cellTypePairs = unique(annEdges$cellTypePair)
    cutoff = c()

    for(ctp in cellTypePairs)
    {
        lengths = annEdges$length[annEdges$cellTypePair == ctp]
        
        a = hist(lengths,nbins,plot=FALSE)
        d = a$density
        d = matrix(d,nrow=1)
        res = watershed(d,tolerance)

        med = median(lengths)
        ## Where median occurs:
        idx = a$mids >= med

        ## Maybe there are no bins beyond the median:
        if(sum(idx) == 0)
        {
            delta = a$mids[2] - a$mids[1]
            cutoff[ctp] = a$mids[length(a$mids)] + delta
            next
        }
        clusteredAs = res[min(which(idx),na.rm=TRUE)]
        upTo = max(which(res == clusteredAs),na.rm=TRUE)

        if(length(a$mid) == upTo)
            cutoff[ctp] = a$mid[upTo]
        else
            cutoff[ctp] = a$mid[upTo+1]
    }
    cutoffDF = data.frame(cellTypePair=names(cutoff),
                          cutoff)

    return(cutoffDF)
}


## ####################################################
#' edgeLengthPlot
#'
#' This plots histograms of the edge lengths broken out
#' by the cell types of the cells they connect.  It
#' optionally plots a cutoff for each pair of types.
#'
#' @param annEdges - A data frame as produced by
#'     edgeLengthsAndCellTypePairs
#' @param cutoffDF - A data frame with columns cellTypePair and
#'     cutoff. This defaults to NULL in which case no cutoffs will be
#'     plotted. 
#' @param whichPairs - Which cellTypePairs to plot.  If this is NULL,
#'     we plot all pairs.  If this is a numeric, we plot only pairs
#'     that have at least this many edges.  If this is a character
#'     vector, we plot the pairs in this list.
#' @param xLim - limits the extent of the plots. Defaults to 100.  Can
#'     be set to NULL.
#' @param legend - Show legend, defaults to FALSE
#' @return This returns a ggplot object
#' @export
#' @examples
#' getExample = make.getExample()
#' centroids = getExample('centroids')
#' clusters = getExample('clusters')
#' delaunayNeighbours = getExample('delaunayNeighbours') 
#' annEdges =
#'    edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
#' cutoffDF = edgeCutoffsByPercentile(annEdges,95)
#' g = edgeLengthPlot(annEdges,cutoffDF,whichPairs=60)
edgeLengthPlot = function(annEdges,
                          cutoffDF,
                          whichPairs,
                          xLim=100,
                          legend=FALSE)
{
    ## ####################################################
    ## What to plot:

    ## All:
    if(is.null(whichPairs))
    {
        useThesPairs = unique(annEdges$cellTypePair)
    }

    ## Sufficiently large:
    if(isa(whichPairs,'numeric'))
    {
        a = table(annEdges$cellTypePair)
        b = as.numeric(a)
        names(b) = names(a)
        useThesePairs = names(b[b >= whichPairs])
    }

    ## A pre-chosen set:
    if(isa(whichPairs,'character'))
    {
        useThesePairs = whichPairs
    }

    ## Make the plots:
    idx = annEdges$cellTypePair %in% useThesePairs
    plotDF = annEdges[idx,]

    g = ggplot(plotDF,aes(x=length,fill=cellTypePair)) +
        geom_density() +
        facet_wrap(~cellTypePair)

    ## Do we want a legend:
    if(! legend)
        g = g + theme(legend.position='none')

    ## Do we want an xlim?
    if(! is.null(xlim))
        g = g + xlim(0,xLim)

    ## Do we want cutoffs?
    ## No. We're done:
    if(is.null(cutoffDF))
        return(g)

    ## Yes.  Plot cutoffs:
    idx = cutoffDF$cellTypePair %in% useThesePairs
    g = g +
        geom_vline(data=cutoffDF[idx,],aes(xintercept=cutoff)) +
        facet_wrap(~cellTypePair)

    return(g)
}


## ####################################################
#' This subsets edges by our chosen critera
#'
#' @param annEdges - a data frame with columns nodeA, nodeB, length
#'     and cellTypePair as produced by edgeLengthsAndCellTypePairs.
#' @param cutoffSpec - This can be either a numeric value which will
#'     be applied across all edges as an upper limit or a data frame
#'     with columns cellTypePair and cutoff as produced by any of the
#'     edgeCutoffsBy functions
#' @return This returns a subset of the annotated edges
#' @export
#' @examples
#' getExample = make.getExample()
#' centroids = getExample('centroids')
#' clusters = getExample('clusters')
#' delaunayNeighbours = getExample('delaunayNeighbours') 
#' annEdges =
#'     edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
#' tolerance = 5
#' nbins = 15
#' cutoffDFWater = edgeCutoffsByWatershed(annEdges,
#'                                       tolerance=tolerance,
#'                                       nbins=nbins)
#' culledEdges = cullEdges(annEdges,cutoffDFWater)
cullEdges = function(annEdges,cutoffSpec)
{
    if(isa(cutoffSpec,'numeric'))
    {
        idx = annEdges$length <= cutoffSpec
        annEdges = annEdges[idx,]

        return(annEdges)
    }

    ## Otherwise cutoffSpec is a data frame:
    for(i in seq_len(nrow(cutoffSpec)))
    {
        idx = annEdges$cellTypePair == cutoffSpec$cellTypePair[i] &
            annEdges$length <= cutoffSpec$cutoff[i]

        a = annEdges[idx,]

        if(i == 1)
            culledEdges = a
        else
            culledEdges = rbind(culledEdges,a)
    }

    return(culledEdges)
}

## ####################################################
## This functions focus on the analysis of spatial
## neighbourhoods.
## ####################################################

## ####################################################
#' This function computes a matrix where neighbourhoods are rows and
#' cell types are columns. The values in the matrix indicate the
#' number of cells of a given type within a neighbourhood.
#'
#' @param  spatialGraph - a spatial graph in neighbour list format.
#' @param cellTypes - named vector of cell types where names are each cell and
#' cell types are a factor
#' @return a matrix of neighbourhoods by cell types
#' @export
#' @examples
#' getExample = make.getExample()
#' clusters = getExample('clusters')
#' delaunayNeighbours = getExample('delaunayNeighbours')
#' NBHDByCTMatrix = computeNBHDByCTMatrix(delaunayNeighbours,clusters)
computeNBHDByCTMatrix = function(spatialGraph, cellTypes){
  
  spatialGraphBA = spatialGraph[,c(2,1)]
  names(spatialGraphBA) = c("nodeA","nodeB")
  spatialGraph = rbind(spatialGraph,spatialGraphBA)
  spatialGraph[,2] = cellTypes[spatialGraph[,2]]
  NBHDByCTMatrix = table(spatialGraph[,1],spatialGraph[,2])
  NBHDByCTMatrix  = as.data.frame.matrix(NBHDByCTMatrix)
  NBHDByCTMatrix  =   NBHDByCTMatrix[names(cellTypes),]
  
  return(NBHDByCTMatrix)
}

## ####################################################
#' This function creates a seurat object using a neighbourhood by cell type 
#' matrix
#' 
#' @param dataMatrix - a matrix of neighbourhoods by cell types or its 
#' transpose.
#' @param resolution - resolution for clustering (default 0.1).
#' @param npcs - number of pcs used for PCA, defaults to 10.
#' @param n.neighbors - number of neighbors used by UMAP, defaults to 30.
#' @param transpose - defaults to FALSE.
#' @param verbose - defaults to TRUE, used to limit trace if FALSE
#' @param returnType - Will return a SingleCellExperiment if this is either
#' of SCE, SingleCellExperiment or their lower-case equivalents.  Otherwise,
#' returns a Seurat object
#' @return a seurat object based on a neighbourhood by cell type matrix or its 
#' transpose, containing clusters and UMAP. This can also be a
#' SingleCellExperiment depending on the parameter returnType.
#' @import SeuratObject
#' @export
#' @examples
#' NBHDByCTMatrix = make.getExample()('NBHDByCTMatrix',toy=TRUE)
#' NBHDByCTSeurat = computeNBHDVsCTObject(NBHDByCTMatrix)
#' NBHDByCTSingleCell_sce = computeNBHDVsCTObject(NBHDByCTMatrix,returnType='SCE')
computeNBHDVsCTObject= function(dataMatrix, resolution = 0.1, 
                                npcs = 10, n.neighbors = 30L, 
                                transpose = FALSE,
                                verbose=TRUE,
                                returnType='Seurat'){
    dataMatrix = t(dataMatrix)
    NBHDSeurat = CreateSeuratObject(dataMatrix)
    NBHDSeurat[['RNA']]$data = NBHDSeurat[['RNA']]$counts
    NBHDSeurat = ScaleData(NBHDSeurat,verbose=verbose)
    NBHDSeurat = RunPCA(NBHDSeurat, assay = "RNA", 
                        features = rownames(NBHDSeurat), 
                        npcs = npcs,
                        verbose=verbose)
    if (transpose){
      NBHDSeurat = RunUMAP(NBHDSeurat,assay='RNA',
                           dims = seq_len(npcs), n.neighbors = n.neighbors,
                           verbose=verbose)
  
    } else{
      NBHDSeurat = RunUMAP(NBHDSeurat,assay='RNA',
                           features=rownames(NBHDSeurat), 
                           n.neighbors = n.neighbors,
                           verbose=verbose)
    }
    if (transpose){
        NBHDSeurat = FindNeighbors(NBHDSeurat, dims = seq_len(npcs),
                                   verbose=verbose)
    } else{
        NBHDSeurat = FindNeighbors(NBHDSeurat, 
                                   features=rownames(NBHDSeurat),
                                   verbose=verbose)
    }
    NBHDSeurat = FindClusters(NBHDSeurat,
                              resolution=resolution)

    ## Rename seurat_clusters to neighbourhood_clusters
    idx = names(NBHDSeurat@meta.data) == 'seurat_clusters'
    names(NBHDSeurat@meta.data)[idx] = 'neighbourhood_clusters'
    
    return(returnAs(NBHDSeurat,returnType))
}

## ####################################################
#' This function adds a force directed graph embedding to a seurat object
#' 
#' @param seuratObj - a seurat object of SingleCellExperiment to be
#' turned into a Seurat object
#' @param graph - which graph to extract.  Defaults to
#' paste0(f@active.assay,'_snn')
#' @param returnType - Will return a SingleCellExperiment if this is either
#' of SCE, SingleCellExperiment or their lower-case equivalents.  Otherwise,
#' returns a Seurat object
#' @return a seurat object with a "graph" dimensionality reduction. Can also
#' be a SingleCellExperiment depending on parameter returnType.
#' @importFrom igraph graph_from_adjacency_matrix layout_with_fr
#' @importFrom igraph V E V<- E<-
#' @export
#' @examples
#' NBHDByCTSeurat = make.getExample()('NBHDByCTSeurat',toy=TRUE)
#' objWithEmbedding = computeGraphEmbedding(NBHDByCTSeurat)
computeGraphEmbedding = function(seuratObj, graph=defaultGraph(seuratObj),
                                 returnType='Seurat'){
    seuratObj = acceptor(seuratObj)
    graph = seuratObj@graphs[[graph]]
    igraphGraph = igraph::graph_from_adjacency_matrix(graph)
    graphLayout = igraph::layout_with_fr(igraphGraph)
    colnames(graphLayout) = c("graph_1","graph_2")
    rownames(graphLayout) = colnames(seuratObj)
    graphDimReduc = CreateDimReducObject(embeddings = graphLayout,   
                                         key = "graph",   assay = "RNA")
    seuratObj[["graph"]] = graphDimReduc
    return(returnAs(seuratObj,returnType))
}


## ####################################################
#' For each cell type, this function looks at the neighbourhoods
#' around cells of that type and discovers the fractions (or numbers if 
#' normalise = F) of those cells of each type.
#'
#' @param nbhdByCellType - A matrix whose rows are neighbourhoods
#' each denoted by the cell at their center, whose columns are
#' cell types, and whose entries are counts.
#' @param cellTypes - named vector of cell types where names are each cell and
#' cell types are a factor
#' @param normalise - boolean, defaults to TRUE
#' @return A square matrix whose rownames and colnames are the
#' seurat_clusters as character strings.  Each row corresponds
#' to neighbourhoods around all cells of that type and the entries
#' give the fractions of those neighbourhoods occupied by cells
#' of each type.
#' @export
#' @examples
#' getExample = make.getExample()
#' NBHDByCTMatrix = getExample('NBHDByCTMatrix')
#' clusters = getExample('clusters')
#' cellTypesPerCellType = computeCellTypesPerCellTypeMatrix(NBHDByCTMatrix,clusters)
computeCellTypesPerCellTypeMatrix = function(nbhdByCellType,cellTypes, 
                                             normalise = TRUE)
{
  MM = aggregate(nbhdByCellType, list(cellTypes), sum)
  rownames(MM) = MM$Group.1
  MM = MM[,seq(from=2,to=ncol(MM))]
  if (normalise){
    MM = MM/rowSums(MM)
  }
  MM = as.matrix(MM)
  return(MM)
}


## ####################################################
#' This function converts a matrix as found by
#' cellTypesPerCellTypeMatrix into a directed igraph
#' whose vertices correspond to seurat_clusters and whose
#' edge correspond to occupancy fraction.
#'
#' @param M - a matrix as found by cellTypesPerCellTypeMatrix. Note,
#' however, that this matrix may need to be reduced to a square matrix
#' as the matrix produced from a subset object may be missing certain
#' cell types as rows.
#' @param colours - a named vector of colours used to colour the
#' vertices of the graph.  The names are the seurat_clusters
#' as character strings.
#' @param selfEdges - a logical which determines whether to include
#' self edges.  Defaults to FALSE
#' @param minWeight - Allows one to exclude edges of low weight.
#' Defaults to 0, thus including all edges.
#' @param edgeWeighting - a parameter used to thicken the edges
#' in the display.  Defaults to 20.
#' @param edgeCurved - a parameter to set curvature of the edges.
#' Defaults to 0.2
#' @param arrowSize - a parameter to set arrow size. Defaults to 4.
#' @param arrowWidth - a parameter to set arrow width. Defaults to 4.
#' @param plotGraph - a logical which determines whether to
#' plot the graph.  Defaults to TRUE.
#' @return This returns a directed igraph whose vertices are
#' the cell types and whose arrows indicate "ownership" of
#' cells of the target type by neighbourhoods of cells of the
#' source type.  Layout is done witht the FR algorithm and
#' coordinates are found in the coords attribute of G.  If colours
#' were supplied these are found in color attribute of V(G).  Edge
#' weights and widths are found in the weight and width attributes
#' of E(G).
#' @export
#' @examples
#' getExample = make.getExample()
#' cellTypesPerCellTypeMatrix = getExample('cellTypesPerCellTypeMatrix')
#' colours = getExample('colours')
#' G = cellTypesPerCellTypeGraphFromCellMatrix(cellTypesPerCellTypeMatrix, 
#'                                    minWeight = 0.05, colours = colours)
cellTypesPerCellTypeGraphFromCellMatrix = function(M,
                                               colours=NULL,
                                               selfEdges=FALSE,
                                               minWeight=0,
                                               edgeWeighting=20,
                                               edgeCurved=0.2,
                                               arrowSize=4,
                                               arrowWidth=4,
                                               plotGraph=TRUE)
{
    idx = M >= minWeight
    M[!idx] = 0
    
    G = graph_from_adjacency_matrix(M,
                                    mode='directed',
                                    weighted=TRUE,
                                    diag=selfEdges)

    if(! is.null(colours))    
        V(G)$color = colours[names(V(G))]
    G$coords = layout_with_fr(G)
    E(G)$width = edgeWeighting * E(G)$weight

    if(plotGraph)
    {
        if(! is.null(colours))
            plot(G,
                 layout=G$coords,
                 vertex.color=V(G)$color,
                 edge.width=E(G)$width,
                 edge.curved = edgeCurved,
                 arrow.size = arrowSize,
                 arrow.width = arrowWidth)
        else
            plot(G,
                 layout=G$coords,
                 edge.width=E(G)$width,
                 edge.curved = edgeCurved,
                 arrow.size = arrowSize,
                 arrow.width = arrowWidth)
    }
    return(G)    
}

## ####################################################
#' This function takes a neighbourhood-by-cell type
#' matrix and produces a directed igraph showing the
#' fractions of cells of each type in the neighbourhoods
#' around cells of each type.
#'
#' @param nbhdByCellType - A matrix whose rows are neighbourhoods
#' each denoted by the cell at their center, whose columns are
#' cell types, and whose entries are counts.
#' @param clusters - a named vector whose names are the cells
#' and whose entries are their seurat_clusters.
#' @param colours - a named vector of colours used to colour the
#' vertices of the graph.  The names are the seurat_clusters
#' as character strings.
#' @param selfEdges - a logical which determines whether to include
#' self edges.  Defaults to FALSE
#' @param minWeight - Allows one to exclude edges of low weight.
#' Defaults to 0, thus including all edges.
#' @param edgeWeighting - a parameter used to thicken the edges
#' in the display.  Defaults to 20.
#' @param edgeCurved - a parameter to set curvature of the edges.
#' Defaults to 0.2
#' @param arrowSize - a parameter to set arrow size. Defaults to 4.
#' @param arrowWidth - a parameter to set arrow width. Defaults to 4.
#' @param plotGraph - a logical which determines whether to
#' plot the graph.  Defaults to TRUE.
#' @return This returns a directed igraph whose vertices are
#' the cell types and whose arrows indicate "ownership" of
#' cells of the target type by neighbourhoods of cells of the
#' source type.  Layout is done witht the FR algorithm and
#' coordinates are found in the coords attribute of G.  If colours
#' were supplied these are found in the color attribute of V(G).
#' Edge weights and widths are found in the weight and width
#' attributes of E(G).
#' @export
cellTypesPerCellTypeGraphFromNbhdMatrix = function(nbhdByCellType,
                                                   clusters,
                                                   colours=NULL,
                                                   selfEdges=FALSE,
                                                   minWeight=0,
                                                   edgeWeighting=20,
                                                   edgeCurved=0.2,
                                                   arrowSize=4,
                                                   arrowWidth=4,
                                                   plotGraph=TRUE)
{
    M = computeCellTypesPerCellTypeMatrix(nbhdByCellType,clusters)
    G = cellTypesPerCellTypeGraphFromCellMatrix(M,
                                            colours=colours,
                                            selfEdges=selfEdges,
                                            minWeight=minWeight,
                                            edgeWeighting=edgeWeighting,
                                            plotGraph=plotGraph,
                                            edgeCurved = edgeCurved,
                                            arrowSize = arrowSize,
                                            arrowWidth = arrowWidth)

    
    return(G)
}


## ####################################################
#' This function calculates P values for whether cell types are more frequently 
#' neighbours than expected by chance. By default it calculates P values 
#' analytically using a hypergeometric test on the edges, where the arguments 
#' to the  R phyper function are as follows:
#'  q = number of edges between cell type A and B 
#'  m = number of edges between cell type B and any other cell type
#'  n = the number of edges between any cell type apart from cell type B
#'  k =  number of edges between cell type B and any other cell type
#' The purist may object to the use of the hypergeometric test here.
#' We may think of "edges out of a cell of type A" as being the random draw 
#' balls (here, edges) from the urn and "edges out of cells of type B" as being 
#' success. However, all edges out of a given cell of type A are in this 
#' "random draw". Clearly the edges in this draw are not independent. However, 
#' empirically we find that p-values computed using this method correspond very 
#' closely to those computed using permutation while the computation time is 
#' orders of magnitude faster.
#' 
#' For legacy purposes, and for user flexibility, it allows for the 
#' calculation of P values by comparison to randomised graphs. 
#' It offers two distinct randomisations.
#' One is by permuting the cell types on the neighbour (e.g., delaunay)
#' graph.  The other is by comparison to randomised  neighbour graphs
#' where edges are randomised but the degree of each node is  
#' preserved.
#'
#' @param spatialGraph - a spatial graph in neighbour list format.
#' @param cellTypes - named vector of cell types where names are each cell and
#' cell types are a factor.
#' @param method - method for computing p-values. Defaults to "analytical", in 
#' which case an edge-based hypergeometric test is performed. If "permutation" 
#' is selected p-values are calculated by comparison to randomised graphs (note
#' this is slower than the analytical approach).   
#' @param nSim - the number of randomised graphs to create for pvalue 
#' calculation, if the method is set to permutation.
#' @param maxTries - the maximum number of tries to remove self edges during 
#' graph randomisation. If self edges are remeining this will be reported.
#' @param randomiseBy - This takes either the value 'cells' (the default)
#' or 'graph'.  In the former case randomisation is carried out by permuting
#' the cell types on the existing graph.  In the latter case, the graph is
#' permuted using the function randomiseGraph() which is a heuristic
#' algorithm to preserve the distribution of vertex degrees.
#' @param verbose - whether to print trace.  Defaults to TRUE
#' @return A square matrix containing upper tail p values describing
#' whether two  cell types are more frequently found together than
#' expected by chance.
#' @importFrom stats phyper
#' @importFrom abind abind
#' @export
#' @examples
#' getExample = make.getExample()
#' delaunayNeighbours = getExample('delaunayNeighbours')
#' clusters = getExample('clusters')
#' cellTypesPerCellTypePValues = computeNeighbourEnrichment(delaunayNeighbours, 
#'                                         clusters, verbose = FALSE)
computeNeighbourEnrichment = function(spatialGraph, cellTypes, 
                                      method = "analytical", 
                                      nSim = 1000,
                                      maxTries = 1000,
                                      randomiseBy='cells',
                                      verbose=TRUE)
{
  stopifnot(method %in% c('analytical','permutation')) 
  stopifnot(randomiseBy %in% c('cells','graph'))
  
  results = list()
  spatialGraphOrig = spatialGraph
  NBHDByCTmatrix = computeNBHDByCTMatrix(spatialGraphOrig,cellTypes) 
  cellTypeMatrix = computeCellTypesPerCellTypeMatrix(NBHDByCTmatrix, cellTypes) 
  
  ##compute analytical results
  if (method == "analytical"){
    cellTypeMatrixNumbers = computeCellTypesPerCellTypeMatrix(NBHDByCTmatrix, cellTypes, FALSE) 
    CTEdges = rowSums(cellTypeMatrixNumbers)
    cellTypesNames = levels(cellTypes)
    results = matrix(, nrow = length(cellTypesNames), length(cellTypesNames))
    rownames(results) = cellTypesNames
    colnames(results) = cellTypesNames
    for (CT1 in cellTypesNames){
      for (CT2 in cellTypesNames){
        pval = phyper(q =  cellTypeMatrixNumbers[CT1,CT2], m = CTEdges[CT2] , n = sum(CTEdges) - CTEdges[CT2] , k = CTEdges[CT1] , lower.tail = F)
        results[CT1,CT2] = pval
      }    
    }
    return(results)
  }
  
  ## Randomise by graph:
  if(randomiseBy == 'graph')
  {
    for (i in seq_len(nSim))
    { 
      spatialGraph = spatialGraphOrig
      simGraph = randomiseGraph(spatialGraph, maxTries = maxTries)
      NBHDByCTmatrix = computeNBHDByCTMatrix(simGraph,cellTypes)
      results[[i]] = computeCellTypesPerCellTypeMatrix(NBHDByCTmatrix,cellTypes)
      if (i %% 10 == 0 & verbose)
      {
        writeLines(as.character(i))
      }
    }
  }
  
  ## Randomise by cells:
  if(randomiseBy == 'cells')
  {
    for(i in seq_len(nSim))
    {
      permCellTypes = sample(cellTypes)
      ## This keeps the unpermuted graph, but the count of cell types
      ## in each neighbourhood is permuted cell types
      NBHDByCTmatrix = computeNBHDByCTMatrix(spatialGraph,permCellTypes)
      
      ## This uses actual cell types to classify the individual cells at
      ## the center of each neighourhood and then counts up the permuted
      ## cell types found above in each neighbourhood
      results[[i]] = computeCellTypesPerCellTypeMatrix(NBHDByCTmatrix,cellTypes)
      if (i %% 10 == 0 & verbose)
      {
        writeLines(as.character(i))
      }
    }
  }
  
  results = lapply(results, function(x, y) y > x, y = cellTypeMatrix)
  results = abind(results, along = 3L)
  results = rowSums(results, dims = 2)
  results = abs((results - nSim)/nSim) 
  results = pmax(results,(1/nSim))
  return(results)
}


## ####################################################
#' This function takes a Seurat object and a list of
#' neighbourhoods and creates a Seurat object where the
#' columns are the neighbourhoods, the rows are are the
#' genes and the values are gene expression totals for
#' the cells in each neighbourhood
#'
#' @param f - a Seurat object with layer counts or a SingleCellExperiment
#' to be turned into a Seurat object
#' @param neighbourhoods - Neighbourhoods as given by a
#' collapsed expanded edge graph, as produced by
#' collapseNeighbourhoods. In particular, each cell should
#' appear as nodeA.
#' @param self - include cell in its neighbourhood, defaults to FALSE
#' @param verbose - used to control trace, defaults to TRUE
#' @param returnType - Will return a SingleCellExperiment if this is either
#' of SCE, SingleCellExperiment or their lower-case equivalents.  Otherwise,
#' returns a Seurat object or SingleCellExperiment, depending on the
#' parameter returnType.
#' @return a Seurat object giving total gene expression
#' in each neighbourhood or SingleCellExperiment
#' @export
#' @examples
#' getExample = make.getExample()
#' smallXenium = getExample('smallXenium',toy=TRUE)
#' extendedNeighbours = getExample('extendedNeighbours',toy=TRUE)
#' agg = aggregateGeneExpression(smallXenium,extendedNeighbours,verbose=FALSE)
aggregateGeneExpression = function(f, neighbourhoods, self = FALSE, verbose=TRUE,
                                         returnType='Seurat')
{
    f = acceptor(f)
    
    cells = colnames(f)
    nbhds = unique(c(neighbourhoods$nodeA,
                     neighbourhoods$nodeB))

    ## Sanity check:
    stopifnot(identical(cells[order(cells)],
                        nbhds[order(nbhds)]))

    genes = rownames(f)
    counts = FetchData(f,genes,layer='counts')
    counts = t(counts)
    counts = data.matrix(counts)

    nbhdList =nbhdsAsEdgesToNbhdsAsList(cells,
                                        neighbourhoods, self = self)
    
    C = aggregateFeatureMatrix(counts, nbhdList, rowSums)

    nbhdObj = CreateSeuratObject(counts=C)
    nbhdObj = NormalizeData(nbhdObj,verbose=verbose)
    nbhdObj = ScaleData(nbhdObj,verbose=verbose)
    nbhdObj = FindVariableFeatures(nbhdObj,verbose=verbose)
    nbhdObj = RunPCA(nbhdObj,verbose=verbose)
    nbhdObj = RunUMAP(nbhdObj,dims=seq_len(20),verbose=verbose)
    nbhdObj = FindNeighbors(nbhdObj,verbose=verbose)
    nbhdObj = FindClusters(nbhdObj,verbose=verbose)

    ## Rename the clusters:
    idx = names(nbhdObj@meta.data) == 'seurat_clusters'
    names(nbhdObj@meta.data)[idx] = 'aggregation_clusters'
    
    return(returnAs(nbhdObj,returnType))
}

## ####################################################
#' This function takes a matrix where rows are features and columns are cells,
#' and a neighbourhood list, and creates an matrix where columns are the 
#' neighbourhoods, the rows are are the features and the values are aggregated 
#' expression values for cells in each neighbourhood.
#'
#' @param M - a matrix where column names are cells and row names are 
#' features.
#' @param nbhdList - a named list with memberships of the neighbourhoods
#' of cells
#' @param aggregateFunction - a function to aggregate expression (e.g. rowSums,
#' rowMeans)
#' @return a matrix giving aggregated gene expression for a cell's neighbourhood.
#' @export
aggregateFeatureMatrix = function(M, nbhdList, aggregateFunction)
{
    cells = colnames(M)
    res = lapply(cells, function(x, M, nbhdList)
                        aggregateFunction(M[,nbhdList[[x]], drop = FALSE]), 
                 M = M, nbhdList = nbhdList)
    
    aggrM = do.call(cbind, res)
    colnames(aggrM ) = cells

    return(aggrM)
}

## ####################################################
#' This function takes a matrix where rows are features and columns are cells,
#' and a neighbourhood list, and computes Moran's I. 
#'
#' @param M - a matrix where column names are cells and row names are features.
#' @param nbhdList - a named list with memberships of the neighbourhoods
#' of cells
#' @return a matrix giving aggregated gene expression for a cell's neighbourhood.
#' @export
computeMoransI = function(M,nbhdList){
  aggrM = aggregateFeatureMatrix(M,nbhdList, rowMeans)
  means = rowMeans(M)
  zi = M - means
  zj = aggrM - means
  moransI = rowSums(zi*zj)/rowSums((zi^2))
  return(moransI)
}

## ####################################################
#' This function takes a matrix where rows are features and columns are cells,
#' and a neighbourhood list, and computes Moran's I. 
#'
#' @param obj - a Seurat object
#' @param spatialGraph - a data frame of neighbouring
#' cell pairs. 
#' @param assay - assay to pull data from, defaults to RNA.
#' @param layer - layer to pull data from, defaults to data.
#' @param nSim - number of simulations to perform for p value calculation. 
#' Defaults to 100.
#' @param verbose - whether to print trace, defaults to TRUE
#' @import SingleCellExperiment
#' @import SpatialExperiment
#' @importFrom S4Vectors SelfHits
#' @return a dataframe containing Moran's I and p values for each feature.
#' @export
#' @examples
#' getExample = make.getExample()
#' smallXenium = getExample('smallXenium',toy=TRUE)
#' delaunayNeighbours = getExample('delaunayNeighbours',toy=TRUE)
#' moransI = runMoransI(smallXenium, delaunayNeighbours, assay = "SCT", 
#' layer = "data", nSim = 10, verbose = FALSE)

runMoransI = function(obj, spatialGraph, assay = "RNA", layer = "data",
                      nSim = 100, verbose = TRUE){
    obj = acceptor(obj)
    
    spatialGraph = symmetriseNN(spatialGraph)
    
    M = as.matrix(LayerData(obj, assay = assay, layer = layer))
    nbhdList = nbhdsAsEdgesToNbhdsAsList(colnames(M),
                                         spatialGraph)
    
    moransI = computeMoransI(M,nbhdList)
    results = list()
    for (i in seq_len(nSim)){
        permuted = permuteMatrix(M)
        results[[i]] = computeMoransI(permuted,nbhdList)
        if (i %% 10 == 0 & verbose){
            writeLines(as.character(i))
        }
    }
    
    results = do.call(cbind, results)
    simResults = rowSums(moransI > results) 
    pValues = abs((simResults - nSim)/nSim) 
    pValues = pmax(pValues, (1/nSim))
    results = cbind(moransI, pValues)
    results = results[order(moransI, decreasing = TRUE),]
    results = data.frame(results)
    return(results)
}

## ####################################################
## These functions focus of analysing ligand-receptor
## interactions between neighbouring cells is spatial
## data,
## ####################################################

## ####################################################
#' This function retrieves the Nichenetr ligand-
#' receptor network for mouse or human.
#'
#' @param species - either 'human' or 'mouse'
#' @return This returns a data frame whose first two
#' columns are from and to, i.e., ligand and receptor.
#' These are derived from the nichenetr ligand receptor
#' networks.
#' @export
#' @examples
#' lrn = getLigandReceptorNetwork('human')
getLigandReceptorNetwork = function(species)
{
    stopifnot(species %in% c('human','mouse'))

    getExample = make.getExample()
    
    if(species == 'human')
    {
        humanLRN = getExample('humanLRN')
        return(humanLRN)
    }

    if(species == 'mouse')
    {
        mouseLRN = getExample('mouseLRN')
        return(mouseLRN)
    }
}

## ####################################################
#' This functions takes an Seurat object, its species
#' and a ligand receptor network and subsets the ligand
#' receptor network to those pairs that occur in the
#' panel
#'
#' @param obj - a Seurat object or SingleCellExperiment to
#' be converted to a Seurat object
#' @param species - either 'human' or 'mouse'
#' @param lrn - a ligand-receptor network, i.e., a
#' data frame with columns from and to.  By default, it
#' retrieves the nichenetr ligand receptor network
#' @return This returns a data frame with columns ligand and
#' receptor
#' @export
#' @examples
#' smallXenium = make.getExample()('smallXenium')
#' lrPairs = getLigandReceptorPairsInPanel(smallXenium, "mouse")
getLigandReceptorPairsInPanel = function(obj,species,
                                         lrn = getLigandReceptorNetwork(species))
{
  
  obj = acceptor(obj)
  
  ## The panel
  panel = rownames(obj)
  panel = str_replace(panel,"-",".")
  lrn$from = str_replace(lrn$from,"-",".")
  lrn$to = str_replace(lrn$to,"-",".")
  
  pairsFoundDF = lrn[(lrn$from %in% panel) & (lrn$to %in% panel),]
  colnames(pairsFoundDF) = c("ligand","receptor")
  return(pairsFoundDF)
}

## ####################################################
#' This function takes a binarised expression matrix, a set of ligand receptor
#' pairs and a set of edges denoting neighbouring cells and
#' annotates these with the ligand receptor interactions taking
#' place on those edges in each direction.
#'
#' @param M - a binarised expression matrix where rows are genes and columns
#' are cells.
#' @param pairDF - a data frame giving the ligand-receptor pairs
#' @param spatialGraph - a data frame of neighbouring
#' cell pairs.  Note that each row is a directed edge (A,B) so
#' that this data frame should have both the edge (A,B) and the
#' edge (B,A)
#' @return This returns a data frame whose first two columns give
#' the neighbouring cells.  Each of the remaining columns is a logical
#' corresponding to a ligand-receptor pair telling whether the ligand
#' is expressed in the first cell and the receptor is expressed in the
#' second cell.
getInteractionsOnEdges = function(M,pairDF,spatialGraph)
{
  ## Find the interactions on the edges:
  edges = spatialGraph
  
  for(i in seq_len(nrow(pairDF)))
  {
    tag = paste(pairDF$ligand[i],pairDF$receptor[i],sep='-')
    edges[,tag] = (M[pairDF$ligand[i],edges$nodeA] &
                     M[pairDF$receptor[i],edges$nodeB])
  }
  
  return(edges)
}

## ####################################################
#' This function permutes the rows of a matrix.
#'
#' @param M - a binarised expression matrix where rows are genes and columns
#` are cells.
#' @return This returns a matrix in which the values have been permuted within
#' rows.
permuteMatrix = function(M){
  n = ncol(M)
  for (i in seq_len(nrow(M))){
    M[i,] = M[i,sample(n)]
  }
  return(M)
}

## ####################################################
#' This function permutes the columns of a sparse dcG matrix.
#'
#' @param M - a binarised expression matrix in sparse format where rows are cells and columns
#` are genes.
#' @return This returns a matrix in which the values have been permuted within
#' columns.
permuteColumns = function(M){
  nRow = M@Dim[1]
  rowInd = vector(mode="integer", length=length(M@i))
  j = 1
  for (i in diff(M@p)){
    if (i != 0){
      value = sort(sample(0:(nRow-1),i,replace = FALSE))
      rowInd[j:(j+i-1)] = value
      j = j + i
    }
  }
  M@i = rowInd
  return(M)
}


## ####################################################
#' This functions retrieves an expression matrix from a
#' seurat object or SingleCellExperiment and binarises it.
#'
#' @param obj - a Seurat object or SingleCellExperiment to be
#' turned into a Seurat object
#' @param cutoff - a cutoff for binarisation. Defaults to 0.
#' @param layer - layer to fetch data from. Defaults to count.
#' @return A binarised sparse expression matrix where rows are genes and 
#' columns are cells.
getBinarisedMatrix = function(obj, cutoff = 0, layer = 'counts'){
  obj = acceptor(obj)
  M = GetAssayData(obj, layer = "counts")
  M = M > 0
  rownames(M) = str_replace(rownames(M),"-",".")
  M = t(M)
  M = as(M, "dMatrix")
  return(M)
}

## ####################################################
#' This function takes a listing of the neighbouring
#' cells together with the presence or absence of each
#' ligand-receptor pair on each edge and produces a count
#' showing for each cell, how many neighbours it has with
#' that interaction either as source or as target
#'
#' @param edges - A data frame of neighbouring cells
#' together with their interactions as produced by
#' getInteractionsOnEdges()
#' @param sourceOrTarget - a character, either 'source' or
#' 'target' telling which direction of interaction to count
#' @return This returns a data frame with one row for each
#' cell and a column giving the name of that cell and the
#' other columns giving the counts of interactions that it
#' has with its neighbours.
#' @export
countLRInteractionsPerCell = function(edges,sourceOrTarget)
{
    stopifnot(sourceOrTarget %in% c('source','target'))
    
    if(sourceOrTarget == 'source')
        by = factor(edges$nodeA)
    if(sourceOrTarget == 'target')
        by = factor(edges$nodeB)

    edges = edges[,seq(from=3,to=ncol(edges))]
    interactionCountDF = aggregate(edges,by=list(by),FUN=sum) 

    interactionCountDF$Group.1 =
        str_replace(interactionCountDF$Group.1,'Group.','')
    rownames(interactionCountDF) = interactionCountDF$Group.1
    names(interactionCountDF)[1] = 'cell'
     
    return(interactionCountDF)
}

## ####################################################
#' This takes a data frame of interaction counts as found
#' by countLRInteractionsPerCell(), the underlying Seurat object
#' and the neighbourhood Seurat object and annotates the counts
#' with the cell type and the neighbourhood type corresponding
#' to the cells of the interaction counts.
#'
#' @param interactionCounts - as found by countLRInteractionsPerCell()
#' @param obj - a Seurat object, or SingleCellExperiment to be turned
#' into a Seurat object
#' @param nbhdObj - a neighbourhood x cell type Seurat object or a
#' SingleCellExperiment to be turned into a Seurat object
#' @return This returns the interaction counts annotated with the
#' cell type and neighbourhood type of each cell.
#' @export
annotateLRInteractionCounts = function(interactionCounts,obj,nbhdObj)
{
    obj = acceptor(obj)
    nbhdObj = acceptor(nbhdObj)
    
    ## Get nbhd and cell types:
    annotated = data.frame(cell=colnames(obj),
                           cellType=obj$seurat_clusters)
    rownames(annotated) = annotated$cell

    ## Append nbhd type:
    annotated$nbhdType = nbhdObj$seurat_clusters[annotated$cell]

    ## Bung in the interaction counts:
    pairs = names(interactionCounts)[seq(from=2,to=ncol(interactionCounts))]

    annotated[,pairs] = 0
    annotated[rownames(interactionCounts),pairs] =
        interactionCounts[rownames(interactionCounts),pairs]

    return(annotated)
}

#' Given a seurat object, a spatial graph, clusters and
#' species this function identifies ligand-receptor 
#' interactions between neighbouring cells, identifies
#' ligand-receptor interactions within and between clusters
#' and calculates whether these are observed more frequently 
#' than expected by chance. If the "analytical" method is selected, an upper 
#' tail p-value for observing a given number of A-B edges positive for a given 
#' interaction is calculated using a binomial test (R pbinom function) where:
#'  q = number of A-B edges positive for an interaction
#'  size = total number of A-B edges   
#'  prob = pL*pR
#' Where pL is the probability of a cell expressing a specific ligand (number 
#' of cells positive for a ligand/total cells), 
#' and pR is the probability of a cell expressing a specific receptor (number 
#' of cells positive for a receptor/total cells). 
#' If conditional = True  p-values will be calculated given the proportion of 
#' cells that express ligands and receptors in the specific clusters (pL = 
#' number of cells in cluster A positive for a ligand/number of cells in cluster
#' A, pR = number of cells in cluster B positive for a receptor/number of cells
#' in cluster B).
#'   
#' We recommend to use the analytical method, which has a much faster runtime 
#' than the permutation-based method, however for legacy purposes and user 
#' flexibility we retain the permutation-based method.
#' @param obj - a Seurat object
#' @param spatialGraph - a data frame of neighbouring
#' cell pairs. 
#' @param clusters - named vector of clusters where names are each cell and
#' clusters are a factor
#' @param species - either 'human' or 'mouse'
#' @param method - method for computing p-values. Defaults to "analytical". 
#' If "permutation" is selected p-values are calculated by comparison to 
#' randomised graphs (note this is slower than the analytical approach).
#' @param conditional - if method is "analytical" and conditional is true, 
#' p-values will be calculated given the proportion of cells that express 
#' ligands and receptors in the specific clusters. Otherwise global proportions 
#' of ligand and receptor expression are used. Defaults to FALSE.
#' @param minEdgesPos - the minimum edges that need to be positive for a 
#' ligand-receptor interaction between two clusters for a p-value to be 
#' calculated. Only taken into consideration when the analytical method is 
#' selected.   
#' @param nSim - number of simulations to perform for pvalue calculation.
#' @param lrn - a ligand-receptor network, i.e., a
#' data frame with columns from and to.  By default, it
#' retrieves the nichenetr ligand receptor network
#' @param verbose - whether to print trace, defaults to TRUE
#' @return A list containing:
#' interactionsOnEdges - a sparse matrix where the rownames give pairs of 
#' neighbouring cells and column names give ligand-receptor pairs. 
#' Entries are TRUE if the ligand is expressed in the first cell and the receptor is expressed in 
#' the second cell and FALSE if not.
#' interactionsOnEdgesMeta - a dataframe where the first two columns are the
#' cells that comprise the edges in interactionsOnEdges, and the next two
#' columns are their clusters.
#' totalInteractionsByCluster - a dataframe where the rownames are 
#' sender-receiver cluster pairs and column names are ligand receptor pairs. 
#' Entries are total numbers of edges on which particular ligand receptor 
#' interactions are present.
#' meanInteractionsByCluster - a dataframe where the rownames are 
#' sender-receiver cluster pairs and column names are ligand receptor pairs. 
#' Entries are total numbers of edges on which particular ligand receptor i
#' nteractions are present (for that cluster pair) divided by the total number 
#' of edges between those clusters.
#' simResults - a dataframe where the rownames are sender-receiver cluster 
#' pairs and column names are ligand receptor pairs. 
#' Values give the number of simulations for which observed values are greater 
#' than simulated values. Only returned if method = "permutation".
#' pValues - a dataframe where the rownames are sender-receiver cluster pairs 
#' and column names are ligand receptor pairs. Entries are uppertail p-values 
#' describing whether a particular ligand receptor interaction is observed more 
#' frequently between 2 clusters than expected.
#' totalEdges - a vector of total edges between cluster pairs.
#' @importFrom stringr str_split_1
#' @export
#' @examples
#' getExample = make.getExample()
#' smallXenium = getExample('smallXenium')
#' delaunayNeighbours = getExample('delaunayNeighbours')
#' clusters = getExample('clusters')
#' performLigandReceptorAnalysis(smallXenium, delaunayNeighbours, 
#'                                       "mouse", clusters)

performLigandReceptorAnalysis = function(obj, spatialGraph, species, clusters,
                                      method = "analytical",
                                      conditional = FALSE,
                                      minEdgesPos = 10,
                                      nSim = 1000, 
                                      lrn = getLigandReceptorNetwork(species),
                                      verbose=TRUE){
  
  stopifnot(method %in% c('analytical','permutation')) 
  if (method == "analytical"){
    results = performLigandReceptorAnalysisAnalytical(obj, spatialGraph, 
                                            species, clusters, 
                                            conditional = conditional,
                                            lrn = lrn,
                                            minEdgesPos = minEdgesPos)
  }
  if (method == "permutation"){
    results = performLigandReceptorAnalysisPermutation (obj, spatialGraph, 
                                              species, clusters, nSim = 1000, 
                                              lrn = lrn,  minEdgesPos = minEdgesPos,
                                              verbose=verbose)
  }
  return(results)
}


## ####################################################
#' Given a seurat object, a spatial graph, clusters and
#' species this function identifies ligand-receptor 
#' interactions between neighbouring cells, identifies
#' ligand-receptor interactions within and between clusters
#' and calculates whether these are observed more frequently 
#' than expected by chance using a permutation-based approach. 
#' @param obj - a Seurat object
#' @param spatialGraph - a data frame of neighbouring
#' cell pairs. 
#' @param clusters - named vector of clusters where names are each cell and
#' clusters are a factor
#' @param species - either 'human' or 'mouse'
#' @param nSim - number of simulations to perform for p value calculation.
#' @param lrn - a ligand-receptor network, i.e., a
#' data frame with columns from and to.  By default, it
#' retrieves the nichenetr ligand receptor network
#' @param minEdgesPos - the minimum edges that need to be positive for a 
#' ligand-receptor interaction between two clusters for a p-value to be 
#' calculated. Only taken into consideration when the analytical method is 
#' selected.   
#' @param verbose - whether to print trace, defaults to TRUE
#' @return A list containing:
#' interactionsOnEdges - a sparse matrix where the rownames give pairs of 
#' neighbouring cells and column names give ligand-receptor pairs. 
#' Entries are TRUE if the ligand is expressed in the first cell and the receptor is expressed in 
#' the second cell and FALSE if not.
#' interactionsOnEdgesMeta - a dataframe where the first two columns are the
#' cells that comprise the edges in interactionsOnEdges, and the next two
#' columns are their clusters.
#' totalInteractionsByCluster - a dataframe where the rownames are 
#' sender-receiver cluster pairs and column names are ligand receptor pairs. 
#' Entries are total numbers of edges on which particular ligand receptor 
#' interactions are present.
#' meanInteractionsByCluster - a dataframe where the rownames are 
#' sender-receiver cluster pairs and column names are ligand receptor pairs. 
#' Entries are total numbers of edges on which particular ligand receptor i
#' nteractions are present (for that cluster pair) divided by the total number 
#' of edges between those clusters.
#' simResults - a dataframe where the rownames are sender-receiver cluster 
#' pairs and column names are ligand receptor pairs. 
#' Values give the number of simulations for which observed values are greater 
#' than simulated values. 
#' pValues - a dataframe where the rownames are sender-receiver cluster pairs 
#' and column names are ligand receptor pairs. Entries are uppertail p-values 
#' describing whether a particular ligand receptor interaction is observed more 
#' frequently between 2 clusters than expected.
#' totalEdges - a vector of total edges between cluster pairs.
#' @examples
#' getExample = make.getExample()
#' smallXenium = getExample('smallXenium')
#' delaunayNeighbours = getExample('delaunayNeighbours')
#' clusters = getExample('clusters')
#' performLigandReceptorAnalysis(smallXenium, delaunayNeighbours, 
#'                                       "mouse", clusters,  minEdgesPos = 10, nSim = 10,
#'                                        verbose=FALSE)

performLigandReceptorAnalysisPermutation = function(obj, spatialGraph, species, 
                                              clusters,
                                              nSim = 1000, 
                                              lrn = getLigandReceptorNetwork(species),
                                              minEdgesPos = 10,
                                              verbose=TRUE){
  
  #symmetrise spatial graph
  spatialGraphBA = spatialGraph[,c(2,1)]
  names(spatialGraphBA) = c("nodeA","nodeB")
  spatialGraph = rbind(spatialGraph,spatialGraphBA)
  spatialGraph = unique(spatialGraph)
  
  #get ligand receptor pairs
  lrPairs = getLigandReceptorPairsInPanel(obj, species, lrn)
  
  #get binarised expression matrix for ligand receptor pairs
  M = getBinarisedMatrix(obj)
  M = M[,colnames(M) %in% c(lrPairs$ligand,lrPairs$receptor)]
  
  #get ligand receptor interactions between cells
  interactionsOnEdges = M[spatialGraph$nodeA,lrPairs$ligand] &
    M[spatialGraph$nodeB,lrPairs$receptor]
  interactionsOnEdges = t(interactionsOnEdges)
  
  ligRecNames = paste(lrPairs$ligand,lrPairs$receptor,sep='_')
  cellNames = paste(spatialGraph$nodeA, spatialGraph$nodeB, sep = "_")
  
  pair = 
    paste0(clusters[spatialGraph$nodeA],  "-", clusters[spatialGraph$nodeB])
  interactionsOnEdgeslgT = as(interactionsOnEdges, "TsparseMatrix")
  interactionsOnEdgesAnno = cbind(pair[(interactionsOnEdgeslgT@j+1)],ligRecNames[(interactionsOnEdgeslgT@i+1)])
  totalInteractionsByCluster = table(interactionsOnEdgesAnno[,1],interactionsOnEdgesAnno[,2])
  totalInteractionsByCluster = as.data.frame.matrix(totalInteractionsByCluster)
  
  #get total edges per cluster pair
  totalEdges = table(pair)
  
  meanInteractionsByCluster = totalInteractionsByCluster/
    totalEdges[rownames(totalInteractionsByCluster)]
  
  interactionsOnEdges = t(interactionsOnEdges)
  colnames(interactionsOnEdges) = ligRecNames
  rownames(interactionsOnEdges) = cellNames
  
  #perform simulations
  results = list()
  clusterPairs = rownames(totalInteractionsByCluster)
  ligRecPairs = colnames(totalInteractionsByCluster)
  
  for (i in seq_len(nSim)){
    permuted = permuteColumns(M)
    sim = permuted[spatialGraph$nodeA,lrPairs$ligand] &
      permuted[spatialGraph$nodeB,lrPairs$receptor]
    sim = t(sim)
    sim = as(sim, "TsparseMatrix")
    sim = cbind(pair[(sim@j+1)],ligRecNames[(sim@i+1)])
    sim = table(sim[,1],sim[,2])
    sim = as.data.frame.matrix(sim)
    
    missingRows = clusterPairs[!(clusterPairs %in% rownames(sim))]
    missingColumns = ligRecPairs[!(ligRecPairs %in% colnames(sim))]
    
    for (row in missingRows){
      sim[row,] = rep(0, ncol(sim))
    }
    for (col in missingColumns){
      sim[,col] = rep(0, nrow(sim))
    }
    
    sim = sim[clusterPairs,ligRecPairs]
    results[[i]] = sim
    
    if (i %% 10 == 0 & verbose){
      writeLines(as.character(i))
    }
  }
  interactionsOnEdgesMetaData = cbind(spatialGraph, clusters[spatialGraph$nodeA], clusters[spatialGraph$nodeB])
  names(interactionsOnEdgesMetaData)[3:4] = c("cellTypeA","cellTypeB")
  if (nSim > 0){
    #calculate summary statistics for simulation results
    results = lapply(results, function(x, y) y > x, y = 
                       totalInteractionsByCluster)
    results = abind(results, along = 3L)
    simResults = rowSums(results, dims = 2)
    rownames(simResults) = rownames(totalInteractionsByCluster)
    pValues = abs((simResults - nSim)/nSim) 
    pValues = pmax(pValues, (1/nSim))
    pValues[totalInteractionsByCluster < minEdgesPos] = NA
    return(list("interactionsOnEdges" = interactionsOnEdges, 
                "interactionsOnEdgesMeta" = interactionsOnEdgesMetaData,
                "totalInteractionsByCluster" = totalInteractionsByCluster,
                "meanInteractionsByCluster" = meanInteractionsByCluster,
                "simResults" = as.data.frame(simResults),
                "pValues" = as.data.frame(pValues),
                "totalEdges" = totalEdges))
  } else {
    return(list("interactionsOnEdges" = interactionsOnEdges,
                "interactionsOnEdgesMeta" = interactionsOnEdgesMetaData,
                "totalInteractionsByCluster" = totalInteractionsByCluster,
                "meanInteractionsByCluster" = meanInteractionsByCluster,
                "totalEdges" = totalEdges))
  }
}

## ####################################################
#' Given a seurat object, a spatial graph, clusters and
#' species this function identifies ligand-receptor 
#' interactions between neighbouring cells, identifies
#' ligand-receptor interactions within and between clusters
#' and calculates whether these are observed more frequently 
#' than expected by chance using an analytical approach.
#'
#' @param obj - a Seurat object
#' @param spatialGraph - a data frame of neighbouring
#' cell pairs. 
#' @param clusters - named vector of clusters where names are each cell and
#' clusters are a factor
#' @param species - either 'human' or 'mouse'
#' @param conditional - if method is "analytical" and conditional is true, 
#' p-values will be calculated given the proportion of cells that express 
#' ligands and receptors in the specific clusters. Otherwise global proportions 
#' of ligand and receptor expression are used. Defaults to FALSE.
#' @param lrn - a ligand-receptor network, i.e., a
#' data frame with columns from and to.  By default, it
#' retrieves the nichenetr ligand receptor network
#' @param minEdgesPos - the minimum edges that need to be positive for a 
#' ligand-receptor interaction between two clusters for a p-value to be 
#' calculated. 
#' @importFrom stats pbinom phyper
#' @return A list containing:
#' interactionsOnEdges - a sparse matrix where the rownames give pairs of 
#' neighbouring cells and column names give ligand-receptor pairs. 
#' Entries are TRUE if the ligand is expressed in the first cell and the receptor is expressed in 
#' the second cell and FALSE if not.
#' interactionsOnEdgesMeta - a dataframe where the first two columns are the
#' cells that comprise the edges in interactionsOnEdges, and the next two
#' columns are their clusters.
#' totalInteractionsByCluster - a dataframe where the rownames are 
#' sender-receiver cluster pairs and column names are ligand receptor pairs. 
#' Entries are total numbers of edges on which particular ligand receptor 
#' interactions are present.
#' meanInteractionsByCluster - a dataframe where the rownames are 
#' sender-receiver cluster pairs and column names are ligand receptor pairs. 
#' Entries are total numbers of edges on which particular ligand receptor 
#' interactions are present (for that cluster pair) divided by the total number 
#' of edges between those clusters.
#' pValues - a dataframe where the rownames are sender-receiver cluster pairs 
#' and column names are ligand receptor pairs. Entries are uppertail p-values 
#' describing whether a particular ligand receptor interaction is observed more 
#' frequently between 2 clusters than expected.
#' totalEdges - a vector of total edges between cluster pairs.
performLigandReceptorAnalysisAnalytical = function(obj, spatialGraph, species, clusters,
                                                    conditional = FALSE,
                                                    lrn = getLigandReceptorNetwork(species),
                                                    minEdgesPos = 10)
  {
  
  #symmetrise spatial graph
  spatialGraph = spatialGraph[,c(1,2)]
  spatialGraphBA = spatialGraph[,c(2,1)]
  names(spatialGraphBA) = c("nodeA","nodeB")
  spatialGraph = rbind(spatialGraph,spatialGraphBA)
  spatialGraph = unique(spatialGraph)
  
  #get ligand receptor pairs
  lrPairs = getLigandReceptorPairsInPanel(obj, species, lrn)
  
  #get binarised expression matrix for ligand receptor pairs
  M = getBinarisedMatrix(obj)
  M = M[,colnames(M) %in% c(lrPairs$ligand,lrPairs$receptor)]
  
  #get ligand receptor interactions between cells
  interactionsOnEdges = M[spatialGraph$nodeA,lrPairs$ligand] &
    M[spatialGraph$nodeB,lrPairs$receptor]
  interactionsOnEdges = t(interactionsOnEdges)
  
  ligRecNames = paste(lrPairs$ligand,lrPairs$receptor,sep='_')
  cellNames = paste(spatialGraph$nodeA, spatialGraph$nodeB, sep = "_")
  
  #get sum of interactions within and between clusters
  pair = 
    paste0(clusters[spatialGraph$nodeA],  "-", clusters[spatialGraph$nodeB])
  interactionsOnEdgeslgT = as(interactionsOnEdges, "TsparseMatrix")
  interactionsOnEdgesAnno = cbind(pair[(interactionsOnEdgeslgT@j+1)],ligRecNames[(interactionsOnEdgeslgT@i+1)])
  totalInteractionsByCluster = table(interactionsOnEdgesAnno[,1],interactionsOnEdgesAnno[,2])
  totalInteractionsByCluster = as.data.frame.matrix(totalInteractionsByCluster)
  
  #get total edges per cluster pair
  totalEdges = table(pair)
  
  meanInteractionsByCluster = totalInteractionsByCluster/
    totalEdges[rownames(totalInteractionsByCluster)]
  
  interactionsOnEdges = t(interactionsOnEdges)
  colnames(interactionsOnEdges) = ligRecNames
  rownames(interactionsOnEdges) = cellNames
  
  
  geneTotals = colSums(M)
  clusterTotals = table(clusters)
  totalCells = length(clusters)
  
  #calculate stats
  results = list()
  clusterPairs = rownames(totalInteractionsByCluster)
  ligRecPairs = colnames(totalInteractionsByCluster)
  
  genes = colnames(M)
  MlgT = as(M, "TsparseMatrix")
  MAnno = cbind(as.character(clusters)[(MlgT@i+1)],genes[(MlgT@j+1)])
  clusterMatrix = table(MAnno[,1],MAnno[,2])
  clusterMatrix = as.data.frame.matrix(clusterMatrix)
  
  pValues = matrix(NA, nrow = nrow(totalInteractionsByCluster), ncol = ncol(totalInteractionsByCluster))
  rownames(pValues) = rownames(totalInteractionsByCluster)
  colnames(pValues) = colnames(totalInteractionsByCluster)
  for (CTPair in rownames(totalInteractionsByCluster)){
    CTs = str_split_1(CTPair, "-")
    for (genePair in colnames(totalInteractionsByCluster)){
      genes = str_split_1(genePair, "_")
      if (conditional){
        pG1 = clusterMatrix[CTs[1],genes[1]]/clusterTotals[CTs[1]]
        pG2 = clusterMatrix[CTs[2],genes[2]]/clusterTotals[CTs[2]]
      } else {
      pG1 = geneTotals[genes[1]]/totalCells
      pG2 = geneTotals[genes[2]]/totalCells
      }
      nEdgePos =   totalInteractionsByCluster[CTPair,genePair] 
      if(nEdgePos >= minEdgesPos){
        pValues[CTPair,genePair] = pbinom(nEdgePos,totalEdges[CTPair],pG1*pG2, lower.tail = F)
      }
    }
  } 
  
  interactionsOnEdgesMetaData = cbind(spatialGraph, clusters[spatialGraph$nodeA], clusters[spatialGraph$nodeB])
  names(interactionsOnEdgesMetaData)[3:4] = c("cellTypeA","cellTypeB")
  return(list("interactionsOnEdges" = interactionsOnEdges, 
              "interactionsOnEdgesMeta" = interactionsOnEdgesMetaData, 
              "totalInteractionsByCluster" = totalInteractionsByCluster,
              "meanInteractionsByCluster" = meanInteractionsByCluster,
              "pValues" = as.data.frame(pValues),
              "totalEdges" = totalEdges))
}



## ####################################################
#' This function takes ligandReceptorResults and plots a heatmap of -log10(pvalues).
#' If the minimum p-value is 0 a pseudocount of 0.001 will be added before log 
#' transformation.
#'
#' @param ligandReceptorResults - as returned by performLigandReceptorAnalysis()
#' @param clusters - named vector of cell types where names are each cell and
#' clusters are a factor
#' @param colours - a named list of colours where names are clusters. If not 
#' specified the default pheatmap colour scheme will be used.
#' @param  pValCutoffClusterPair - a cutoff for showing interactions between two
#' clusters. A cluster pair must have at least one ligand-receptor interaction
#' pvalue <  pValCutoffClusterPair. Defaults to 0.05.
#' @param  pValCutoffLigRec  - a cutoff for showing interactions between a 
#' ligand and receptor. At least one cluster pair must have 
#' pvalue <  pValCutoffLigRec for ligand-receptor pair. Defaults to 0.05.
#' @param  labelClusterPairs - show labels for cluster pairs. Defaults to TRUE.
#' @importFrom Rfast rowMins colMins
#' @import pheatmap
#' @return matrix of -log10(pvalues) that underlies the heatmap.
#' @export
#' @examples
#' getExample = make.getExample()
#' clusters = getExample('clusters')
#' ligandReceptorResults = getExample('ligandReceptorResults')
#' cellTypePerCellTypeLigRecMatrix = 
#' makeSummedLRInteractionHeatmap(ligandReceptorResults, clusters, "mean")
makeLRInteractionHeatmap = function(ligandReceptorResults,
                                  clusters,
                                  colours = c(),
                                  pValCutoffClusterPair = 0.05, 
                                  pValCutoffLigRec = 0.05,
                                  labelClusterPairs = TRUE)
{ 
  pValues = as.matrix(ligandReceptorResults$pValues)
  if (min(pValues, na.rm = T) == 0){
    pValues = pValues + 0.001 
  }
  pValuesMod = pValues
  pValuesMod[is.na(pValuesMod)] = 1
  
  selectedPValues = pValues[rowMins(pValuesMod, value = TRUE) < pValCutoffClusterPair,
                            colMins(pValuesMod, value = TRUE) < pValCutoffLigRec]
  selectedPValuesMod = pValuesMod[rowMins(pValuesMod, value = TRUE) < pValCutoffClusterPair,
                                  colMins(pValuesMod, value = TRUE) < pValCutoffLigRec]

  negLog10PValues = -log10(selectedPValues + 0.001)
  negLog10PValuesMod = -log10(selectedPValuesMod + 0.001)

  rowAnno = str_split_fixed(rownames(selectedPValues), pattern = "-", 2)
  rowAnno = as.data.frame(rowAnno)
  names(rowAnno) = c("sender","receiver")
  rowAnno$sender = factor(rowAnno$sender, levels = levels(clusters))
  rowAnno$receiver = factor(rowAnno$receiver, levels = levels(clusters))
  rownames(rowAnno) = rownames(selectedPValues)
  rowAnno = rowAnno[,c("receiver","sender")]
  p1 = pheatmap(negLog10PValuesMod, annotation_row = rowAnno, show_rownames = labelClusterPairs)
  negLog10PValues = negLog10PValues[p1$tree_row$order,p1$tree_col$order]
  if (length(colours) > 0){
    p2 = pheatmap(negLog10PValues, annotation_row = rowAnno,  annotation_colors = list("sender" = colours, "receiver" = colours),
          show_rownames = labelClusterPairs, cluster_rows = F,cluster_cols = F, na_col = "grey")
    p2$tree_row = p1$tree_row
    p2$tree_col = p1$tree_col
    print(p2)
  } else{
    p2 = pheatmap(negLog10PValues, annotation_row = rowAnno, show_rownames = labelClusterPairs, cluster_rows = F,cluster_cols = F, na_col = "grey")
    p2$tree_row = p1$tree_row
    p2$tree_col = p1$tree_col
    print(p2)
  }
  return(negLog10PValues)
}

## ####################################################
#' This function takes ligandReceptorResults and plots a heatmap of the total 
#' number of ligand receptor interactions between clusters.
#'
#' @param ligandReceptorResults - as returned by performLigandReceptorAnalysis()
#' @param clusters - named vector of cell types where names are each cell and
#' clusters are a factor
#' @param type - "total" or "mean" to plot raw total interactions or mean
#' interactions per edge.
#' @param  logScale - plot heatmap using log scale (defaults to TRUE)
#' @import pheatmap
#' @return matrix of total ligand receptor interactions that underlies t
#' he heatmap.
#' @export
#' @examples
#' getExample = make.getExample()
#' clusters = getExample('clusters')
#' ligandReceptorResults = getExample('ligandReceptorResults')
#' cellTypePerCellTypeLigRecMatrix = 
#' makeSummedLRInteractionHeatmap(ligandReceptorResults, clusters, "mean")
makeSummedLRInteractionHeatmap = function(ligandReceptorResults, clusters, type, logScale = TRUE){ 
  if (type == "total"){
    interactionsByCluster = ligandReceptorResults$totalInteractionsByCluster
  } 
  if (type == "mean"){
    interactionsByCluster = ligandReceptorResults$meanInteractionsByCluster
  } 
  summedInteractionsByCluster = rowSums(interactionsByCluster)
  pair = str_split_fixed(names(summedInteractionsByCluster), pattern = "-", 2)
  summedInteractionsByCluster = as.data.frame(cbind(pair,summedInteractionsByCluster))
  colnames(summedInteractionsByCluster) = c("Sender", "Receiver", "nInteractions")
  summedInteractionsByCluster$nInteractions = as.numeric(summedInteractionsByCluster$nInteractions)
  clusterNames = levels(clusters)
  nClusters = length(clusterNames)
  summedInteractionsByClusterMatrix = matrix(0, ncol = nClusters, nrow = nClusters)
  for (i in seq_len(nClusters)){
    for (j in seq_len(nClusters)){
        value = summedInteractionsByCluster$nInteractions[(summedInteractionsByCluster$Sender == clusterNames[i]) &
                                                          (summedInteractionsByCluster$Receiver == clusterNames[j])]
      if (length(value) > 0){
        summedInteractionsByClusterMatrix[i,j] = value
      } 
    }  
  }
  
  colnames(summedInteractionsByClusterMatrix) = clusterNames
  rownames(summedInteractionsByClusterMatrix) = clusterNames
  if (logScale){
    pheatmap(log(summedInteractionsByClusterMatrix +1))
    return(log(summedInteractionsByClusterMatrix +1))
  } else{
    pheatmap((summedInteractionsByClusterMatrix))
    return(summedInteractionsByClusterMatrix)
  }
}


## ####################################################
#' This is a utility function for converting entries in ligandReceptorResults 
#' to long format. 
#'
#' @param data - item from ligandReceptorResults
#' @param name - name to give column of returned data
#' @importFrom reshape2 melt
#' @return dataframe with item from ligandReceptorResults in long format
formatData = function(data, name){
  data$clusterPair = rownames(data)
  data = melt(data, id.vars = c("clusterPair"), variable.name = "interaction")
  names(data)[3] = name
  return(data)
}


## ####################################################
#' This is a utility function for converting ligandReceptor cluster-level 
#' results to long format and calculates adjusted p-values.
#'
#' @param ligandReceptorResults - ligandReceptorReults calculated using 
#' performLigandReceptorAnalysis()
#' @importFrom stringr str_split_fixed
#' @return ligand receptor results in long format

convertToLong = function(ligandReceptorResults){
  meanLong = formatData(ligandReceptorResults$meanInteractionsByCluster, "mean")
  totalLong = formatData(ligandReceptorResults$totalInteractionsByCluster, "total")
  pvalLong = formatData(ligandReceptorResults$pValues, "pValue")
  resultsLong = cbind(totalLong, meanLong$mean, pvalLong$pValue)
  resultsLong = cbind(resultsLong, str_split_fixed(resultsLong$clusterPair, pattern = "-", 2))
  names(resultsLong)[4:7] = c("mean","pValue","sender","receiver")
  resultsLong$negLog10PValue = -log10(resultsLong$pValue + 0.001)
  resultsLong$padj = p.adjust(resultsLong$pValue, method = "fdr")
  return(resultsLong)
}


## ####################################################
#' This is a function to create a dotplot using the ligand receptor results
#'
#' @param ligandReceptorResults - ligandReceptorResults calculated using 
#' performLigandReceptorAnalysis().
#' @param senderClusters - sender clusters to plot (defaults to all).
#' @param receiverClusters - receiver clusters to plot (defaults to all).
#' @param padjCutoff - only plot results with p-adj < padjCutoff (defaults to 
#' 0.05).
#' @param pvalCutoff - only plot results with p-value < pvalCutoff (defaults to 
#' False in which case padjCutoff is used).
#' @param pvalCutoff - only plot results with p-value < pvalCutoff (defaults to 
#' False in which case padjCutoff is used).
#' @param splitBy - split plots by "sender" or "receiver" cell types (defaults 
#' to sender).
#' @importFrom stringr str_split_fixed
#' @import ggplot2
#' @return matrix of total ligand receptor interactions that underlies the heatmap.
#' @export
#' @examples
#' getExample = make.getExample()
#' centroids = getExample('centroids')
#' ligandReceptorResults = getExample('ligandReceptorResults')
#' p = plotLRDotplot(ligandReceptorResults)
plotLRDotplot = function(ligandReceptorResults, senderClusters = unique(ligandReceptorResults$interactionsOnEdgesMeta$cellTypeA),
                         receiverClusters = unique(ligandReceptorResults$interactionsOnEdgesMeta$cellTypeB),  padjCutoff = 0.05,pvalCutoff = F, splitBy = "sender"){
  ligandReceptorResultsLong = convertToLong(ligandReceptorResults)
  resultsLongSelected = ligandReceptorResultsLong[(ligandReceptorResultsLong$sender %in% senderClusters)
                                            & (ligandReceptorResultsLong$receiver %in%  receiverClusters),]
  resultsLongSelected = resultsLongSelected[!(is.na(resultsLongSelected$padj)),]
  if (pvalCutoff){
    resultsLongSelected = resultsLongSelected[resultsLongSelected$pValue < pvalCutoff,]
  } else {resultsLongSelected = resultsLongSelected[resultsLongSelected$padj < padjCutoff,]}
  if (splitBy == "receiver"){
  p = ggplot(resultsLongSelected, aes(x=interaction, y=sender)) + geom_point(aes(size = mean,color = negLog10PValue)) +  facet_wrap(~receiver) +
    theme_classic() + theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1)) + scale_colour_gradient(low = "#ffcccb", high = "#8B0000") +
    guides(color=guide_legend(title="-Log10(p-val + 0.001)"), size=guide_legend(title="Mean")) + ylab("Sender") + xlab("") + ggtitle("Receiver")
  }
  if (splitBy == "sender"){
    p = ggplot(resultsLongSelected, aes(x=interaction, y=receiver)) + geom_point(aes(size = mean,color = negLog10PValue)) +  facet_wrap(~sender) +
    theme_classic() + theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1)) + scale_colour_gradient(low = "#ffcccb", high = "#8B0000") +
    guides(color=guide_legend(title="-Log10(p-val + 0.001)"), size=guide_legend(title="Mean")) + ylab("Receiver") + xlab("") + ggtitle("Sender")
  }
  print(p)
  return(p)
}



## ####################################################
#' This function takes interactionResults and creates a seurat object where 
#' each point represents an edge between cells, and spatial coordinates are the 
#' centroids of edges between cells. The "expression matrix" is the 
#' binarised presence/absence of an interaction (ligand receptor pair) on an edge. 
#'
#' @param ligandReceptorResults - as returned by performLigandReceptorResultsAnalysis()
#' @param centroids - a dataframe containing centroids 
#' where rownames are cellnames and the first two columns
#' contain x and y coordinates respectively.
#' @param npcs - number of pcs used for PCA, defaults to 10
#' @param returnType Determines whether to return a Seurat object or a
#' SpatialExperiment.  Will do the later if this is set to either SCE,
#' SingleCellExperiment or lower case versions of either.
#' @return This returns a seurat object where 
#' each point represents an edge between cells, and spatial coordinates are the 
#' centroids of edges between cells. The "expression matrix" is the 
#' binarised presence/absence of an interaction (ligand receptor pair) on an edge.
#' Depending on the parameter returnType, this can alternatively be returned as
#' a SpatialExperiment.
#' @export
#' @examples
#' getExample = make.getExample()
#' centroids = getExample('centroids')
#' ligandReceptorResults = getExample('ligandReceptorResults')
#' edgeSeurat = computeEdgeObject(ligandReceptorResults, centroids)
computeEdgeObject = function(ligandReceptorResults, centroids, npcs = 10,
                             returnType='Seurat'){
  interactionsOnEdges = ligandReceptorResults$interactionsOnEdges
  
  #rownames(interactionsOnEdges) = paste0(interactionsOnEdges$nodeA, "-", interactionsOnEdges$nodeB)
  #interactionsOnEdgesMat = as.matrix(interactionsOnEdges[,seq(from=5,to=ncol(interactionsOnEdges))])
  interactionsOnEdges= 1 * interactionsOnEdges
  edgeSeurat = CreateSeuratObject(t(interactionsOnEdges), meta.data = ligandReceptorResults$interactionsOnEdgesMeta)
  edgeCoords = as.data.frame(cbind(centroids[ligandReceptorResults$interactionsOnEdgesMeta$nodeA, seq_len(2)], 
                                  centroids[ligandReceptorResults$interactionsOnEdgesMeta$nodeB, seq_len(2)]))
  
  edgeCoords$edgeX = 0.6 * edgeCoords[,1] + 0.4 * edgeCoords[,3]
  edgeCoords$edgeY = 0.6 * edgeCoords[,2] + 0.4 * edgeCoords[,4] 
  
  
  edgeCentroidDF = data.frame(
    x = edgeCoords$edgeX,
    y = edgeCoords$edgeY,
    cell = colnames(edgeSeurat),
    stringsAsFactors = FALSE
  )
  
  centroidData <- list(
    "centroids" = CreateCentroids(edgeCentroidDF)
  )
  coords = CreateFOV(
    coords = centroidData,
    type = c("centroids"),
    assay = "RNA"
  )
  
  edgeSeurat[["global"]] = coords
  return(returnAs(edgeSeurat,returnType,spatial=TRUE))
}

## ####################################################
#' This function takes a spatial graph and computes a new spatial graph where
#' edges become nodes and A-B edges (in the original graph) become connected to
#' all A- edges and all B- edges. 
#' 
#' @param spatialGraph - a data frame of neighbouring edge pairs. 
#' @param selfEdges - a logical determining whether to include self edges. 
#' Defaults to False.
#' @return a graph in neighbour format  where edges in the original graph 
#' become nodes and A-B edges (in the original graph) become connected to
#' all A- edges and all B- edges. 
#' @importFrom Matrix sparseMatrix
#' @export
#' @examples
#' delaunayNeighbours = make.getExample()('delaunayNeighbours')
#' edgeNeighbours = computeEdgeGraph(delaunayNeighbours)

computeEdgeGraph = function(spatialGraph, selfEdges = FALSE){
  spatialGraph = data.table(spatialGraph)
  spatialGraph$edge = paste0(spatialGraph$nodeA, "-", spatialGraph$nodeB)
  spatialGraphEdgesA = merge.data.table(spatialGraph,
                             spatialGraph[,c(1,3)], by = "nodeA", allow.cartesian = TRUE)
  spatialGraphEdgesB = merge.data.table(spatialGraph,
                                        spatialGraph[,c(2,3)], by = "nodeB", allow.cartesian = TRUE)
  
  spatialGraphEdges = rbind(spatialGraphEdgesA[,c(3,4)],spatialGraphEdgesB[,c(3,4)])
  names(spatialGraphEdges) = c("nodeA","nodeB")
  if (! selfEdges){
    spatialGraphEdges = spatialGraphEdges[spatialGraphEdges$nodeA != spatialGraphEdges$nodeB,]
  }
  
  return(spatialGraphEdges)
}



 

## ----[SCE1] setup, include = FALSE, warning = FALSE---------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    fig.dim = c(6,6),
    comment = "#>"
)

## ----[SCE2], message=FALSE----------------------------------------------------
library(SingleCellExperiment,quietly=TRUE)
library(CatsCradle,quietly=TRUE)
library(ggplot2,quietly=TRUE)
getExample = make.getExample()
S_sce = getExample('S_sce')
df = as.data.frame(reducedDim(S_sce,'UMAP'))
df$seurat_clusters = S_sce$seurat_clusters
g = ggplot(df,aes(x=UMAP_1,y=UMAP_2,color=seurat_clusters)) +
    geom_point() +
    ggtitle('SingleCellExperiment of cells colored by seurat_cluster')
print(g)    

## ----[SCE3]-------------------------------------------------------------------
STranspose_sce = getExample('STranspose_sce')
print(STranspose_sce)
df = as.data.frame(reducedDim(STranspose_sce,'UMAP'))
df$seurat_clusters = STranspose_sce$seurat_clusters
h = ggplot(df,aes(x=umap_1,y=umap_2,color=seurat_clusters)) +
    geom_point() +
    ggtitle('SingleCellExperiment of genes colored by cluster')
print(h)  



## ----[SCE4]-------------------------------------------------------------------
hallmark = getExample('hallmark')
h = 'HALLMARK_OXIDATIVE_PHOSPHORYLATION'

idx = colnames(STranspose_sce) %in% hallmark[[h]]

k = ggplot(data=df,aes(x=umap_1,y=umap_2,color=seurat_clusters)) +
    geom_point() + 
    geom_point(data=df[idx,],aes(x=umap_1,y=umap_2),color='black',size=2.7) +
    geom_point(data=df[idx,],aes(x=umap_1,y=umap_2),color='green') +
    ggtitle(paste(h,'\non gene clusters'))

print(k)
pValue = getObjectSubsetClusteringPValue(STranspose_sce,idx)
pValue

## ----[SCE5], message=FALSE----------------------------------------------------
library(SpatialExperiment)
X_sce = getExample('X_sce')
cellCoords = as.data.frame(spatialCoords(X_sce))
cellCoords$cluster = X_sce$cluster
ell = ggplot(cellCoords,aes(x=x,y=y,color=cluster)) +
      geom_point() +
      ggtitle('Spatial coordinates colored by cell type')
print(ell)      

## ----[SCE6]-------------------------------------------------------------------
delaunayNeighbours = getExample('delaunayNeighbours')
head(delaunayNeighbours,10)

## ----[SCE7]-------------------------------------------------------------------
NBHDByCTMatrixExtended = getExample('NBHDByCTMatrixExtended')
clusters = getExample('clusters')
colours = getExample('colours')
cellTypesPerCellTypeMatrixExtended =
      computeCellTypesPerCellTypeMatrix(NBHDByCTMatrixExtended,clusters)

cellTypesPerCellTypeGraphFromCellMatrix(cellTypesPerCellTypeMatrixExtended,
                                        minWeight = 0.05, colours = colours)

## ----[SCE8]-------------------------------------------------------------------
NBHDByCTSingleCellExtended_sce = getExample('NBHDByCTSingleCellExtended_sce')
cellCoords$neighbourhood_clusters =
  NBHDByCTSingleCellExtended_sce$neighbourhood_clusters
nbhdPlot = ggplot(cellCoords,aes(x=x,y=y,color=neighbourhood_clusters)) +
           geom_point() +
	   ggtitle('Cell coordinates colored by neighbourhood_clusters')
print(nbhdPlot)	   

## ----[SCE9]-------------------------------------------------------------------
extendedNeighbours = getExample('extendedNeighbours')
smallXenium = getExample('smallXenium')
agg_sce = aggregateGeneExpression(smallXenium,extendedNeighbours,
                                    verbose=FALSE,
				    returnType='sce')
cellCoords$aggregation_clusters = agg_sce$aggregation_clusters
aggPlot = ggplot(cellCoords,aes(x=x,y=y,color=aggregation_clusters)) +
           geom_point() +
	   ggtitle('Cell coordinates colored by aggregation_clusters')
print(aggPlot)	  

## ----[SCE10], message=FALSE---------------------------------------------------
library(pheatmap)
tab = table(cellCoords[,c('aggregation_clusters','neighbourhood_clusters')])
                             
M = matrix(as.numeric(tab),nrow=nrow(tab))
rownames(M) = paste('aggregation_cluster',rownames(tab))
colnames(M) = paste('nbhd_cluster',colnames(tab))
for(i in 1:nrow(M)) M[i,] = M[i,] / sum(M[i,])
pheatmap(M)

## ----[SCE11], message=FALSE---------------------------------------------------
library(fossil,quietly=TRUE)
adjustedRandIndex = adj.rand.index(agg_sce$aggregation_clusters,
                       NBHDByCTSingleCellExtended_sce$neighbourhood_clusters)
adjustedRandIndex

## ----[SCE12]------------------------------------------------------------------
ligandReceptorResults = getExample('ligandReceptorResults')
ligandReceptorResults$interactionsOnEdges[1:10,1:10]

## ----[SCE13]------------------------------------------------------------------
sessionInfo()

## ####################################################
## These functions transpose a single cell object and
## use this to cluster and annotate genes.
## ####################################################


## ###################################################
#' Create the transpose of a Seurat object
#'
#' This takes a Seurat object f and creates a new Seurat object whose
#' expression matrix is the transpose of that of f.  This can
#' also be a SingleCellExperiment which will be converted to a
#' Seurat object
#'
#' @param f - a Seurat object
#' @param active.assay - the assay to use.  Defaults to 'RNA'
#' @param npcs - number of principal components, defaults to 30
#' @param dims - dimensions to use for umap and nearest neighbors,
#'     defaults to 1:20
#' @param res - the clustering resolution, defaults to 1
#' @param returnType - Will return a SingleCellExperiment if this is either
#' of SCE, SingleCellExperiment or their lower-case equivalents.  Otherwise,
#' returns a Seurat object
#' @param verbose - Controls whether to display trace from the Seurat
#'     functions. Defaults to FALSE
#' @return A Seurat object or SingleCellExperiment
#' @export
#' @import Seurat
#' @examples
#' exSeuratObj = make.getExample()('exSeuratObj',toy=TRUE)
#' STranspose = transposeObject(exSeuratObj)
#' STransposeAsSCE = transposeObject(exSeuratObj,returnType='SCE')
transposeObject = function(f,active.assay='RNA',
                           npcs=30,dims=seq_len(20),res=1,
                           returnType='Seurat',
                           verbose=FALSE)
 {
    f = acceptor(f)

    data = f[[active.assay]]$data
    data = t(data)
    rownames(data) = str_replace_all(rownames(data),'_','-')

    fPrime = CreateSeuratObject(data,assay=active.assay)
    fPrime[[active.assay]]$data = data
    ## fPrime[[active.assay]]$counts = NULL

    fPrime = ScaleData(fPrime,verbose=verbose)
    fPrime = FindVariableFeatures(fPrime,assay=active.assay,verbose=verbose)
    fPrime = RunPCA(fPrime,npcs=npcs,verbose=verbose)
    fPrime = RunUMAP(fPrime,reduction='pca',dims=dims,verbose=verbose)
    fPrime = FindNeighbors(fPrime,verbose=verbose)
    fPrime = FindClusters(fPrime,resolution=res,verbose=verbose)

    return(returnAs(fPrime,returnType))
 }

## ####################################################
#' This compares the gene clusters to other gene sets
#' e.g., GO, Hallmark, and determines the p-value for
#' their overlaps when compared to a set of background
#' genes.
#'
#' @param geneSets - a named list of gene sets
#' @param clusterDF - a data frame giving the cluster
#' membership of each gene with columns gene and geneCluster
#' @param backgroundGenes - a character vector of genes
#' @param adjust - a logical deciding whether to adjust
#' p values.  Defaults to FALSE.
#' @return a matrix of p-values rows correspond to the gene
#' sets and the columns correspond the the CatsCradle gene
#' clusters
#' @export
#' @examples
#' getExample = make.getExample()
#' STranspose = getExample('STranspose',toy=TRUE)
#' clusterDF = data.frame(gene=colnames(STranspose),
#'                        geneCluster=STranspose$seurat_clusters)
#' hallmark = getExample('hallmark')
#' geneSet = intersect(hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]],colnames(STranspose))
#' pvalueMatrix = geneSetsVsGeneClustersPValueMatrix(geneSet,
#'                                               clusterDF,
#'                                               colnames(STranspose))
geneSetsVsGeneClustersPValueMatrix = function(geneSets,
                                              clusterDF,
                                              backgroundGenes,
                                              adjust=FALSE)
{
    background = length(backgroundGenes)
    clusters = as.character(unique(clusterDF$geneCluster))
    clusters = as.numeric(unique(clusterDF$geneCluster))
    clusters = clusters[order(clusters)]
    NClusters = length(clusters)
    NGeneSets = length(geneSets)
    
    M = matrix(0,nrow=NGeneSets,ncol=NClusters)
    rownames(M) = names(geneSets)
    colnames(M) = as.character(clusters)
    
    for(i in seq_len(NGeneSets))
    {
        for(j in seq_len(NClusters))
        {
            cluster = clusters[j]
            idx = clusterDF$geneCluster == cluster
            clusterGenes = clusterDF$gene[idx]
            
            A = length(geneSets[[i]])
            B = length(clusterGenes)
            C = length(intersect(geneSets[[i]],clusterGenes))
            
            ## Maybe there's a better way to do this:
            M[i,j] = geneListPValue(A,B,C,background)
        }
    }
    
    if(adjust)
    {
        N = p.adjust(M,method='fdr')
        N = matrix(N,nrow=nrow(M))
        rownames(N) = rownames(M)
        colnames(N) = colnames(M)
        
        return(N)
    } else {
        return(M)
    }
}    

## ####################################################
#' This function gets the neighbors of a given gene
#' using either the gene Seurat object or its nearest
#' neighbor graph returned from getNearestNeighbourLists
#'
#' @param gene - the gene in question
#' @param NN - either the gene Seurat object or its nearest
#' neighbor graph as found by getNearestNeighbourLists.
#' This can also be a SingleCellExperiment which will be converted
#' to a Seurat object
#' @return the neighboring genes
#' @export
#' @examples
#' library(Seurat)
#' getExample = make.getExample()
#' STranspose = getExample('STranspose',toy=TRUE)
#' NN = getExample('NN',toy=TRUE)
#' neighbors = getGeneNeighbors("Ccl6",STranspose)
#' neighborsAgain = getGeneNeighbors("Ccl6",NN)
getGeneNeighbors = function(gene,NN)
{
    ## Somewhat of a fudge when NN is a NN graph:
    NN = acceptor(NN)
    
    if(inherits(NN,'Seurat'))
        NN = getNearestNeighbourLists(NN)

    NN = symmetriseNN(NN)

    idx = NN$nodeA == gene

    return(NN$nodeB[idx])
}


## ####################################################
#' Discovers the combinatorial ball of a given radius
#' around a fixed set of genes in the nearest neighbor
#' graph of a Seurat object.
#'
#' @param NN - a nearest neighbors graph
#' @param origin - a gene or list of genes
#' @param radius - the radius of the combinatorial ball
#' to be found.
#'
#' @return This returns a data frame whose columns are the
#' gene name, the radius from the origin at which it is found
#'
#' @export
#' @examples
#' getExample = make.getExample()
#' NN = getExample('NN',toy=TRUE)
#' STranspose = getExample('STranspose',toy=TRUE)
#' spheres = combinatorialSpheres(NN,'Ccl6',3)
#' hallmark = getExample('hallmark')
#' geneSet = intersect(hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]],colnames(STranspose))
#' sphereAroundSet = combinatorialSpheres(NN,geneSet,1)
combinatorialSpheres = function(NN,origin,radius)
{
    NN = symmetriseNN(NN)
    
    nodes = unique(NN$nodeA)
    ball = data.frame(nodes,
                      radius=-1)
    rownames(ball) = nodes

    ## Initialise:
    ball[origin,'radius'] = 0

    numGenes = length(nodes)
    numFound = sum(ball$radius > -.5)
    ## Iterate:
    r = 0
    while(numFound < numGenes)
    {
        r = r + 1
        if(is.numeric(radius) &
           r > radius)
            break
           
        writeLines(paste('radius',r))
        ## Which vertices can we sprout from:
        fromThese = rownames(ball[ball$radius==r-1,])

        ## Candidate nodes:
        idx = NN$nodeA %in% fromThese
        toThese = NN$nodeB[idx]
        toThese = unique(toThese)

        ## Which ones are new:
        idx = rownames(ball) %in% toThese &
            ball$radius == -1

        ball$radius[idx] = r

        newFound = sum(ball$radius > -.5)
        writeLines(paste('   ',newFound))

        ## Can only happen in a disconnected graph
        ## or with radius too large:
        if(newFound == numFound)
            break
    }

    ## Trim:
    idx = ball$radius != -1
    ball = ball[idx,]

    ## Order:
    ball = ball[order(rownames(ball)),]
    ball = ball[order(ball$radius),]

    return(ball)
}


## ####################################################
#' This function annotates genes with terms
#'
#' This essentially inverts a list of gene sets.  It takes
#' a list (e.g., Hallmark or GO) where each list item is a
#' name of a gene set and gives the genes in that set and
#' returns a list where each item is a gene and gives the
#' gene sets that gene is in.
#'
#' @param geneSets - a list of gene sets, e.g., as produced by
#' readGmt
#' @return - A list where names are genes and values are lists of terms
#' @export
#' @examples
#' hallmark = make.getExample()('hallmark')
#' annotatedGenes = annotateGenesByGeneSet(hallmark)
annotateGenesByGeneSet = function(geneSets)
{
    genes = unique(unlist(geneSets))
    genes = genes[order(genes)]

    numSets = length(geneSets)
    numGenes = length(genes)

    M = matrix(NA,nrow=numGenes,ncol=numSets)
    rownames(M) = genes
    colnames(M) = names(geneSets)
    

    for(geneSet in names(geneSets))
        for(gene in geneSets[[geneSet]])
            M[gene,geneSet] = geneSet

    genesAnno = list()
    for(gene in genes)
    {
        genesAnno[[gene]] = M[gene,]
        genesAnno[[gene]] = genesAnno[[gene]][!is.na(genesAnno[[gene]])]
    }
    return(genesAnno)   
}


## ####################################################
#' This function returns a numeric indicating which gene
#' sets it does and does not belong to.  This vector can
#' be normalised to account for the sizes of the sets.
#'
#' @param gene - the gene to annotate
#' @param geneSets - a list of gene sets
#' @param normalise - whether to normalise by set size
#' @return a numeric
#' @export
#' @examples
#' hallmark = make.getExample()('hallmark')
#' Myc = annotateGeneAsVector('Myc',hallmark)
#' MycNormalised = annotateGeneAsVector('Myc',hallmark,TRUE)
annotateGeneAsVector = function(gene,geneSets,normalise=FALSE)
{
    N = length(geneSets)
    annotation = rep(0,N)
    names(annotation) = names(geneSets)
    for(set in names(geneSets))
    {
        if(! gene %in% geneSets[[set]])
            next
        if(normalise)
            annotation[set] = 1 / length(geneSets[[set]])
        else
            annotation[set] = 1
    }
    return(annotation)
}


## ####################################################
#' This function makes annotation predictions for a
#' set of genes based on gene sets (e.g., hallmark)
#' and a CatsCradle object by considering the annotations of
#' its neighboring genes.
#'
#' @param genes - a character vector of genes
#' @param geneSets - a set of annotations, e.g., hallmark
#' or GO
#' @param fPrime - a Seurat object of genes SingleCellExperiment 
#' to be converted to a Seurat object
#' @param radius - radius for prediction neighborhood
#' @param metric - reduction or NN, defaults to umap
#' @param numPCs - used only if reduction is pca, defaults to NULL
#' @param normaliseByGeneSet - determines whether vector annotations
#'     are normalised by gene set size.  Defaults to TRUE
#' @param normaliseByDistance - determines whether neighbor
#'     contributions are normalised by edge weight.  Defaults to
#'     TRUE.
#' @param normaliseToUnitVector - determines whether to normalise
#'     returned values to unit length.  Defaults to TRUE
#' @return This returns a list of prediction vectors, one vector
#' for each gene in genes, each vector corresponding to the sets
#' in geneSets
#' @export
#' @examples
#' getExample = make.getExample()
#' STranspose = getExample('STranspose',toy=TRUE)
#' STranspose_sce = getExample('STranspose_sce',toy=TRUE)
#' hallmark = getExample('hallmark',toy=TRUE)
#' set.seed(100)
#' genes = sample(colnames(STranspose),5)
#' predictions = predictAnnotation(genes,hallmark,STranspose,radius=.5)
#' predictions_sce = predictAnnotation(genes,hallmark,STranspose_sce,radius=.5)
predictAnnotation = function(genes,
                             geneSets,
                             fPrime,
                             radius,
                             metric='umap',
                             numPCs=NULL,
                             normaliseByGeneSet=TRUE,
                             normaliseByDistance=TRUE,
                             normaliseToUnitVector=TRUE)
{
    fPrime = acceptor(fPrime)
    
    genesAnno = annotateGenesByGeneSet(geneSets)

    predictions = list()
    for(gene in genes)
    {
        ## This returns only the gene sets that show up:
        p = predictGeneAnnotationImpl(gene,fPrime,genesAnno,
                                      radius,metric,numPCs,
                                      normaliseByDistance)

        ## The 'wrapping' here is to produce a vector using
        ## all the gene sets:
        wrapped = rep(0,length(geneSets))
        names(wrapped) = names(geneSets)

        for(n in names(p))
        {
            if(normaliseByGeneSet)
                wrapped[n] = p[n] / length(geneSets[[n]])
            else
                wrapped[n] = p[n]
        }

        if(normaliseToUnitVector & sum(wrapped) != 0)
            wrapped = wrapped / Norm(wrapped)

        predictions[[gene]] = wrapped
    }

    return(predictions)
}


## ####################################################
#' This function is the implementation for predicting
#' the functions of a gene based on the functions of its
#' neighbours.
#'
#' @param gene - gene to annotate
#' @param fPrime - a Seurat object of genes or
#' SingleCellExperiment to be converted to a Seurat object
#' @param genesAnno - genes annotated with gene sets
#' @param radius - radius of neighbours to consider
#' @param metric - which metric to use to discover
#' neighbours, can be one of 'umap', 'tsne', 'pca', 'NN',
#' defaults to umap
#' @param numPCs - used only if metric is pca. Defaults to NULL
#' @param normaliseByDistance - choose whether to normalise
#' contributions of neighbors by their distance, defaults to
#' TRUE
#' @return This returns a named list.  The names are
#' the anotations that apply to the neighbour genes, the values
#' are the relative wieghts of the contributions.
#' @export 
#' @examples
#' getExample = make.getExample()
#' STranspose = getExample('STranspose',toy=TRUE)
#' hallmark = getExample('hallmark',toy=TRUE)
#' genesAnno = annotateGenesByGeneSet(hallmark)
#' predictions = predictGeneAnnotationImpl('Myc',STranspose,genesAnno,
#' radius=.5,metric='umap')
predictGeneAnnotationImpl = function(gene,fPrime,genesAnno,
                                     radius,metric,numPCs=NULL,
                                     normaliseByDistance=TRUE)
{
    fPrime = acceptor(fPrime)
    
    predictedTerms = list()

    ## Use weights only in this case:
    weights = (metric == 'NN' & normaliseByDistance==TRUE)
    nearby = getNearbyGenes(fPrime,gene,radius,metric,numPCs,weights)

    ## Neighbour genes are names.
    ## Distances are values.
    for(neighbour in names(nearby))
    {
        if (!(neighbour %in% names(genesAnno)))
            next
        
        ## extract terms for neighbour
        terms = genesAnno[[neighbour]]

        ## add these to predicted terms with appropriate weights
        for (term in terms)
        {
            if(term %in% names(predictedTerms))
                predictedTerms[[term]] =
                    predictedTerms[[term]] + 1 / nearby[neighbour]
            else
                predictedTerms[[term]] = 1 / nearby[neighbour]
        }
    }
    termNames = names(predictedTerms)
    predictedTerms = as.numeric(predictedTerms)
    names(predictedTerms) = termNames
    predictedTerms =  
        predictedTerms[order(-predictedTerms)]
    
    return(predictedTerms)
}


## ####################################################
#' This function predicts the functions of all genes based
#' on the functions of their neighbours.
#'
#' @param geneSets - a set of gene sets, e.g., hallmark
#' @param fPrime - a transposed Seurat object (generated with 
#' transposeObject()) or SingleCellExperiment to
#' be converted to a Seurat object
#' @param radius - radius of the region to use for prediction
#' @param metric - reduction or NN, defaults to umap
#' @param normaliseByGeneSet - normalise by size of each gene set,
#' defaults to TRUE
#' @param normaliseByDistance - attenutate neighbour contributions
#' based on distance, defaults to TRUE
#' @param normaliseToUnitVector - return results as unit
#' vectors, defaults to TRUE
#' @return - A list where names are genes and values are vectors
#' of gene annotations whose entries correspond to the geneSets
#' @export
#' @examples
#' getExample = make.getExample()
#' STranspose = getExample('STranspose',toy=TRUE)
#' hallmark = getExample('hallmark',toy=TRUE)
#' predictions = predictAnnotationAllGenes(hallmark,STranspose,radius=.5)
predictAnnotationAllGenes = function(geneSets,
                                     fPrime,
                                     radius,
                                     metric='umap',
                                     normaliseByGeneSet=TRUE,
                                     normaliseByDistance=TRUE,
                                     normaliseToUnitVector=TRUE)
{

    fPrime = acceptor(fPrime)
    genes = colnames(fPrime)
    
    return(predictAnnotation(genes,
                             geneSets,
                             fPrime,
                             radius,
                             metric,
                             normaliseByGeneSet,
                             normaliseByDistance,
                             normaliseToUnitVector))
}


## ####################################################
#' Nearby genes
#'
#' This finds the genes near a give subset using either
#' a dimensional reduction or the nearest neighbor graph
#'
#' @param fPrime - a Seurat object of genes or
#' SingleCellExperiment to be converted to a Seurat object
#' @param geneSet - set of genes
#' @param metric - the metric to use, one of umap, tsne,
#' pca or nearest neighbor
#' @param radius - the distance around the given set
#' @param numPCs - used only if the metric is pca
#' @param weights - whether to use edge weights in the NN case
#' @return This returns a named vector whose values are distance
#' from geneSet and whose names are the nearby genes.
#' @export
#' @examples
#' getExample = make.getExample()
#' STranspose = getExample('STranspose',toy=TRUE)
#' hallmark = getExample('hallmark')
#' geneSet = intersect(colnames(STranspose),hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]])
#' geometricallyNearby = getNearbyGenes(STranspose,geneSet,radius=0.2,metric='umap')
#' combinatoriallyNearby = getNearbyGenes(STranspose,geneSet,radius=1,metric='NN')
#' weightedNearby = getNearbyGenes(STranspose,'Myc',radius=1,metric='NN',weights=TRUE)
getNearbyGenes = function(fPrime,geneSet,radius,metric='umap',
                       numPCs=NULL,weights=FALSE)
{
    stopifnot(metric %in% c('umap','tsne','pca','NN'))

    fPrime = acceptor(fPrime)

    ## ####################################################
    ## Combinatorial with weights:
    if(weights)
    {
        stopifnot(length(geneSet) == 1 &
                  metric == 'NN' &
                  radius == 1)
        return(nearbyGenesWeighted(fPrime,geneSet))
    }

    ## ####################################################
    ## Combinatorial:
    if(metric == 'NN')
    {
        NN = getNearestNeighbourLists(fPrime)
        spheres = combinatorialSpheres(NN,geneSet,radius)
        idx = ! spheres$nodes %in% geneSet
        spheres = spheres[idx,]

        
        nearby = spheres$radius
        names(nearby) = spheres$nodes
        
        return(nearby)
    }

    ## ####################################################
    ## Metric:
    ## Get the coords:
    if(metric == 'umap')
        a = fetchUMAP(fPrime)
    if(metric == 'tsne')
        a = FetchData(fPrime,c('tSNE_1','tSNE_2'))
    if(metric == 'pca')
    {
        pcs = paste0('PC_',seq_len(numPCs))
        a = FetchData(fPrime,pcs)
    }
    S = data.matrix(a)
    rownames(S) = colnames(fPrime)

    idx = rownames(S) %in% geneSet
    X = S[idx,]
    SHat = S[!idx,]

    D = distmat(SHat,X)
    rowMin = c()
    for(i in seq_len(nrow(D)))
        rowMin[i] = min(D[i,])
    idx = rowMin <= radius

    nearby = rowMin[idx]
    names(nearby) = rownames(SHat)[idx]

    return(nearby)
}

## ####################################################
nearbyGenesWeighted = function(fPrime,gene)
{
    NN = getNearestNeighbourLists(fPrime)
    NN = symmetriseNN(NN)
    idx = NN$nodeA == gene
    neighbours = NN$nodeB[idx]
    distance = 1 / NN$weight
    names(distance) = neighbours

    return(distance)
}


## ####################################################
#' This orders the gene set p-values (or -log10 p-values) and
#' applies a cutoff (if given) to show only the significant
#' gene sets for each gene cluster
#'
#' @param M - A matrix of gene set p-values (or their logs)
#' to be ordered by their significance
#' @param ascending - Direction in which to order the columns.
#' Defaults to TRUE, so that p-values will be ordered according
#' to decreasing significance, should be set to FALSE if ordering
#' -log p-value
#' @param cutoff - if non-null this is used to extract only
#' significant cases
#' @param nameTag - can be used to modify the names of the list.
#' @return This returns a list of whose entries are data frames,
#' one for each gene cluster, each giving the significant gene sets
#' for that cluster and their significance.
#' @export
orderGeneSetPValues = function(M,ascending=TRUE,cutoff=NULL,nameTag='')
{
    answer = list()
    
    for(i in seq_len(ncol(M)))
    {
        tag = paste0(nameTag,colnames(M)[i])

        ## Get the two column df:
        a = data.frame(geneSet=rownames(M),
                       value=M[,i],
                       stringsAsFactors=FALSE)
        rownames(a) = NULL

        ## Re-order the two-col df:
        if(ascending)
            a = a[order(a$value),]
        else
            a = a[order(-a$value),]
        
        ## Apply the cutoff:
        if(!is.null(cutoff))
        {
            if(ascending)
                idx = a$value <= cutoff
            else
                idx = a$value >= cutoff

            a = a[idx,]
        }
        answer[[tag]] = a
    }
    return(answer)
}

## ####################################################
#' This gets the clusters in their cannonical order
#'
#' This deals with skullduggery in which seurat_clusters
#' has been converted from a factor to a character or a
#' numeric. 
#'
#' @param f - a Seurat object with meta.data column seurat_clusters
#'  or SingleCellExperiment to be turned into a Seurat object
#' @return A vector of these unique values in order
#' @export
#' @examples
#' STranspose = make.getExample()('STranspose',toy=TRUE)
#' geneClusters = getClusterOrder(STranspose)
getClusterOrder = function(f)
{
    f = acceptor(f)
    a = unique(f$seurat_clusters)
    a = a[order(as.numeric(a))]

    return(a)
}

## ####################################################
#' This function reads in gene sets in .gmt format
#'
#' @param gmtFile - a .gmt file containing gene sets, e.g., Hallmark of GO
#' @param addDescr - include gene set description (2nd column in .gmt file) in 
#' gene set name  
#' @return - A named list of gene sets
#' @export
readGmt = function(gmtFile, addDescr = FALSE){
    lines = readLines(gmtFile)
    geneSets = list()
    for (line in lines){
        info = strsplit(line, "\t")[[1]]
        if (addDescr){
            name = paste(info[1],info[2])
        } else {
            name = info[1]
        }
        geneSets[name] = list(info[seq(from=3,
                                       to=length(info))])
    } 
    return(geneSets)
}

## ####################################################
#' This function strips out non-gene information from
#' the beginning of GO sets, etc.
#'
#' @param geneSet - a list of gene sets
#' @return a named list of gene sets
#' @export
stripGeneSet = function(geneSet)
{
    names = character(length(geneSet))
    for(i in seq_len(length(geneSet)))
    {
        names[i] = geneSet[[i]][1]
        geneSet[[i]] = geneSet[[i]][seq(from=3,to=length(geneSet[[i]]))]
    }
    
    return(geneSet)
}

## ----setup, include = FALSE, warning = FALSE----------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    fig.dim=c(6,6),
    comment = "#>"
)

## ----[CC1], message=FALSE-----------------------------------------------------
library(CatsCradle,quietly=TRUE)
getExample = make.getExample()
exSeuratObj = getExample('exSeuratObj')
STranspose = transposeObject(exSeuratObj)

## ----[CC2], message=FALSE-----------------------------------------------------
library(Seurat,quietly=TRUE)
library(ggplot2,quietly=TRUE)
DimPlot(exSeuratObj,cols='polychrome') + ggtitle('Cell clusters on UMAP')

## ----[CC3]--------------------------------------------------------------------
DimPlot(STranspose,cols='polychrome') + ggtitle('Gene clusters on UMAP')

## ----[CC4], eval = FALSE------------------------------------------------------
# library(plotly,quietly=TRUE)
# umap = FetchData(STranspose,c('UMAP_1','UMAP_2','seurat_clusters'))
# umap$gene = colnames(STranspose)
# plot = ggplot(umap,aes(x=UMAP_1,y=UMAP_2,color=seurat_clusters,label=gene) +
#        geom_point()
# browseable = ggplotly(plot)
# print(browseable)
# htmlwidgets::saveWidget(as_widget(browseable),'genesOnUMAP.html')

## ----[CC5], message=FALSE-----------------------------------------------------
library(pheatmap,quietly=TRUE)
averageExpMatrix = getAverageExpressionMatrix(exSeuratObj,STranspose,layer='data')
averageExpMatrix = tagRowAndColNames(averageExpMatrix,
                                     ccTag='cellClusters_',
                                     gcTag='geneClusters_')
pheatmap(averageExpMatrix,
      treeheight_row=0,
      treeheight_col=0,
      fontsize_row=8,
      fontsize_col=8,
      cellheight=10,
      cellwidth=10,
      main='Cell clusters vs. Gene clusters')

## ----[CC6], eval = FALSE------------------------------------------------------
# catsCradle = sankeyFromMatrix(averageExpMatrix,
#                               disambiguation=c('cells_','genes_'),
#                               plus='cyan',minus='pink',
#                               height=800)
# print(catsCradle)

## ----[CC7]--------------------------------------------------------------------
hallmark = getExample('hallmark')
h = 'HALLMARK_G2M_CHECKPOINT'
umap = FetchData(STranspose,c('umap_1','umap_2'))
idx = colnames(STranspose) %in% hallmark[[h]]
g = DimPlot(STranspose,cols='polychrome') +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2),color='black',size=2.7) +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2),color='green') +
    ggtitle(paste(h,'\non gene clusters'))
print(g)

## ----[CC8]--------------------------------------------------------------------
g2mGenes = intersect(colnames(STranspose),
                     hallmark[['HALLMARK_G2M_CHECKPOINT']])
stats = getObjectSubsetClusteringStatistics(STranspose,
                                      g2mGenes,
                                      numTrials=1000)

## ----[CC9]--------------------------------------------------------------------
statsPlot = ggplot(data.frame(medianComplementDistance=stats$randomSubsetDistance),
                  aes(x=medianComplementDistance)) +
    geom_histogram(bins=50) +
    geom_vline(xintercept=stats$subsetDistance,color='red') +
    ggtitle('Hallmark G2M real and random median complement distance')
print(statsPlot)

## ----[CC10]-------------------------------------------------------------------
df = read.table('hallmarkPValues.txt',header=TRUE,sep='\t')
g = ggplot(df,aes(x=logPValue)) +
    geom_histogram() +
    geom_vline(xintercept=-log10(0.05)) +
    ggtitle('Hallmark gene set p-values')
print(g)

## ----[CC10.5]-----------------------------------------------------------------
h = "HALLMARK_ALLOGRAFT_REJECTION"
theSubset = hallmark[[h]]
theSubset = intersect(theSubset,colnames(STranspose))
alpha = .5
clusters = getSubsetComponents(STranspose,theSubset,alpha)
umap = FetchData(STranspose,c('umap_1','umap_2'))
umap$gene = rownames(umap)
numClusters = length(clusters)
umap$component = 0
for(i in 1:length(clusters))
    umap[clusters[[i]],'component'] = i
umap$component = factor(umap$component)
title = paste(h,'alpha =',alpha)
idx = umap$component != 0
g = ggplot() +
    geom_point(data=umap,aes(x=umap_1,y=umap_2),color='grey') +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2),color='black',size=3.5) +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2,color=component),size=3) +
    ggtitle(title)
print(g)

## ----[CC11]-------------------------------------------------------------------
meanZDF	 = meanZPerClusterOnUMAP(exSeuratObj,
                                 STranspose,
	                         'shortName')


h = ggplot(meanZDF,aes(x=umap_1,y=umap_2,color=EntericGliaCells)) +
    geom_point() +
    scale_color_gradient(low='green',high='red') +
    ggtitle('Mean z-score, Enteric glia on gene UMAP')
print(h)

## ----[CC12]-------------------------------------------------------------------
TDiff = meanZDF[,1:3]
TDiff$TDiff = meanZDF$TCells3 - meanZDF$TCells1
k = ggplot(TDiff,aes(x=umap_1,y=umap_2,color=TDiff)) +
    geom_point() +
    scale_color_gradient(low='green',high='red') +
    ggtitle('Mean z-score, TCells3 - TCells1')
print(k)

## ----[CC13]-------------------------------------------------------------------
geneSet = intersect(colnames(STranspose),
                    hallmark[['HALLMARK_INTERFERON_ALPHA_RESPONSE']])
geometricallyNearbyGenes = getNearbyGenes(STranspose,geneSet,radius=0.2,metric='umap')
theGeometricGenesThemselves = names(geometricallyNearbyGenes)
combinatoriallyNearbyGenes = getNearbyGenes(STranspose,geneSet,radius=1,metric='NN')
theCombinatoricGenesThemselves = names(combinatoriallyNearbyGenes)
df = FetchData(STranspose,c('umap_1','umap_2'))
df$gene = colnames(STranspose)
geneSetIdx = df$gene %in% geneSet
nearbyIdx = df$gene %in% theGeometricGenesThemselves
g = ggplot() +
    geom_point(data=df,aes(x=umap_1,y=umap_2),color='gray') +
    geom_point(data=df[geneSetIdx,],aes(x=umap_1,y=umap_2),color='blue') +
    geom_point(data=df[nearbyIdx,],aes(x=umap_1,y=umap_2),color='red') +
    ggtitle(paste0('Genes within geometric radius 0.2 (red) of \n',
                     'HALLMARK_INTERFERON_ALPHA_RESPONSE (blue)'))
print(g)

## ----[CC14]-------------------------------------------------------------------
annotatedGenes = annotateGenesByGeneSet(hallmark)
names(annotatedGenes[['Myc']])

## ----[CC15]-------------------------------------------------------------------
 Myc = annotateGeneAsVector('Myc',hallmark)
 MycNormalised = annotateGeneAsVector('Myc',hallmark,TRUE)

## ----[CC16]-------------------------------------------------------------------
predicted = predictAnnotation('Myc',hallmark,STranspose,radius=.5)
predicted$Myc[1:10]

## ----[CC17]-------------------------------------------------------------------
sessionInfo()

## ####################################################
## These functions focus on creating and manipulating
## neighbourhoods.
## ####################################################

## ####################################################
#' This function computes a spatial graph where 
#' neighbors are identified based on Delaunay triangulation.
#' 
#' @param centroids - a dataframe containing centroids 
#' where rownames are cellnames and the first two columns
#' contain x and y coordinates respectively.
#' @return a graph in neighbour format, i.e., a data frame with
#'     columns nodeA and nodeB.
#' @importFrom geometry delaunayn
#' @importFrom Rfast rowSort
#' @export
#' @examples
#' centroids = make.getExample()('centroids')
#' delaunayNeighbours = computeNeighboursDelaunay(centroids)
computeNeighboursDelaunay = function(centroids){
    
    ## This is confounded when centroids has extra columns:
    centroids = centroids[,seq_len(2)]
    
    ##get cell names
    cellNames = rownames(centroids)
    
    ##compute delaunay triangulation
    triangles = delaunayn(centroids)
    
    ##extract neighbours
    results = rbind(triangles[,c(1,2)],
                    triangles[,c(1,3)],
                    triangles[,c(2,3)])
    
    ##sort rows
    results = rowSort(results)
    
    ## remove duplicates
    results = unique(results)
    
    ##convert indices to cell names
    results[,1] = cellNames[as.numeric(results[,1])]
    results[,2] = cellNames[as.numeric(results[,2])]
    
    ## Convert to data.frame and name as nodeA, nodeB:
    results = as.data.frame(results)
    names(results) = c('nodeA','nodeB')
    
    return(results)
}


## ####################################################
#' This function computes a spatial graph where 
#' neighbors are identified based on euclidean distance and a 
#' user defined threshold. 
#' 
#' @param centroids - a dataframe containing centroids 
#' where rownames are cellnames and columns contain x 
#' and y coordinates respectively.
#' @param threshold - a distance cut off to compute neighbours.
#' @return a graph in neighbour format, i.e., a data frame with
#'     columns nodeA and nodeB.
#' @importFrom rdist pdist
#' @export
#' @examples
#' centroids = make.getExample()('centroids')
#' euclideanNeighbours = computeNeighboursEuclidean(centroids,20)
computeNeighboursEuclidean = function(centroids, threshold){
    centroids = centroids[,c(1,2)]
    colnames(centroids) = c("x","y")
    maxX = max(centroids[,"x"])
    minX = min(centroids[,"x"])
    maxY = max(centroids[,"y"])
    minY = min(centroids[,"y"])
    XStep = (maxX - minX)/10
    YStep = (maxY - minY)/10
    results = list()
    k = 1
    
    ##calculate distances for cells in overlapping windows  
    for (i in seq(from=0,to=9)){
        for (j in seq(from=0,to=9)){
            x1 = i * XStep + minX 
            x2 = x1 + 2*XStep
            y1 = j * YStep + minY 
            y2 = y1 + 2*YStep
            selected = centroids[(centroids[,"x"] >= x1) &
                                 (centroids[,"x"] <= x2) &
                                 (centroids[,"y"] >= y1) &
                                 (centroids[,"y"] <= y2), ]
            distances = pdist(selected) 
            distances = reshape2::melt(distances)
            
            ##retain edges within distance threshold
            distances = distances[distances$value < threshold,]
            distances = distances[,c(1,2)]
            
            ##sort rows
            distances =  rowSort(as.matrix(distances))
            distances[,1] = rownames(selected)[distances[,1]]
            distances[,2] = rownames(selected)[as.numeric(distances[,2])]
            
            results[[k]] = distances
            k = k+1
        }
    }
    
    results = do.call(rbind, results)
    
    ##remove duplicate edges
    results = unique(results)
    colnames(results) = c("nodeA", "nodeB")
    results = results[results[,"nodeA"] != results[,"nodeB"],]
    results = as.data.frame(results)
    return(results)
}

## ####################################################
#' neighbourhoodDiameter
#'
#' This function takes a list of neighbourhoods and and the
#' centroids of the cells and finds their diameters, i.e.,
#' for each neighbourhood, the maximum distance between.
#'
#' @param neighbourhoods - a list of neighbourhoods as
#' returned by nbhdsAsEdgesToNbhdsAsList
#' @param centroids - the centroids of the cells
#' @return a named numeric.  The names are the names
#' of the list neighbourhoods and the values are the
#' maximum distance within each neighbourhood
#' @export
#' @examples
#' getExample = make.getExample()
#' centroids = getExample('centroids')
#' delaunayNeighbours = getExample('delaunayNeighbours')
#' cells = unique(c(delaunayNeighbours[,'nodeA'],delaunayNeighbours[,'nodeB']))
#' nbhds = nbhdsAsEdgesToNbhdsAsList(cells,delaunayNeighbours)
#' diameters = neighbourhoodDiameter(nbhds[seq_len(100)],centroids)
neighbourhoodDiameter = function(neighbourhoods,centroids)
{
    rownames(centroids) = centroids$cell
    nbhds = names(neighbourhoods)
    diameters = c()
    for(nbhd in nbhds)
    {
        theseCells = neighbourhoods[[nbhd]]
        N = length(theseCells)
        M = matrix(0,nrow=N,ncol=2)
        M[seq_len(N),seq_len(2)] = as.matrix(centroids[theseCells,seq_len(2)])
        D = distmat(M,M)
        
        diameters[nbhd] = max(D)
    }
    
    return(diameters)
}

## ####################################################
#' This function takes a nearest neighbour graph and a radius
#' and calculates nth degree neighbour graphs where max(n) == radius
#'
#' @param spatialGraph - a nearest neighbour graph
#' @param n - the maximum degree to calculate a neighbour graph with edges 
#' connecting vertices of degree n for.
#' @return A named list of neighbour graphs, where each graph contains edges 
#' connecting vertices of degree n. Each graph is named according to degree n.
#' @importFrom data.table data.table merge.data.table
#' @export
#' @examples
#' delaunayNeighbours = make.getExample()('delaunayNeighbours')
#' extendedNeighboursList = getExtendedNBHDs(delaunayNeighbours, 4)
getExtendedNBHDs = function(spatialGraph, n){
    spatialGraph = data.table(spatialGraph)
    
    spatialGraphR = spatialGraph
    names(spatialGraphR) = c("nodeB","nodeA")
    spatialGraph = rbind(spatialGraph,spatialGraphR[,c(2,1)])
    neighbours = list()
    neighbours[[1]] = spatialGraph
    
    for (i in seq(from=2,to=n)){
        writeLines(paste('radius',i))
        graph = merge.data.table(neighbours[[i-1]], neighbours[[1]],
                                 by.x = "nodeB", 
                                 by.y = "nodeA",
                                 allow.cartesian = TRUE)
        graph = graph[,c("nodeA","nodeB.y")]
        names(graph) = c("nodeA","nodeB")
        graph = unique(graph)
        orig = c(paste0(neighbours[[i-1]]$nodeB,
                        "_",
                        neighbours[[i-1]]$nodeA))
        if (i > 2){
            orig = c(orig,paste0(neighbours[[i-2]]$nodeB,
                                 "_",
                                 neighbours[[i-2]]$nodeA))
        }
        graph = graph[graph$nodeA != graph$nodeB,]
        new = paste0(graph$nodeA,"_",graph$nodeB)
        graph = graph[!(new %in% orig),]
        neighbours[[i]] = graph
    }
    return(neighbours)
}

## ####################################################
#' This function takes an expanded neighbourhood list and collapses it to a 
#' nearest neighbourhood graph where all neighbours of degree <= n in the 
#' original graph are considered first neighbours.
#'
#' @param extendedNeighboursList - the results of getExtendedNBHDs()
#' @param n - the maximum degree to connect neighbours. Defaults to the maximum 
#' degree neighbourhoods were expanded to in the results of getExtendedNBHDs().
#' @return a graph in neighbour format, i.e., a data frame with
#'     columns nodeA and nodeB, where nodes that were originally of degree <= n 
#'     are connected.
#' @export
#' @examples
#' extendedNeighboursList = make.getExample()('extendedNeighboursList',toy=TRUE)
#' extendedNeighbours = collapseExtendedNBHDs(extendedNeighboursList, 4)
collapseExtendedNBHDs = function(extendedNeighboursList,
                                 n = length(extendedNeighboursList)){ 
    collapsedGraph = as.data.frame(do.call(rbind,
                                           extendedNeighboursList[seq_len(n)]))
    collapsedGraph = desymmetriseNN(collapsedGraph)
    
    return(collapsedGraph)
}

## ####################################################
#' nbhdsAsEdgesToNbhdsAsList
#'
#' This function takes a set of neighbourhoods given
#' by edges and turns it into a named list giving the
#' memberships of each neighbourhood
#'
#' @param cells - The cells whose neighbourhoods to
#' extract. 
#' @param neighbourhoods - neighbourhoods given as a
#' data frame with columns nodeA and nodeB, for example
#' the output of collapseNeighbourhoods
#' @param self - include cell in its neighbourhood, defaults to FALSE
#' @return a named list with memberships of the neighbourhoods
#' of cells
#' @export
#' @examples
#' delaunayNeighbours = make.getExample()('delaunayNeighbours')
#' cells = unique(c(delaunayNeighbours[,'nodeA'],delaunayNeighbours[,'nodeB']))
#' nbhdsList = nbhdsAsEdgesToNbhdsAsList(cells,delaunayNeighbours)
nbhdsAsEdgesToNbhdsAsList = function(cells,
                                     neighbourhoods, self = FALSE)
{
    nbhdList = list()
    if (self) {
      for(cell in cells)
      {
          nbhdList[[cell]] = c(cell,
                               neighbourhoods$nodeB[neighbourhoods$nodeA == cell])
      }
    } else {
      for(cell in cells)
      {
        nbhdList[[cell]] = c(neighbourhoods$nodeB[neighbourhoods$nodeA == cell])
      }
    }
    return(nbhdList)
}

## ####################################################
#' exSeuratObj
#'
#' A Seurat object of 2000 genes by 540 cells.
#'
#' @format A Seurat object
#'
#' \describe{
#' A Seurat object of cells.  It includes a UMAP of the
#' cells and annotated clustering into cell types. It has
#' been severely reduced in size to accommodate Bioconductor
#' size restrictions.
#' }  
#' @source This is subset from the data associated with
#'     https://www.nature.com/articles/s41586-021-04006-z
"exSeuratObj"

## ####################################################
#' humanLRN
#'
#' A data frame giving 12019 human ligand receptor pairs
#'
#' @format a data frame with two columns, 'from' and 'to'
#' \describe{A data frame with two columns, 'from' and 'to'.
#' Each row represents a human ligand - receptor pair.
#' }
#' @source This is taken from the nichenetr package,
#' url = {https://www.nature.com/articles/s41592-019-0667-5}.
#' Specifically we use the human ligand - receptor network.
"humanLRN"

## ####################################################
#' mouseLRN
#'
#' A data frame giving 11592 mouse ligand receptor pairs
#'
#' @format a data frame with two columns, 'from' and 'to'
#' \describe{A data frame with two columns, 'from' and 'to'.
#' Each row represents a mouse ligand - receptor pair.
#' }
#' @source This is taken from the nichenetr package,
#' url = {https://www.nature.com/articles/s41592-019-0667-5}.
#' Specifically, we use the mouse ligand - receptor network.
"mouseLRN"

## ####################################################
#' smallXenium
#'
#' A spatial Seurat object of 4261 cells and 248 genes
#'
#' @format A Seurat object
#'
#' \describe{
#' A spatial Seurat object subset from the Xenium object used in
#' https://satijalab.org/seurat/articles/seurat5_spatial_vignette_2.
#' }
#' 
#' @source This is subset from the Xenium spatial Seurat object
#' https://cf.10xgenomics.com/samples/xenium/1.0.2/Xenium_V1_FF_Mouse_Brain_Coronal_Subset_CTX_HP/Xenium_V1_FF_Mouse_Brain_Coronal_Subset_CTX_HP_outs.zip
#' to include a small region of the field of view surrounding the dentate gyrus. 
"smallXenium"

## ####################################################
#' moransI
#'
#' A data fame containing Moran's I and related pvalues.
#'
#' @format A data fame containing Moran's I and related pvalues.
#'
#' \describe{
#' Moran's I values calculated for the genes in smallXenium (using the SCT 
#' assay). Pvalues derived using 100 permutations.
#' }
#'
#' @source Created from smallXenium and delaunayNeighbours by using
#' runMoransI()
"moransI"


## ####################################################
#' ligandReceptorResults
#'
#' The result of performLigandReceptorAnalysis(smallXenium, delaunayNeighbours, 
#' "mouse", clusters,verbose=FALSE) 
#'
#' @format A list of data frames.
#'
#' \describe{
#' A list containing:
#' interactionsOnEdges - a data frame whose first two columns give
#' the neighbouring cells and next two columns give their corresponding 
#' clusters. Each of the remaining columns is a logical
#' corresponding to a ligand-receptor pair telling whether the ligand
#' is expressed in the first cell and the receptor is expressed in the
#' second cell.
#' totalInteractionsByCluster - a dataframe where the first column gives a 
#' directed (sender-receiver) pair of clusters. The second column gives the 
#' total number of edges between those clusters. The remaining columns give the 
#' total numbers of edges on which particular ligand receptor interactions are 
#' present.
#' meanInteractionsByCluster - a dataframe where the first column gives a 
#' directed (sender-receiver) pair of clusters. The second column gives the 
#' total number of edges between those clusters. The remaining columns give the 
#' total numbers of edges on which particular ligand receptor interactions are 
#' present (for that cluster pair) divided by the total number of edges between 
#' those clusters.
#' simResults - a dataframe where the rownames are sender-receiver cluster pairs 
#' and column names are ligand receptor pairs. Values give the number of 
#' simulations for which observed values are greater than simulated values.
#' pValues - a dataframe where the rownames are sender-receiver cluster pairs 
#' and column names are ligand receptor pairs. Entries are uppertail pvalues 
#' describing whether a particular ligand receptor interaction is observed more 
#' frequently between 2 clusters than expected.
#' }
#'
#' @source Created from smallXenium and delaunayNeighbours by using
#' performLigandReceptorAnalysis(()
"ligandReceptorResults"

## ####################################################
#' moransILigandReceptor
#' 
#' Moran's I for the ligand receptor pairs
#'
#' @format A data frame showing the spatial autocorrelation of the
#' 28 ligand receptor pairs
#'
#' \describe{
#' A data frame with rownames giving the  28 ligand-receptor pairs and columns moransI
#' and pValues
#' }
#'
#' @source Computed using the function runMoransI on the object edgeSeurat and
#' neighbours edgeNeighbours = computeEdgeGraph(delaunayNeighbours) with 100 trials.
#' For more informations see the CatsCradleSpatial vignette.
"moransILigandReceptor"

## ####################################################
#' seuratGenes
#'
#' A vector of genes used for subsetting exSeuratObj
#'
#' @format A vector of genes
#'
#' \describe{
#' A vector of the top 100 most variable genes in exSeuratObj
#' used to subset this object to give toy examples.
#' }
#'
#' @source Computed by retrieving the data layer from exSeuratObj and
#' subsetting to the 100 genes with the highest standard deviation.
"seuratGenes"

## ####################################################
#' seuratCells
#'
#' A vector of cells used for subsetting exSeuratObj
#'
#' @format A vector of cells
#'
#' \describe{
#' A vector of cells consisting of half the cells from
#' each seurat_cluster in exSeuratObj used to subset this
#' object to give toy examples.
#' }
#'
#' @source Computed by retrieving half the cells from each
#' cluster in exSeuratObj
"seuratCells"

## ####################################################
#' xeniumCells
#'
#' A vector of cells used for subsetting exSeuratObj
#'
#' @format A vector of cells
#'
#' \describe{
#' A vector of cells consisting of approximately one
#' quarter of the cells in smallXenium used to subset
#' this object to give toy examples.
#' }
#'
#' @source We extracted a rectangle whose width and
#' height were one half the width and height of smallXenium
#' and which was centered in the field of view of
#' smallXenium
"xeniumCells"

## ----setup, include = FALSE, warning = FALSE----------------------------------
knitr::opts_chunk$set(
    collapse = TRUE,
    fig.dim = c(6,6),
    comment = "#>"
)

## ----[QS1], message=FALSE-----------------------------------------------------
library(Seurat,quietly=TRUE)
library(CatsCradle,quietly=TRUE)
data(exSeuratObj)
DimPlot(exSeuratObj,cols='polychrome')

## ----[QS2]--------------------------------------------------------------------
getExample = make.getExample()
STranspose = getExample('STranspose')
print(STranspose)
DimPlot(STranspose,cols='polychrome')

## ----[QS3], message=FALSE-----------------------------------------------------
library(ggplot2,quietly=TRUE)
hallmark = getExample('hallmark')
h = 'HALLMARK_OXIDATIVE_PHOSPHORYLATION'
umap = FetchData(STranspose,c('umap_1','umap_2'))
idx = colnames(STranspose) %in% hallmark[[h]]
g = DimPlot(STranspose,cols='polychrome') +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2),color='black',size=2.7) +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2),color='green') +
    ggtitle(paste(h,'\non gene clusters'))
print(g)
pValue = getObjectSubsetClusteringPValue(STranspose,idx)
pValue

## ----[QS4]--------------------------------------------------------------------
smallXenium = getExample('smallXenium')
ImageDimPlot(smallXenium,cols='polychrome')

## ----[QS5]--------------------------------------------------------------------
delaunayNeighbours = getExample('delaunayNeighbours')
head(delaunayNeighbours,10)

## ----[QS6]--------------------------------------------------------------------
NBHDByCTMatrixExtended = getExample('NBHDByCTMatrixExtended')
clusters = getExample('clusters')
colours = getExample('colours')
cellTypesPerCellTypeMatrixExtended = computeCellTypesPerCellTypeMatrix(NBHDByCTMatrixExtended,clusters)

cellTypesPerCellTypeGraphFromCellMatrix(cellTypesPerCellTypeMatrixExtended, minWeight = 0.05, colours = colours)

## ----[QS7]--------------------------------------------------------------------
NBHDByCTSeuratExtended = getExample('NBHDByCTSeuratExtended')
smallXenium$NBHDClusterExtended= 
  NBHDByCTSeuratExtended@active.ident
ImageDimPlot(smallXenium, group.by = c("NBHDClusterExtended"), 
             size = 1, cols = "polychrome")

## ----[QS8]--------------------------------------------------------------------
extendedNeighbours = getExample('extendedNeighbours')
agg = aggregateGeneExpression(smallXenium,extendedNeighbours,
                                    verbose=FALSE)
smallXenium$aggregateNBHDClusters = agg@active.ident
ImageDimPlot(smallXenium,group.by='aggregateNBHDClusters',cols='polychrome',
             size=1)

## ----[QS9], message=FALSE-----------------------------------------------------
library(pheatmap)
tab = table(smallXenium@meta.data[,c("aggregateNBHDClusters",
                                     "NBHDClusterExtended" )])
M = matrix(as.numeric(tab),nrow=nrow(tab))
rownames(M) = paste('aggregation_cluster',rownames(tab))
colnames(M) = paste('nbhd_cluster',colnames(tab))
for(i in 1:nrow(M)) M[i,] = M[i,] / sum(M[i,])
pheatmap(M)

## ----[QS10], message=FALSE----------------------------------------------------
library(fossil,quietly=TRUE)
adjustedRandIndex = adj.rand.index(agg@active.ident,
                                   NBHDByCTSeuratExtended@active.ident)
adjustedRandIndex

## ----[QS11]-------------------------------------------------------------------
ligandReceptorResults = getExample('ligandReceptorResults')
ligandReceptorResults$interactionsOnEdges[1:10,1:10]

## ----[QS12]-------------------------------------------------------------------
sessionInfo()

## ####################################################
## These functions focus on manipulation of nearest
## neighbour graphs.
## ####################################################


## ####################################################
#' This function extracts a shared nearest neighbor network
#' from a Seurat object
#'
#' @param f - a Seurat object or SingleCellExperiment to
#' be converted to a Seurat object
#' @param graph - which graph to extract.  Defaults to
#' paste0(f@active.assay,'_snn')
#' @return - This returns dataframe of neighbors:
#' nodeA - node names for node A 
#' nodeB - node names for node B
#' weight - edge weight
#' @export
#' @examples
#' STranspose = make.getExample()('STranspose',toy=TRUE)
#' NN = getNearestNeighbourLists(STranspose)
getNearestNeighbourLists = function(f, graph=defaultGraph(f)){

    f = acceptor(f)
  
    ## convert to TsparseMatrix and extract relevant information
    graph = as(f@graphs[[graph]], "TsparseMatrix") 
    neighborListDf = data.frame("nodeA" = graph@Dimnames[[1]][graph@i+1],
                                "nodeB" =  graph@Dimnames[[2]][graph@j+1], 
                                "weight" = graph@x)
    
    ## remove self-loops
    neighborListDf = 
        neighborListDf[neighborListDf$nodeA != neighborListDf$nodeB,]
    return(neighborListDf)
}


## ####################################################
#' This function takes the data frame of neighbor genes
#' and reduces it so that each undirected edge is
#' represented by only one directed edge.  This ensures
#' that randomisation does not magically split undirected
#' edges into two edges.
#'
#' @param NN - a dataframe containing the neighborlist
#' @return - a neighborListDF with only one directed edge per
#' undirected edge.
#' @export
#' @examples
#' NN = make.getExample()('NN',toy=TRUE)
#' print(dim(NN))
#' NNN = desymmetriseNN(NN)
#' print(dim(NNN))
desymmetriseNN = function(NN)
{
    ## Use this to order the genes on each edge
    orderGenes = function(i)
    {
        if(order(c(NN$nodeA[i],NN$nodeB[i]))[1] == 2)
            return(TRUE)
        return(FALSE)
    }

    ## Order the genes on each edge:
    idx = unlist(lapply(seq_len(nrow(NN)),orderGenes))
    NN[idx,seq_len(2)] = NN[idx,seq(from=2,to=1)]

    ## Delete the duplicates:
    tag = paste(NN$nodeA,NN$nodeB,sep='___')
    NN = NN[!duplicated(tag),]

    return(NN)
}

## ####################################################
#' This function generates random indices for node B
#'
#' @param neighborListDf - a dataframe containing the neighborlist
#' @param n - the number of times to randomise indices
#' @param useWeights - whether to preserve edgeweights.
#' @return - a matrix with randomised indices for node B
#' @export
#' @examples
#' NN = make.getExample()('NN')
#' NN = desymmetriseNN(NN)
#' randomIndices = randomiseNodeIndices(NN,10,TRUE)
randomiseNodeIndices = function(neighborListDf, n = 100, useWeights = FALSE){
    NN = desymmetriseNN(neighborListDf)
    if(!identical(NN,neighborListDf))
        stop(paste0('randomiseNodeIndices is meant to be used',
                    'with a desymmetrised neighborList'))
  
  #determine number of edges and create empty matrix for randomised indices
  nEdges = nrow(neighborListDf)
  indices = seq_len(nEdges)
  randomIndices = matrix(, nrow = nEdges, ncol = n)
  
  #check if weights are to be used
  if (useWeights){
    
    #determine unique weights
    weights = unique(neighborListDf$weight)
    for (i in seq_len(n)){
      
      #randomise indices within each weight category
      for (weight in weights){
        selected = neighborListDf$weight == weight
        randomIndices[selected,i] =
          sample(indices[selected])
      }
    }
  }
  
  #otherwise ignore weights and randomise indices
  else {
    for (i in seq_len(n)){
      randomIndices[,i] = sample(indices)
    }
  }
  return(randomIndices)
}


## ####################################################
#' Tests whether a nearest neighbor graph is symmetric
#'
#' The nearest neighbor relationship is not inherently
#' symmetric.  This tests whether the nearest neighbor graph
#' retrieved from a Seurat object is.
#'
#' @param NN - a nearest neighbor graph.  This is in the form
#' of a data frame  as returned by getNearestNeighbourLists.
#' Its coloumns include nodeA and nodeB.
#' @return TRUE or FALSE
#' @export
#' @examples
#'  NN = make.getExample()('NN',toy=TRUE)
#' symmetryTest = symmetryCheckNN(NN)
symmetryCheckNN = function(NN)
{
    tagA = paste(NN$nodeA,NN$nodeB)
    tagA = tagA[order(tagA)]
    
    tagB = paste(NN$nodeB,NN$nodeA)
    tagB = tagB[order(tagB)]
    
    return(identical(tagA,tagB))
}


## ####################################################
#' This symmetrises a nearest neighbors graph.
#'
#' This first checks to see if the NN graph is symmetric
#' and if not symmetrises it.
#'
#' @param NN - a nearest neighbors graph as returned
#' by getNearestNeighbourLists
#'
#' @return a nearest neighbors graph
#' 
#' @export
#' @examples
#' NN = make.getExample()('NN',toy=TRUE)
#' NNStar = symmetriseNN(NN)
symmetriseNN = function(NN)
{
    ## Is it already symmetric?
    if(symmetryCheckNN(NN))
        return(NN)

    ## If not, symmetrise:
    NN2 = NN
    NN2[,seq_len(2)] = NN2[,seq(from=2,to=1)]
    NN = rbind(NN,NN2)

    tag = paste(NN$nodeA,NN$nodeB)
    idx = ! duplicated(tag)

    NN = NN[idx,]

    return(NN)
}

## ####################################################
#' This function performs degree-preserving randomisation of neighbour graphs. 
#'
#' @param spatialGraph - a spatial graph in neighbour list format.
#' @param maxTries - the maximum number of tries to remove self edges during 
#' graph randomisation. If self edges are remaining this will be reported.
#' @return A randomised graph where degree from the original graph is preserved.
#' We also report any duplicated edges.
#' @export
randomiseGraph = function(spatialGraph, maxTries = 1000){
  n = nrow(spatialGraph)
  toFlip = sample(seq_len(n), size = round(n/2))
  simGraph = spatialGraph[toFlip,c(2,1)]
  names(simGraph) =  c("nodeA","nodeB")
  simGraph = rbind(simGraph, spatialGraph[!(seq_len(n) %in% toFlip),])
  simGraph[,2] = sample(simGraph[,2])
  selfEdge = which(simGraph$nodeA == simGraph$nodeB)
  i = 1
  while((length(selfEdge) > 0) & (i <= maxTries)){
    nodeB = simGraph$nodeB
    simGraph$nodeB[selfEdge] = nodeB[(selfEdge + 1) %% n]
    simGraph$nodeB[(selfEdge + 1) %% n] = nodeB[selfEdge]
    selfEdge = which(simGraph$nodeA == simGraph$nodeB)
    i = i + 1
  }
  nSelf = length(selfEdge)
  if (nSelf > 0){
    print(paste0(nSelf, " self edges not removed"))
  }
  ndup = sum(duplicated(simGraph))
  if(ndup > 0) {
      print(paste0(ndup," duplicated edges"))
  }
      
  return(simGraph)
}

## ####################################################
## These functions study how expression of the gene
## clusters vary across cells and cell clusters.
## ####################################################



## ###################################################
#' This computes average expression of each gene cluster in
#' each cell cluster and returns the result as a matrix
#'
#' @param f - The Seurat object of cells, or SingleCellExperiment
#' to be turned into a Seurat object
#' @param fPrime - The Seurat object of genes,  or SingleCellExperiment
#' to be turned into a Seurat object
#' @param clusteringName In many cases, this will be the cell
#'     clustering, i.e., seurat_clusters, which is the default, but
#'     for neighbourhood Seurat objects, this can be
#'     neighbourhood_clusters.
#' @param layer - layer to use for expression values
#' @return A matrix of the average expression where the rows
#' correspond to cell clusters and the columns correspond to
#' gene clusters.
#' @export
#' @importFrom stringr str_replace str_replace_all
#' @importFrom stringr str_split str_split_fixed
#' @examples
#' getExample = make.getExample()
#' STranspose = getExample('STranspose',toy=TRUE)
#' exSeuratObj = getExample('exSeuratObj',toy=TRUE)
#' M = getAverageExpressionMatrix(exSeuratObj,STranspose,layer='data')
getAverageExpressionMatrix = function(f,fPrime,
                                      clusteringName='seurat_clusters',
                                      layer='scale.data')
{
    f = acceptor(f)
    fPrime = acceptor(fPrime)
    
    f$clustering = as.character(f@meta.data[,clusteringName])
    cellCluster = unique(f$clustering)
    cellCluster = cellCluster[order(as.numeric(cellCluster))]
    
    fPrime$clustering = as.character(fPrime@meta.data[,clusteringName])
    geneCluster = unique(fPrime$clustering)
    geneCluster = geneCluster[order(as.numeric(geneCluster))]
    
    ## Get assay data:
    X = GetAssayData(f,layer=layer)
    
    ## Seems X can be smaller:
    selected = rownames(X)
    f = f[selected,]
    selectedUnderscore = str_replace(selected,'-','_')
    if (sum(selectedUnderscore %in% colnames(fPrime)) > 
        sum(selected %in% colnames(fPrime))){
      fPrime = fPrime[,selectedUnderscore]
    } else {
      fPrime = fPrime[,selected]
    }
   
    M = matrix(0,nrow=length(cellCluster),ncol=length(geneCluster))
    rownames(M) = cellCluster
    colnames(M) = geneCluster
  
    for(i in cellCluster)
    {
        for(j in geneCluster)
        {
            idxI = f$clustering == i
            idxJ = fPrime$clustering == j
            
            M[i,j] = sum(X[idxJ,idxI]) / (sum(idxI) * sum(idxJ))
        }
    }
    return(M)
}


## ####################################################
#' This gussies up the rownames and colnames of M
#'
#' @param M - a matrix, typically the average expression matrix
#' @param ccTag - a prefix for the row (cell cluster) names
#' @param gcTag - a prefix for the column (gene cluster) names
#' @return The same matrix with fancier row and col names
#' @export
#' @examples
#' getExample = make.getExample()
#' averageExpMatrix = getExample('averageExpMatrix',toy=TRUE)
#' averageExpMatrix = tagRowAndColNames(averageExpMatrix,'cellCluster_','geneCluster_')
tagRowAndColNames = function(M,ccTag='CC_',gcTag='GC_')
{
  rownames(M) = paste0(ccTag,rownames(M))
  colnames(M) = paste0(gcTag,colnames(M))
  
  return(M)
}

## ####################################################
#' This converts an average gene expression matrix to a
#' data frame.
#'
#' @param M - An average gene expression matrix.
#' @return A data frame with columns cellCluster, geneCluster
#' and average expression
#' @importFrom reshape2 melt
#' @export
#' @examples
#' getExample = make.getExample()
#' averageExpMatrix = getExample('averageExpMatrix',toy=TRUE)
#' averageExpDF = getAverageExpressionDF(averageExpMatrix)
getAverageExpressionDF = function(M)
{
    df = melt(M)
    names(df) = c('cellCluster','geneCluster','averageExpression')

    return(df)
}


## ####################################################
#' This makes a sankey graph from a matrix of average
#' expression.  Our "Cat's Cradle".
#'
#' @param M - a matrix of gene expression
#' @param disambiguation - used to distinguish between
#' the row names and the column names if these overlap
#' @param fontSize - defaults to 20
#' @param minus - colour to use for links with negative
#' values
#' @param plus - colour for positive values
#' @param height - height in pixels, defaults to 1200
#' @param width - width in pixels, defaults to 900
#' @return A sankey graph
#' @export
#' @importFrom networkD3 sankeyNetwork
#' @examples
#' set.seed(100)
#' M = matrix(runif(12)-.3,nrow=3)
#' rownames(M) = as.character(seq_len(3))
#' colnames(M) = as.character(seq_len(4))
#' sankey = sankeyFromMatrix(M)
sankeyFromMatrix = function(M,disambiguation=c('R_','C_'),
                            fontSize=20,minus='red',plus='blue',
                            height=1200,width=900)
{
  ## Maybe the matrix doesn't have row and column names:
  if(is.null(rownames(M)))
    rownames(M) = paste0(disambiguation[1],seq_len(nrow(M)))
  if(is.null(colnames(M)))
    colnames(M) = paste0(disambiguation[1],seq_len(ncol(M)))
  
  ## Create the links DF:
  from = rownames(M)
  to = colnames(M)
  
  if(length(intersect(from,to)) > 0)
  {
    from = paste0(disambiguation[1],from)
    to = paste0(disambiguation[2],to)
  }
  
  source = rep(from,each=length(to))
  target = rep(to,length(from))
  
  value = as.numeric(t(M))
  
  links = data.frame(source,target,value,
                     stringsAsFactors=FALSE)
  
  ## Colour the links DF:
  idx = links$value >= 0
  links$group = ''
  links$group[idx] = 'plus'
  links$group[!idx] = 'minus'
  
  links$value = abs(links$value)
  
  ## Create the nodes DF:
  nodes = unique(c(links$source,links$target))
  nodes = data.frame(name=nodes,
                     stringsAsFactors=FALSE)
  
  links$IDsource = match(links$source,nodes$name) - 1
  links$IDtarget = match(links$target,nodes$name) - 1
  
  linkColour = 'd3.scaleOrdinal() .domain(["minus","plus"]) .range(["X", "Y"])'
  linkColour = str_replace(linkColour,'X',minus)
  linkColour = str_replace(linkColour,'Y',plus)  
  
  
    p = sankeyNetwork(Links = links, Nodes = nodes,
                      Source = "IDsource", Target = "IDtarget", 
                      Value = "value", NodeID = "name", 
                      colourScale=linkColour,
                      LinkGroup="group",
                      fontSize=fontSize,
                      height=height,
                      width=width)
  
  return(p)
}


## ####################################################
#' This produces a matrix giving the average expression of gene
#' clusters in cells.  By default, it uses all cells and all gene
#' clusters.
#'
#' @param f - the cell Seurat object or SingleCellExperiment to be
#' turned into a Seurat object
#' @param fPrime - the genes Seurat object or SingleCellExperiment to be
#' turned into a Seurat object
#' @param cells - the cells to compute this for
#' @param geneClusters - the geneClusters to compute average
#' expression for
#' @param layer - the data layer to use, defaults to 'data'
#' @return A matrix where the rows correspond to cells, the columns
#' correspond to geneClusters and the entries give average expression
#' for each cluster in each cell
#' @export
#' @examples
#' getExample = make.getExample()
#' exSeuratObj = getExample('exSeuratObj',toy=TRUE)
#' STranspose = getExample('STranspose',toy=TRUE)
#' clusterExpression = getGeneClusterAveragesPerCell(exSeuratObj,STranspose)
getGeneClusterAveragesPerCell = function(f,
                                         fPrime,
                                         cells=colnames(f),
                                         geneClusters=getClusterOrder(fPrime),
                                         layer='data')
{
    f = acceptor(f)
    fPrime = acceptor(fPrime)
    
    M = matrix(0,nrow=length(cells),ncol=length(geneClusters))
    rownames(M) = cells
    colnames(M) = paste0('cluster_',geneClusters)

    for(i in seq_len(length(geneClusters)))
    {
        cluster = geneClusters[i]
        idx = fPrime$seurat_clusters == cluster
        theseGenes = colnames(fPrime)[idx]
        expression = data.matrix(FetchData(f,theseGenes,layer=layer))
        expression = expression[rownames(expression) %in% cells,]

        M[,i] = rowMeans(expression)
    }

    return(M)
}


## ####################################################
#' Mean gene cluster on cell umap
#'
#' This function paints gene expression for a
#' given gene cluster on cell umap.
#'
#' @param f - a Seurat object of cells or SingleCellExperiment to
#' be converted to a Seurat object
#' @param fPrime - the corresponding Seurat object of genes
#' SingleCellExperiment to be converted to a Seurat object
#' @param geneCluster - a gene cluster of fPrime
#' @return This returns a ggplot object
#' @importFrom ggplot2 ggplot geom_point geom_density theme xlim ggtitle
#' @importFrom ggplot2 geom_vline facet_wrap aes
#' @export
#' @examples
#' getExample = make.getExample()
#' exSeuratObj = getExample('exSeuratObj',toy=TRUE)
#' STranspose = getExample('STranspose',toy=TRUE)
#' g = meanGeneClusterOnCellUMAP(exSeuratObj,STranspose,geneCluster=0)
meanGeneClusterOnCellUMAP = function(f,fPrime,geneCluster)
{
    f = acceptor(f)
    fPrime = acceptor(fPrime)
    
    plotDF = fetchUMAP(f)
    idx = fPrime$seurat_clusters == geneCluster
    genes = colnames(fPrime)[idx]
    genes = intersect(genes,rownames(f))
    expression = FetchData(f,genes)
    plotDF$expression = rowMeans(expression)

    g = ggplot(plotDF,aes(x=UMAP_1,y=UMAP_2,color=expression)) +
        geom_point()

    return(g)
}

## ####################################################
#' This gets z-scores for the values of features
#'
#' @param f - a Seurat object of cells or SingleCellExperiment to
#' be converted to a Seurat object
#' @param features - a set of features to retrieve z-scores for,
#' defaults to rownames(f)
#' @param layer - the data layer to retrieve
#' @return This returns a data frame with a column for each
#' feature and a row for each cell
#' @export
#' @examples
#' getExample = make.getExample()
#' exSeuratObj = getExample('exSeuratObj',toy=TRUE)
#' df = getFeatureZScores(exSeuratObj)
getFeatureZScores = function(f,features=rownames(f),layer='data')
{
    f = acceptor(f)
    
    df = FetchData(f,features,layer=layer)
    
    for(i in 1:ncol(df))
        df[,i] = (df[,i] - mean(df[,i])) / sd(df[,i])

    return(df)
}

## ####################################################
#' This finds the mean z-score for features in subsets
#' of cells e.g., in each of the seurat_clusters
#'
#' @param f - a Seurat object of cells or SingleCellExperiment to
#' be converted to a Seurat object
#' @param features - a set of features of f
#' @param clusterBy - the name of the column of f@meta.data to
#' be used to subset the cells
#' @param layer - the data layer to be used for z-scores
#' @return This returns a data frame each of whose columns
#' corresponds to a value of the clusterBy data.  In the case
#' where the clusterBy data is a factor or numeric, it prepends
#' cluster_ to the column name.
#' @export
#' @examples
#' getExample = make.getExample()
#' exSeuratObj = getExample('exSeuratObj',toy=TRUE)
#' STranspose = getExample('STranspose',toy=TRUE)
#' df = meanZPerCluster(exSeuratObj,features=colnames(STranspose),
#'                      clusterBy='shortName')
meanZPerCluster = function(f,
                           features,
                           clusterBy='seurat_clusters',
                           layer='data')
{
    f = acceptor(f)
    
    clusters = unique(f@meta.data[,clusterBy])
    clusters = clusters[order(clusters)]
    if(isa(clusters,'numeric') |
       isa(clusters,'factor'))
    {
        Clusters = paste0('cluster_',clusters)
    } else {
        Clusters = as.character(clusters)
    }

    z = getFeatureZScores(f,features=features,layer=layer)

    df = list()
    for(i in 1:length(clusters))
    {
        cluster = clusters[i]
        Cluster = Clusters[i]

        cells = colnames(f)[f@meta.data[,clusterBy] == cluster]
        meanZ = colMeans(z[cells,])

        df[[Cluster]] = meanZ
    }
    df = as.data.frame(df)
    rownames(df) = features

    return(df)
}

## ####################################################
#' This collects together mean z-score data together with
#' UMAP coordinates from the gene seurat object for plotting.
#'
#' @param f - a Seurat object of cells or SingleCellExperiment to
#' be converted to a Seurat object
#' @param fPrime - the corresponding Seurat object of genes
#' SingleCellExperiment to be converted to a Seurat object
#' @param clusterBy - the name of the column of f@meta.data to
#' be used to subset the cells
#' @param layer - the data layer to be used for z-scores
#' @return This returns a data frame with the UMAP coordinates
#' of the gene Seurat object and the average z-score for each
#' gene within each of the cell clusters defined by the clusterBy
#' column of the meta.data of f.
#' @export
#' @examples
#' getExample = make.getExample()
#' exSeuratObj = getExample('exSeuratObj',toy=TRUE)
#' STranspose = getExample('STranspose',toy=TRUE)
#' df = meanZPerClusterOnUMAP(exSeuratObj,STranspose,clusterBy='shortName')
meanZPerClusterOnUMAP = function(f,
                                 fPrime,
                                 clusterBy='seurat_clusters',
                                 layer='data')
{
    f = acceptor(f)
    fPrime = acceptor(fPrime)
    
    umap = FetchData(fPrime,c('umap_1','umap_2'))
    umap = cbind(gene=colnames(fPrime),umap)
    rownames(umap) = umap$gene

    meanZ = meanZPerCluster(f,
                            features=colnames(fPrime),
                            clusterBy=clusterBy,
                            layer=layer)
    umap = cbind(umap,meanZ)

    return(umap)
}

## ####################################################
#' This function makes the function whichretrieves and
#' makes example data objects.
#'
#' @return This returns the function which retrieves
#' and makes example data objects. The latter saves any
#' object it has found for quicker return.  Using the value 'list'
#' causes it to return the list of all objects found so far.
#' @importFrom SummarizedExperiment assays
#' @importFrom msigdbr msigdbr
#' @export
#' @examples
#' getExample = make.getExample()
#' ## Provided:
#' smallXenium = getExample('smallXenium')
#' ## Computed:
#' delaunayNeighbours = getExample('delaunayNeighbours')
make.getExample = function()
{
    
    alreadyFound = list()
    toysFound = list()

    
    
    getExample = function(whichOne,toy=FALSE)
    {
        packageVars = c('exSeuratObj',
                        'smallXenium',
                        'ligandReceptorResults',
                        'moransI',
                        'moransILigandReceptor',
                        'humanLRN',
                        'mouseLRN')

        if(whichOne %in% names(alreadyFound) & !toy)
        {
            answer = alreadyFound[[whichOne]]
            return(answer)
        }

        if(whichOne %in% names(toysFound) & toy)
        {
            answer = toysFound[[whichOne]]
            return(answer)
        }
   
        
        if(whichOne %in% packageVars)
        {
            if(whichOne == 'exSeuratObj')
            {
                data(exSeuratObj,envir=environment())
                answer = exSeuratObj
                if(toy)
                {
                    data(seuratGenes,envir=environment())
                    data(seuratCells,envir=environment())
                    answer = answer[seuratGenes,seuratCells]
                }
            }

            if(whichOne == 'smallXenium')
            {
                data(smallXenium,envir=environment())
                answer = smallXenium
                if(toy)
                {
                    data(xeniumCells,envir=environment())
                    answer = answer[,xeniumCells]
                }
            }

            if(whichOne == 'ligandReceptorResults')
            {
                data(ligandReceptorResults,envir=environment())
                answer = ligandReceptorResults
            }

            if(whichOne == 'moransI')
            {
                data(moransI,envir=environment())
                answer = moransI
            }

            if(whichOne == 'moransILigandReceptor')
            {
                data(moransILigandReceptor,envir=environment())
                answer = moransILigandReceptor
            }

            if(whichOne == 'humanLRN')
            {
                data(humanLRN,envir=environment())
                answer = humanLRN
            }

            if(whichOne == 'mouseLRN')
            {
                data(mouseLRN,envir=environment())
                answer = mouseLRN
            }
        }
            

             
        if(whichOne == 'list')
            return(names(alreadyFound))
        
        if(! whichOne %in% exampleObjects())
            stop(paste(whichOne,
                       'is not an available example'))
        
        ## ####################################################
        ## Otherwise, compute:

        
        
        if(whichOne == 'STranspose')
        {
            exSeuratObj = getExample('exSeuratObj',toy)
            answer = transposeObject(exSeuratObj)
        }
        
        if(whichOne == 'S_sce')
        {
            exSeuratObj = getExample('exSeuratObj',toy)
            answer = SeuratToSCE(exSeuratObj)
        }
        
        if(whichOne == 'STranspose_sce')
        {
            STranspose = getExample('STranspose',toy)
            answer = SeuratToSCE(STranspose)
        }
        
        if(whichOne == 'delaunayNeighbours')
        {
            centroids = getExample('centroids',toy)
            delaunayNeighbours = computeNeighboursDelaunay(centroids)
            answer = delaunayNeighbours
        }
        
        if(whichOne == 'NBHDByCTSeuratExtended')
        {
            NBHDByCTMatrixExtended = getExample('NBHDByCTMatrixExtended',toy)
            answer = computeNBHDVsCTObject(NBHDByCTMatrixExtended,
                                           verbose=FALSE)
        }

        if(whichOne == 'NBHDByCTSingleCellExtended_sce')
        {
            NBHDByCTSeuratExtended = getExample('NBHDByCTSeuratExtended',toy)
            answer = SeuratToSCE(NBHDByCTSeuratExtended)
        }
        
        if(whichOne == 'CTByNBHDSeuratExtended')
        {
            ## Do we use this anywhere?
        }
        
        if(whichOne == 'edgeSeurat')
        {
            ligandReceptorResults = getExample('ligandReceptorResults',toy)
            centroids = getExample('centroids',toy)
            answer = computeEdgeObject(ligandReceptorResults, centroids)
        }
        
        if(whichOne == 'centroids')
        {
            smallXenium = getExample('smallXenium',toy)
            centroids = GetTissueCoordinates(smallXenium)
            rownames(centroids) = centroids$cell
            answer = centroids
        }
        
        if(whichOne == 'clusters')
        {
            smallXenium = getExample('smallXenium',toy)
            answer = smallXenium@active.ident
        }
        
        if(whichOne == 'extendedNeighbours')
        {
            extendedNeighboursList = getExample('extendedNeighboursList',toy)
            answer = collapseExtendedNBHDs(extendedNeighboursList, 4)
        }
        
        if(whichOne == 'extendedNeighboursList')
        {
            delaunayNeighbours = getExample('delaunayNeighbours',toy)
            answer = getExtendedNBHDs(delaunayNeighbours,4)
        }
        
        if(whichOne == 'NBHDByCTSeurat')
        {
            NBHDByCTMatrix = getExample('NBHDByCTMatrix',toy)
            answer = computeNBHDVsCTObject(NBHDByCTMatrix,verbose=FALSE)
        }
        
        if(whichOne == 'NN')
        {
            STranspose = getExample('STranspose',toy)
            answer = getNearestNeighbourLists(STranspose)
        }
        
        if(whichOne == 'CTByNBHDSeurat')
        {
            NBHDByCTMatrix = getExample('NBHDByCTMatrix',toy)
            answer =
                computeNBHDVsCTObject(t(NBHDByCTMatrix), npcs = 10, 
                                      transpose = TRUE, resolution = 1,
                                      n.neighbors = 5,
                                      verbose=FALSE)
        }
        
        if(whichOne == 'edgeNeighbours')
        {
            delaunayNeighbours = getExample('delaunayNeighbours',toy)
            answer =  computeEdgeGraph(delaunayNeighbours)
        }
        
        if(whichOne == 'colours')
        {
            clusters = getExample('clusters')
            clusterNames = levels(clusters) 
            colours = c('#5A5156',
                        '#E4E1E3',
                        '#F6222E',
                        '#FE00FA',
                        '#16FF32',
                        '#3283FE',
                        '#FEAF16',
                        '#B00068',
                        '#1CFFCE',
                        '#90AD1C',
                        '#2ED9FF',
                        '#DEA0FD',
                        '#AA0DFE',
                        '#F8A19F',
                        '#325A9B',
                        '#C4451C')
            names(colours) = clusterNames
            answer = colours
        }

        if(whichOne == 'hallmark')
        {
            h = msigdbr(species = "mouse", category = "H")
            N = unique(h$gs_name)
            
            answer = list()
            for(ell in N)
                answer[[ell]] = h$gene_symbol[h$gs_name == ell]
        }
        
        if(whichOne == 'euclideanNeighbours')
        {
            centroids = getExample('centroids',toy)
            answer =  computeNeighboursEuclidean(centroids,threshold=20)
        }
        
        if(whichOne == 'NBHDByCTMatrixExtended')
        {
            extendedNeighbours = getExample('extendedNeighbours',toy)
            clusters = getExample('clusters',toy)
            answer =  computeNBHDByCTMatrix(extendedNeighbours, clusters)
        }
        
        if(whichOne == 'NBHDByCTMatrix')
        {
            delaunayNeighbours = getExample('delaunayNeighbours',toy)
            clusters = getExample('clusters',toy)
            answer = computeNBHDByCTMatrix(delaunayNeighbours, clusters) 
        }
        
        if(whichOne == 'shorterHallmark')
        {
            hallmark = getExample('hallmark',toy)
            answer = hallmark[seq_len(10)]
        }
        
        if(whichOne == 'clusterDF')
        {
            ## I don't think we use this.
        }
        
        if(whichOne == 'cellTypesPerCellTypeMatrixExtended')
        {
            NBHDByCTMatrixExtended = getExample('NBHDByCTMatrixExtended',toy)
            clusters = getExample('clusters')
            answer = computeCellTypesPerCellTypeMatrix(NBHDByCTMatrixExtended,
                                                       clusters)
        }
        
        if(whichOne == 'cellTypesPerCellTypeMatrix')
        {
            NBHDByCTMatrix = getExample('NBHDByCTMatrix',toy)
            clusters = getExample('clusters',toy)
            answer = computeCellTypesPerCellTypeMatrix(NBHDByCTMatrix,clusters)
        }
        
        if(whichOne == 'cellTypesPerCellTypePValues')
        {
            delaunayNeighbours = getExample('delaunayNeighbours',toy)
            clusters = getExample('clusters',toy)
            answer = computeNeighbourEnrichment(delaunayNeighbours, 
                                                clusters, verbose = FALSE)
        }

        if(whichOne == 'averageExpMatrix')
        {
            exSeuratObj = getExample('exSeuratObj',toy)
            STranspose = getExample('STranspose',toy)
            answer = getAverageExpressionMatrix(exSeuratObj,STranspose,layer='data')
        }

        if(whichOne == 'X_sce')
        {
            smallXenium = getExample('smallXenium',toy)
            answer = SeuratToSCE(smallXenium,spatial=TRUE)
        }
                
        ## Save and return:
        if(! toy)
            alreadyFound[[whichOne]] <<- answer
        if(toy)
            toysFound[[whichOne]] <<- answer

        return(answer)
    }

    return(getExample)
}


## ####################################################
#' This returns the names of available example objects.
#'
#' @return A character vector of the names of available
#' example data objects
#' @export
#' @examples
#' availableObjects = exampleObjects()
exampleObjects = function()
{
    objects = c('exSeuratObj',
                'STranspose',
                'STranspose_sce',
                'S_sce',
                'NBHDByCTSeuratExtended',
                'NBHDByCTSingleCellExtended_sce',                
                'edgeSeurat',
                'smallXenium',
                'extendedNeighbours',
                'extendedNeighboursList',
                'NBHDByCTSeurat',
                'NN',
                'CTByNBHDSeurat',
                'edgeNeighbours',
                'ligandReceptorResults',
                'centroids',
                'delaunayNeighbours',
                'mouseLRN',
                'euclideanNeighbours',
                'NBHDByCTMatrixExtended',
                'hallmark',
                'humanLRN',
                'NBHDByCTMatrix',
                'clusters',
                'shorterHallmark',
                'averageExpMatrix',
                'moransI',
                'cellTypesPerCellTypeMatrixExtended',
                'cellTypesPerCellTypeMatrix',
                'cellTypesPerCellTypePValues',
                'moransILigandReceptor',
                'colours',
                'X_sce')

    return(objects)
}

---
title: "CatsCradle"
author: "Anna Laddach and Michael Shapiro"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{CatsCradle}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE, warning = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    fig.dim=c(6,6),
    comment = "#>"
)
```
`r BiocStyle::Biocpkg("BiocStyle")`

![](CatsCradleLogo.png){width=2in}


## Introduction

Here we describe the functionality in this package concerned with
analysing the clustering of genes in single cell RNASeq data.

A typical Seurat single cell analysis starts with an expression matrix
$M$ where the rows represent genes and the columns represent
individual cells.  After normalisation one uses dimension reduction -
PCA, UMAP, tSNE - to produce lower dimensional representations of the
data for the cells and the Louvain algorithm to cluster cells with
similar expression patterns.

CatsCradle operates based on a simple observation: by transposing the
matrix $M$, we can use the same methods to produce lower-dimensional
representations of the genes and cluster the genes into groups that
show similar patterns of expression.

```{r [CC1], message=FALSE}
library(CatsCradle,quietly=TRUE)
getExample = make.getExample()
exSeuratObj = getExample('exSeuratObj')
STranspose = transposeObject(exSeuratObj)
```
This function transposes the expression matrix and carries out the
basic functions, FindVariableFeatures(), ScaleData(), RunPCA(),
RunUMAP(), FindNeighbors(), and FindClusters().

## Exploring CatsCradle

After transposing the usual Seurat object, the genes are now the
columns (samples) and the individual cells are the rows (features).
The Louvain clustering of the genes is now encoded in
STranspose$seurat_clusters. As with the cells, we can observe these
clusters on UMAP, tSNE or PCA.

```{r [CC2], message=FALSE}
library(Seurat,quietly=TRUE)
library(ggplot2,quietly=TRUE)
DimPlot(exSeuratObj,cols='polychrome') + ggtitle('Cell clusters on UMAP')
```

```{r [CC3]}
DimPlot(STranspose,cols='polychrome') + ggtitle('Gene clusters on UMAP')
```


We have never seen a use case in which there was a reason to query the
identities of the individual cells in a UMAP plot.  However, this
changes with a gene UMAP as each gene has a distinct (and interesting)
identity.  We recommend using plotly to produce a browseable version
of the gene UMAP.  This allows one to hover over the individual points
and discover the genes in each cluster. Typical code might be

```{r [CC4], eval = FALSE}
library(plotly,quietly=TRUE)
umap = FetchData(STranspose,c('UMAP_1','UMAP_2','seurat_clusters'))
umap$gene = colnames(STranspose)
plot = ggplot(umap,aes(x=UMAP_1,y=UMAP_2,color=seurat_clusters,label=gene) +
       geom_point()
browseable = ggplotly(plot)
print(browseable)
htmlwidgets::saveWidget(as_widget(browseable),'genesOnUMAP.html')
```

The question arises as to how to annotate the gene clusters.  Assuming
you have working annotations for the cell clusters it can be useful to
examine which cells each of the gene clusters is expressed in.  Here
we give a heatmap of average expression of each gene cluster (columns)
across each cell cluster (rows).

```{r [CC5], message=FALSE}
library(pheatmap,quietly=TRUE)
averageExpMatrix = getAverageExpressionMatrix(exSeuratObj,STranspose,layer='data')
averageExpMatrix = tagRowAndColNames(averageExpMatrix,
                                     ccTag='cellClusters_',
                                     gcTag='geneClusters_')
pheatmap(averageExpMatrix,
      treeheight_row=0,
      treeheight_col=0,
      fontsize_row=8,
      fontsize_col=8,
      cellheight=10,
      cellwidth=10,
      main='Cell clusters vs. Gene clusters')
```



Another way of seeing the relationship between cell clusters and gene
clusters is in a Sankey graph.  This is a bi-partite graph whose
vertices are the cell clusters and the gene clusters.  The edges of
the graph display mean expression as the width of the edge.  One can
either display all edges with edge weight (width) displaying absolute
value and colour distinguishing up- and down-regulation of expression
or display separate Sankey graphs for the up- and down-regulated gene
sets.  It is these bi-partite graphs that contribute the name
CatsCradle. Here the up-regulated gene sets are shown in cyan, the
down-regulated sets in pink.  The image was produced with the
following code.

```{r [CC6], eval = FALSE}
catsCradle = sankeyFromMatrix(averageExpMatrix,
                              disambiguation=c('cells_','genes_'),
                              plus='cyan',minus='pink',
                              height=800)
print(catsCradle)
```
The print command opens this in a browser.  This allows one to query
the individual vertices and edges. This can be saved with the
saveWidget command as above.


## Biologically relevant gene sets on UMAP

Biologically relevant gene sets often cluster on CatsCradle gene
UMAPs.  Here we see a UMAP plot showing the gene clusters (by color),
over-printed with the HALLMARK_G2M_CHECKPOINT that appear in
STranspose in black.  We see that these are strongly associated with
gene cluster 8, but also show "satellite clusters" including an
association with cluster 4 and with the border between clusters 0 and
3.  In our experience, proliferation associated gene sets are among
the most strongly clustered.


```{r [CC7]}
hallmark = getExample('hallmark')
h = 'HALLMARK_G2M_CHECKPOINT'
umap = FetchData(STranspose,c('umap_1','umap_2'))
idx = colnames(STranspose) %in% hallmark[[h]]
g = DimPlot(STranspose,cols='polychrome') +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2),color='black',size=2.7) +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2),color='green') +
    ggtitle(paste(h,'\non gene clusters'))
print(g)
```

## Determining statistical significance of clustering

Given a set of points, $S$ and a non-empty proper subset $X \subset S$
we would like to determine the statistical significance of the degree
to which $X$ is clustered.  To compute this we ask the opposite
question: what would we see if $X$ were randomly chosen?  In this
case we expect to see $X$ broadly spread out across $S$, i.e., most of
$S$ should be close to some point of $X$.  In particular we expect the
median distance from the complement $S \setminus X$ to $X$ to be low.
Of course, how low, depends on the size of $X$.  Conversely, if the
points of $X$ cluster together, we expect much of $S \setminus X$ to
be further from $X$, at least compared to other sets of the same
size.  We use a distance function inspired by Hausdorf distance. Give
a set $X$, for each $s_k \in S \setminus X$, we take $d_k$ to be the
distance from $s_k$ to the nearest point of $X$. We then take the __median
complement distance__ to be the median of the values $d_k$.  Comparing
this median complement distance for $X \subset S$ with those for
randomly chosen sets $X_i \subset S$ allows us to assess the
clustering of $X$.  (These $X_i$ are chosen to be the same size as
$X$.)
```{r [CC8]}
g2mGenes = intersect(colnames(STranspose),
                     hallmark[['HALLMARK_G2M_CHECKPOINT']])
stats = getObjectSubsetClusteringStatistics(STranspose,
                                      g2mGenes,
                                      numTrials=1000)
```				      

This uses UMAP as the default reduction and returns the median
complement distance to the Hallmark G2M genes in STranspose, and the
distances for 1000 randomly chosen sets of 56 genes.  It computes a
p-value based on the rank of the actual distance among the random
distances and its z-score.  Here we report a p-value of
0.001. However, as can be seen from the figure, the actual p-value is
lower by many orders of magnitude.

```{r [CC9]}
statsPlot = ggplot(data.frame(medianComplementDistance=stats$randomSubsetDistance),
                  aes(x=medianComplementDistance)) +
    geom_histogram(bins=50) +
    geom_vline(xintercept=stats$subsetDistance,color='red') +
    ggtitle('Hallmark G2M real and random median complement distance')
print(statsPlot)
```

Here we have shown the statistics for one of the gene sets that is
most tightly clustered on UMAP.  However, of the 50 Hallmark gene
sets 31 cluster with a p-value better than 0.05.

```{r [CC10]}
df = read.table('hallmarkPValues.txt',header=TRUE,sep='\t')
g = ggplot(df,aes(x=logPValue)) +
    geom_histogram() +
    geom_vline(xintercept=-log10(0.05)) +
    ggtitle('Hallmark gene set p-values')
print(g)
```

We have referred to median complement distance p-values as detecting
clustering, but it might be more accurate to describe it as curdling.
That is, it does not detect the tendency of the subset to co-locate in
one region of the superset.  Rather, it detects the tendency of points
in the subset to stick close to each other.  Accordingly, it makes
sense to look at clustering within the subset.  To do this, we analyse
the Delaunay triangulation of the subset within the gene UMAP.  We
then classify the edges in this triangulation into shorter and longer
edges which are assumed to be intra-subcluster and inter-subcluster
edges.  We discard the longer edges and look for the components of the
resulting graph.

```{r [CC10.5]}
h = "HALLMARK_ALLOGRAFT_REJECTION"
theSubset = hallmark[[h]]
theSubset = intersect(theSubset,colnames(STranspose))
alpha = .5
clusters = getSubsetComponents(STranspose,theSubset,alpha)
umap = FetchData(STranspose,c('umap_1','umap_2'))
umap$gene = rownames(umap)
numClusters = length(clusters)
umap$component = 0
for(i in 1:length(clusters))
    umap[clusters[[i]],'component'] = i
umap$component = factor(umap$component)
title = paste(h,'alpha =',alpha)
idx = umap$component != 0
g = ggplot() +
    geom_point(data=umap,aes(x=umap_1,y=umap_2),color='grey') +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2),color='black',size=3.5) +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2,color=component),size=3) +
    ggtitle(title)
print(g)
```

## Gene z-scores on gene UMAP

Here we show that gene UMAP can reveal co-location of the genes that
are up-regulated in each cell cluster.  To do this, we start by
finding the z-score for expression of each gene computed across all
cells.  For each gene in the gene Seurat object, and each cell cluster
in the cell Seurat object, we then compute the mean z-score for that
gene in the cells of that cluster.  Plot this on the gene umap reveals
localised patterns of gene expression.  Creating a browseable figure
allows for easy querying of the up-regulated genes.

```{r [CC11]}
meanZDF	 = meanZPerClusterOnUMAP(exSeuratObj,
                                 STranspose,
	                         'shortName')


h = ggplot(meanZDF,aes(x=umap_1,y=umap_2,color=EntericGliaCells)) +
    geom_point() +
    scale_color_gradient(low='green',high='red') +
    ggtitle('Mean z-score, Enteric glia on gene UMAP')
print(h)
```

P-values for the spatial autocorrelation of these values can be found
using runMoransI().  

In the case where there are multiple clusters of similar cells,
plotting the difference in mean z-score can illuminate the differences
in the sub-clusters.  Here we plot the difference in z-score between
TCells3 and TCells1.

```{r [CC12]}
TDiff = meanZDF[,1:3]
TDiff$TDiff = meanZDF$TCells3 - meanZDF$TCells1
k = ggplot(TDiff,aes(x=umap_1,y=umap_2,color=TDiff)) +
    geom_point() +
    scale_color_gradient(low='green',high='red') +
    ggtitle('Mean z-score, TCells3 - TCells1')
print(k)
```


## Nearby genes

We have seen that genes with similar annotation have a tendency to
cluster.  This suggests that nearby genes may have similar functions.
To this end, we have supplied a function which finds nearby
genes. This can be done geometrically using either PCA, UMAP or tSNE
as the embedding or combinatorially using the nearest neighbor graph.
The function returns a named vector whose values are the distances
from the gene set and whose names are the genes. Here we find those
genes which are within radius 0.2 in UMAP coordinates of genes in the
HALLMARK_INTERFERON_ALPHA_RESPONSE gene set.  As you can see, the
combinatorial radius of a gene set can grow quite quickly and need not
have a close relation to UMAP distance. This function will also return
weighted combinatorial distance from a single gene where distance is
the reciprocal of edge weight.

```{r [CC13]}
geneSet = intersect(colnames(STranspose),
                    hallmark[['HALLMARK_INTERFERON_ALPHA_RESPONSE']])
geometricallyNearbyGenes = getNearbyGenes(STranspose,geneSet,radius=0.2,metric='umap')
theGeometricGenesThemselves = names(geometricallyNearbyGenes)
combinatoriallyNearbyGenes = getNearbyGenes(STranspose,geneSet,radius=1,metric='NN')
theCombinatoricGenesThemselves = names(combinatoriallyNearbyGenes)
df = FetchData(STranspose,c('umap_1','umap_2'))
df$gene = colnames(STranspose)
geneSetIdx = df$gene %in% geneSet
nearbyIdx = df$gene %in% theGeometricGenesThemselves
g = ggplot() +
    geom_point(data=df,aes(x=umap_1,y=umap_2),color='gray') +
    geom_point(data=df[geneSetIdx,],aes(x=umap_1,y=umap_2),color='blue') +
    geom_point(data=df[nearbyIdx,],aes(x=umap_1,y=umap_2),color='red') +
    ggtitle(paste0('Genes within geometric radius 0.2 (red) of \n',
                     'HALLMARK_INTERFERON_ALPHA_RESPONSE (blue)'))
print(g)
```


## Predicting gene function

Given a particular gene, it is interesting to look at the annotations
of nearby genes in the gene Seurat object.  In this context,
"annotations" might mean GO or Hallmark, "nearby" might mean in terms
of UMAP or PCA coordinates or in the nearest neighbour graph in the
gene Seurat object.  Here we will look at Hallmark annotations and
UMAP coordinates.

Gene annotation lists give lists of genes.  For each gene, we can
collect the annotations is belongs to.  The function annotateGenesByGeneSet()
inverts the gene sets to give a list of the sets each gene belongs
to. 

```{r [CC14]}
annotatedGenes = annotateGenesByGeneSet(hallmark)
names(annotatedGenes[['Myc']])
```
We see that Myc belongs to ten hallmark sets.  We can also give the
annotations of a gene as a vector.

```{r [CC15]}
 Myc = annotateGeneAsVector('Myc',hallmark)
 MycNormalised = annotateGeneAsVector('Myc',hallmark,TRUE)
```
These are named vectors whose names are the Hallmark sets. Myc is a
 0-1 vector indicating membership.  MycNormalised gives these values
 normalised by the size of the sets in question.  This is appropriate
 when we wish to weight the contributions of nearby genes as they are
 more likely to belong to larger gene sets.

```{r [CC16]}
predicted = predictAnnotation('Myc',hallmark,STranspose,radius=.5)
predicted$Myc[1:10]
```

predictAnnotation() accepts a vector of genes and returns a list of
prediction vectors.  Here we have given it a single gene, Myc, and we
see that it has predicted one of the gene sets listed for Myc.

Predictions made in this manner perform well above chance. Of the 2000
genes in STranspose, 922 have the property that they appear in at
least one of the Hallmark gene sets and at least one of their nearby
genes appears in a Hallmark gene set.  This means that we are able to
compare their actual annotation vectors with their predicted
annotation vectors.  After normalising both to unit vectors, we can
take their dot prodcts as a measure of their closeness.  Comparing the
actual dot products for these 922 genes to those produced by 1000
randomised predictions produces the following comparison.

![comparing real and randomised predictions](actualVsRandomisedPredictions.jpg){width=6in}


```{r [CC17]}
sessionInfo()
```

---
title: "CatsCradle Quick Start"
author: "Anna Laddach and Michael Shapiro"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{CatsCradle Quick Start}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE, warning = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    fig.dim = c(6,6),
    comment = "#>"
)
```
`r BiocStyle::Biocpkg("BiocStyle")`

![](CatsCradleLogo.png){width=2in}


## Introduction

CatsCradle provides two types of functionality for analysing single
cell data.  It provides tools for the clustering and annotation of
genes and it provides extensive tools for analysing spatial
transcriptomics data.

## Clustering and annotation of genes

A typical analysis of scRNA-seq data using Seurat involves dimensionality reduction 
(PCA, UMAP, tSNE) and the clustering of cells using the Louvain algorithm. All of
this is done based on the similarities and differences in the genes
cells express.  Here we see a UMAP where the points are cells, coloured by
their seurat_clusters.  

```{r [QS1], message=FALSE}
library(Seurat,quietly=TRUE)
library(CatsCradle,quietly=TRUE)
data(exSeuratObj)
DimPlot(exSeuratObj,cols='polychrome')
```

By transposing the expression matrix, we can use the same technology
to produce a Seurat object where the samples are the genes and the features are 
cells.  Genes now have their own UMAP and their own clusters.  This is all done 
on the basis of the similarities and differences in the cells they are
expressed in.

```{r [QS2]}
getExample = make.getExample()
STranspose = getExample('STranspose')
print(STranspose)
DimPlot(STranspose,cols='polychrome')
```

### Gene clusters vs. cell clusters

There are a number of ways of investigating the relationship between
gene clusters and cell clusters.  One is by computing the average
expression of each gene cluster in each cell cluster; this can be
displayed as a heat map or a Sankey graph - the cat's cradle of our
CatsCradle. (See getAverageExpressionMatrix() and sankeyFromMatrix()
in the CatsCradle vignette CatsCradle.

### Spatial co-location of genes on gene UMAP

Gene sets such as Hallmark tend to localise on gene UMAP, though they
are not necessarily confined to specific gene clusters.  Here we show
HALLMARK_OXIDATIVE_PHOSPHORYLATION (subset to the genes in our Seurat
object) superimposed in green and black on the gene cluster UMAP.



```{r [QS3], message=FALSE}
library(ggplot2,quietly=TRUE)
hallmark = getExample('hallmark')
h = 'HALLMARK_OXIDATIVE_PHOSPHORYLATION'
umap = FetchData(STranspose,c('umap_1','umap_2'))
idx = colnames(STranspose) %in% hallmark[[h]]
g = DimPlot(STranspose,cols='polychrome') +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2),color='black',size=2.7) +
    geom_point(data=umap[idx,],aes(x=umap_1,y=umap_2),color='green') +
    ggtitle(paste(h,'\non gene clusters'))
print(g)
pValue = getObjectSubsetClusteringPValue(STranspose,idx)
pValue
```

The p-value here is limited by the default number of randomisation
trials in getSubsetClusteringPValue().  Of the 50 Hallmark gene sets,
31 have clustering p-values less than 0.05.

The computation of these p-values is ultimately carried out by
medianComplementPValue().  We wish to determine whether a subset _X_
of a set of points _S_ is randomly scattered across _S_ or is somehow
localised.  To do this we consider the complement _C_ of _X_ in _S_.
If _X_ is "spread out" randomly across _S_, every point of _C_ will be
close to some point in _X_.  Accordingly, the median distance from
points in _C_ to their nearest point in _X_ will be small.  If on the
contrary, _X_ is localised, many points in _C_ will be distant from
_X_, thereby producing a larger median complement distance.  We
produce a p-value by comparing the median complement distance for _X_
to those for randomised sets of the same size as _X_.

### Gene annotation

This allows one to predict functions of a given gene by looking at
annotations of its neighbouring genes.  If a gene has its own
annotation and also has neighbours which have annotations, we can
compare the actual annotation to the predicted annotation. These predictions 
perform well above chance as described in the section Predicting gene function 
of the CatsCradle vignette CatsCradle.

## Analysis of spatial transcriptomic data

CatsCradle provides extensive tools for the analysis of spatial
transcriptomics data.  Here we see cell type plotted on the cell
centroids of our example data.

```{r [QS4]}
smallXenium = getExample('smallXenium')
ImageDimPlot(smallXenium,cols='polychrome')
```

With Moran's I, we can see spatial autocorrelation of gene expression
and compute p-values for this.  See CatsCradle spatial vignette
CatsCradleSpatial.


### Neighbourhoods

The key concept in our analysis of this kind of data is the
_neighbourhood_. In general, a neighbourhood is a contiguous set of
cells.  In all of our use cases, each neighbourhood is set of cells
_around a given cell_.  Accordingly, a neighbourhood can be referred
to by the name of the cell at its centre.

The simplest type of neighbourhood arises from Delaunay triangulation
where each neighbourhood consists of a cell and its immediate
neighbours.  This is an undirected graph in which each cell has an
edge connecting it to each of these neighbours.

```{r [QS5]}
delaunayNeighbours = getExample('delaunayNeighbours')
head(delaunayNeighbours,10)
```

We can also make extended neighbourhoods, e.g., by finding the
neighbours of each cell, their neighbours, and their neighbours.
getExtendedNBHDs() produces a list characterising these neighbours by
their combinatorial radius and collapseExtendedNBHDs() collapses
these so that all the extended neighbours of each cell are now treated
as neighbours.

In addition, neighbourhoods can be found on the basis of Euclidean
distance.

For each cell type A, we can ask "What types of cells are found around
cells of type A?"  We can display the answer in a directed graph. Here
we are using an extended neighbourhood.  A fat arrow from type A to
type B indicates that neighbourhoods around cells of type A have lots
of cells of type B. Here we only display an arrow from A to B when
cells of type B compose at least 5 percent of the cells in
neighbourhoods around type A.

```{r [QS6]}
NBHDByCTMatrixExtended = getExample('NBHDByCTMatrixExtended')
clusters = getExample('clusters')
colours = getExample('colours')
cellTypesPerCellTypeMatrixExtended = computeCellTypesPerCellTypeMatrix(NBHDByCTMatrixExtended,clusters)

cellTypesPerCellTypeGraphFromCellMatrix(cellTypesPerCellTypeMatrixExtended, minWeight = 0.05, colours = colours)
```

We can also test for statistical significance for the enrichment of a
given cell type in the neighbourhoods around another cell type 
(see CatsCradleSpatial).

### Neighourhood Seurat objects.

Neighbourhoods naturally want to be encoded into Seurat objects.
There are two ways to do this.

#### Neighbourhoods and cell types

Just as cells express genes, with poetic license we can say that
neighbourhoods "express" cell types.  A cell types by neighbourhoods
matrix gives the counts of cells of each type in each neighbourhood.
This can be taken as the counts matrix for a Seurat object.  We can
then use the Louvain algorithm to cluster neighbourhoods into
neighbourhood types.  If we like, we can display these on a
neighbourhood UMAP.

Here we attach these neighbourhood_clusters from a neighbourhood
Seurat object (created by computeNBHDVsCTObject()) to our original
spatial object and visualize the neighbourhood clusters on the tissue
coordinates. (Again, we are using extended neighbourhoods.)

```{r [QS7]}
NBHDByCTSeuratExtended = getExample('NBHDByCTSeuratExtended')
smallXenium$NBHDClusterExtended= 
  NBHDByCTSeuratExtended@active.ident
ImageDimPlot(smallXenium, group.by = c("NBHDClusterExtended"), 
             size = 1, cols = "polychrome")
```

### Aggregate gene expression

As well as expressing cell types, neighbourhoods also express genes.
We can take the gene expression profile of a neighbourhood to be the
total aggregate expression of each gene across all cells in the
neighbourhood.  This produces a genes by neighbourhoods count
matrix. The function aggregateGeneExpression() takes as
arguments an underlying Seurat object and a set of neighbourhoods and
produces a Seurat object where the rows are the genes of the
underlying Seurat object and the columns are the neighbourhoods.
Since each neighbourhood corresponds to a cell in our usage, this is
formally indistinguishable from a normal Seurat object.  However, to
avoid confusion, we have labelled the clusters as aggregation_clusters
rather than the standard seurat_clusters.

This enables all the usual analyses: dimension reduction and Louvain
clustering (here giving another clustering of the neighbourhoods). In
addition, we can find the marker genes which characterise the transcriptomic
differences between the neighbourhood types.

```{r [QS8]}
extendedNeighbours = getExample('extendedNeighbours')
agg = aggregateGeneExpression(smallXenium,extendedNeighbours,
                                    verbose=FALSE)
smallXenium$aggregateNBHDClusters = agg@active.ident
ImageDimPlot(smallXenium,group.by='aggregateNBHDClusters',cols='polychrome',
             size=1)
```

Notice that since our neighbourhoods are all indexed by cells, we can
compare the different methods of clustering neighbourhoods.  Here we
give a heatmap comparing the clustering based on cell types in our
extended neighbourhoods to that based on aggregate gene expression.
We see, for example, that aggregation clusters 7, 13, 14, 16 and 17
are subsumed into cell type neighbourhood cluster 0.  More generally,
we see that clustering based on aggregate gene expression tends to
subdivide that based on the cell types in each neighbourhood.

```{r [QS9], message=FALSE}
library(pheatmap)
tab = table(smallXenium@meta.data[,c("aggregateNBHDClusters",
                                     "NBHDClusterExtended" )])
M = matrix(as.numeric(tab),nrow=nrow(tab))
rownames(M) = paste('aggregation_cluster',rownames(tab))
colnames(M) = paste('nbhd_cluster',colnames(tab))
for(i in 1:nrow(M)) M[i,] = M[i,] / sum(M[i,])
pheatmap(M)
```

Another way of comparing clusterings is via the adjusted Rand
index. This compares two classifications of the same set of objects.
It can vary between -1 and 1.  It takes the value 1 when these
classifications denote the same subsets and 0 when they agree at a
level no better than chance.

```{r [QS10], message=FALSE}
library(fossil,quietly=TRUE)
adjustedRandIndex = adj.rand.index(agg@active.ident,
                                   NBHDByCTSeuratExtended@active.ident)
adjustedRandIndex
```

## Ligand-receptor analysis

The Delaunay triangulation gives us an opportunity to look at the
ligand-receptor interactions between neighbouring cells.  We use the
Nichenetr ligand-receptor pairs to discover the pairs in our gene
panel.  Nichnetr lists over eleven thousand ligand-receptor pairs.  Of
these 28 appear in our example panel.

The main function for carrying out ligand-receptor analysis is
performLigandReceptorAnalysis().  This produces a list with
multiple entries.  These include a data frame interactionsOnEdges.
Here the first two columns are nodeA and nodeB which give the edge in
question and next two columns give their corresponding clusters. The remaining 
columns are logicals, each corresponding to a
particular ligand-receptor pair and indicating whether that pair is
present on that edge.  Notice that while the neighbour-neighbour
relationship in the Delaunay triangulation is undirected, the
ligand-receptor relationship is directed.  Accordingly, in
interactionsOnEdges, a given edge appears as both the pair cell A -
cell B and as the pair cell B - cell A.  

```{r [QS11]}
ligandReceptorResults = getExample('ligandReceptorResults')
ligandReceptorResults$interactionsOnEdges[1:10,1:10]
```


Other fields are downstream of this data frame.  For example, pValue
tells whether a given ligand-receptor pair is statistically significantly
enriched in the interactions between two specific cell types.  For
further details see the section Ligand receptor analysis in the Cats
Cradle Spatial vignette CatsCradleSpatial.

## CatsCradle, Seurat objects, SingleCellExperiment objects and SpatialExperiment objects 

CatsCradle provides rudimentary support for the Bioconductor classes
SingleCellExperiment and its derived class SpatialExperiment.

Every function which accepts a Seurat object as input can also accept
a SingleCellExperiment in its place.  Any function which returns a
Seurat object can also return a SingleCellExperiment, in one case, a
SpatialExperiment, through the use of the parameter returnType.  In
each case, the internals of the functions are manipulating Seurat
objects.  Conversion to and from these is performed upon entry to and
return from the function and these coversions are rather lightweight
in nature.  They have slightly more functionality than as.Seurat and
as.SingleCellExperiment.  In addition to invoking these functions,
they carry over the nearest neighbour graphs, and in one case carry
over the tissue coordinates from a spatial Seurat object to a
SpatialExperiment object.  Because of the shallow nature of these
conversions, other information may be lost.



```{r [QS12]}
sessionInfo()
```

---
title: "CatsCradle Spatial Vignette"
author: "Anna Laddach and Michael Shapiro"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{CatsCradle Spatial Vignette}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r  [Sp1] setup, include = FALSE, warning = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    fig.dim = c(6,6),
    comment = "#>"
)
```
`r BiocStyle::Biocpkg("BiocStyle")`

![](CatsCradleLogo.png){width=2in}


## Introduction

Here we describe the tools that CatsCradle offers for exploiting the
spatial relationships in spatial transcriptomics data.

 We will
be using a subset  of a Xenium data set that
profiles the mouse hippocampus available from 10x genomics
(https://www.10xgenomics.com/datasets/fresh-frozen-mouse-brain-for-xenium-explorer-demo-1-standard)

Here we visualise this subset coloured by cell type.  Here cell
clusters (Louvain cluster) have been assigned cell type identities
using RCTD along with a reference dataset
(https://www.dropbox.com/s/cuowvm4vrf65pvq/allen_cortex.rds?dl=1)
followed by minimal manual curation. Please note these assignments are
not definitive and are for illustratory purporses only.

```{r [Sp2], message=FALSE}
library(Seurat,quietly=TRUE)
library(CatsCradle,quietly=TRUE)
getExample = make.getExample()
smallXenium = getExample('smallXenium')
ImageDimPlot(smallXenium, cols = "polychrome", size = 1)
```

We will want to answer questions like the following:

Is the expression of a particular gene localised?  Who are the
immediate neighbours of a given cell?  Do certain types of cells tend
to co-localise?  Is the spatial association of two cell types, (say,
glia and macrophages) statistically significant?  For any pair of
neighbouring cells, are they engaged in ligand-repceptor interactions?
Can we classify different types of tissue neighbourhoods?  Are
different neighbourhood types characterised by the cell types found in
them?  The genes expressed in them?  The ligand-receptor interactions
taking place in them?


### Neighbourhoods


A key concept here is that of a _neighbourhood_.  A neighbourhood is a
spatially contiguous set of cells.  In every case we will consider, a
neighbourhood is a set of cells _centred on a particular cell_.

The simplest sort of neighbourhood consists of a cell together with
its immediate neighbours.  We compute these as a Delaunay
triangulation using the centroids of the cells:


```{r [Sp3]}
centroids = GetTissueCoordinates(smallXenium)
rownames(centroids) = centroids$cell
clusters = smallXenium@active.ident
delaunayNeighbours = computeNeighboursDelaunay(centroids)
head(delaunayNeighbours)
```

Two cells are neighbours if they appear on the same row.  The
neighbourhood of the cell 16307 consists of the following 7 cells:

```{r [Sp4]}
idx = (delaunayNeighbours$nodeA == 16307 |
       delaunayNeighbours$nodeB == 16307)
nbhd = unique(c(delaunayNeighbours$nodeA[idx],
                delaunayNeighbours$nodeB[idx]))
nbhd		
```

We can compute extended neighbourhoods by considering (say) each cell's
neighbours, their neighbours, their neighbours and their neighbours.
Here we have such an extended neighbourhood as a list and again as a
data frame with each row giving a pair of cells in a common extended
neighbourhood.  In this case we are building an extended neighbourhood
of combinatorial radius 4.

```{r [Sp5]}
extendedNeighboursList = getExtendedNBHDs(delaunayNeighbours, 4)
extendedNeighbours = collapseExtendedNBHDs(extendedNeighboursList, 4)
```

Now the extended neighbourhood of  cell 16307 consists of 92 cells:

```{r [Sp6]}
idx = (extendedNeighbours$nodeA == 16307 |
       extendedNeighbours$nodeB == 16307)
nbhd = unique(c(extendedNeighbours$nodeA[idx],
                extendedNeighbours$nodeB[idx]))
length(nbhd)		
```

We can also create neighbours based on Euclidean distance in the tissue.


```{r [Sp7]}
euclideanNeighbours = computeNeighboursEuclidean(centroids,
threshold=20)
```

We see two reasons for looking at these extended neighbourhoods.  One
is that while the Delaunay triangulation might be more appropriate for
looking at cell to cell interactions involving direct contact,
extended neighbourhoods might be more appropriate for studying
interactions based on diffusible ligands.  The second is that when we
go on to characterise _types_ of neighbourhoods, these extended
neighbourhoods exhibit less noise.


We offer two viewpoints as to what is "going on" in a neighbourhood.
One is that a neighbourhood is characterised by the cell types that
are found in it.  In this view, just as a cell expresses genes, a
neighbourhood "expresses" cell types.  A second viewpoint is that a
neighbourhood also expresses genes.  That is, for each neighbourhood,
we can compute the total gene expression across all the cells in that
neighbourhood.

In this vignette we will focus on the first viewpoint.  This is not
because we think the first viewpoint is more important or because we
necessarily expect it to be more fruitful.  It's because the second
viewpoint so directly parallels standard Seurat single cell
transcriptomics analyses that it requires less explanation.  So before
we move on to the neighbourhoods / cell types viewpoint, we give an
overview of the neighbourhoods / aggregate gene expression viewpoint.

### Neighbourhoods as characterised by gene expression

Here we create a Seurat object based on the aggregate gene expression
in each of our extended neighbourhoods and display the clustering of
the neighbourhoods (called aggregation_clusters) both on the tissue
plot and on the resulting UMAP.

```{r [Sp8]}
agg = aggregateGeneExpression(smallXenium,extendedNeighbours,
                                    verbose=FALSE)
smallXenium$aggregateNBHDClusters = agg@active.ident
ImageDimPlot(smallXenium,group.by='aggregateNBHDClusters',cols='polychrome')
```

Since neighbourhoods here are indexed by individual cells, the
neighbourhood gene aggregation Seurat object is formally identical to
a standard Seurat object.  In particular, this allows the standard
sorts of analyses including clustering of neighbourhoods into types;
dimension reduction using PCA, UMAP, tSNE; discovery of marker genes
for each neighbourhood type; plotting of aggregate gene expression on
neighbourhood UMAP; use of CatsCradle to discover novel clustering of
genes based on their expression across the different neighbourhoods.


## Neighbourhoods as characterised by cell types



### Calculation of neighbourhood celltype composition 

Given any of the spatial graphs describing neighbourhoods calculated above,
we can now calculate the cell type composition of neighbourhoods.

```{r [Sp9]}
NBHDByCTMatrix = computeNBHDByCTMatrix(delaunayNeighbours, clusters)
```

In the resulting matrix neighbourhoods are rows and cell types are columns. 
The values in the matrix indicate the number of cells of a given type within a 
neighbourhood. 

Let's do the same for our extended neighbourhoods.
```{r [Sp10]}
NBHDByCTMatrixExtended = 
  computeNBHDByCTMatrix(extendedNeighbours, clusters)
```

### Analysis of contact based interactions between cell types

We can go on to calculate a matrix which gives the fraction of contacts cells of
a given type make with cells of another cell type.
```{r [Sp11]}
cellTypesPerCellTypeMatrix = 
  computeCellTypesPerCellTypeMatrix(NBHDByCTMatrix,clusters)
```

Rows and columns both correspond to cell types, but they are playing
different roles.  For a given row (say, cell type A) the entry in
column B represents the fraction of cells in neighbourhoods of cells
of type A that are of type B.

We can then display this as a force directed graph. Here we choose only to 
display contact based interactions that constitute at least 5% of a cell type's
interactions. Of note, this graph is directed as, for example, 50% of cell type
A's interactions might be with cell type B, but only 5% of cell type B's 
interactions might be with cell type A.   
```{r [Sp12]}
colours = DiscretePalette(length(levels(clusters)), palette = "polychrome")
names(colours) = levels(clusters)

cellTypesPerCellTypeGraphFromCellMatrix(cellTypesPerCellTypeMatrix, 
                                    minWeight = 0.05, colours = colours)
```

Here arrows are directed from rows to columns. Thus, we see an arrow
from 12_Pvalb to 18_pyramidal because neighbourhoods of cells of type
12_Pvalb are composed (across the dataset) of ~15% cells of type
18_pyramidal.  We do not see an arrow from 18_pyramidal to 12_Pvalb
because neighbourhoods of cells of type 18_pyramidal have only 3%
cells of type 12_Pvalb, which falls below the chosen cutoff.

It's worth pointing out the following. The number of edges from cells of
type A to cells of type B is the same as the number of edges from
cells of type B to cells of type A.  Thus the matrix of counts of
these edges is symmetric.  However, numbers of cells of types A and B
are not necessarily equal, and this accounts for the assymmetry in the
fractions. 

```{r [Sp13], message=FALSE}
library(pheatmap,quietly=TRUE)
pheatmap(cellTypesPerCellTypeMatrix)
```

Let's do the same for our extended nighbourhoods.

```{r [Sp14]}
cellTypesPerCellTypeMatrixExtended = computeCellTypesPerCellTypeMatrix(NBHDByCTMatrixExtended,clusters)

cellTypesPerCellTypeGraphFromCellMatrix(cellTypesPerCellTypeMatrixExtended,
minWeight = 0.05, colours = colours)
```

We can also calculate p values (upper tail, one-sided) for whether
cell types are more commonly neighbours than expected by chance. To do
this we compare the actual spatial neighbour graph to randomised
spatial neighbour graphs where edges are randomised but the degree of
each vertice is preserved. As this is carried out on the level of
counts of undirected edges between celltypes the p value matrix is
symmetric.

```{r [Sp15]}
cellTypesPerCellTypePValues = computeNeighbourEnrichment(delaunayNeighbours, 
                                          clusters)
```

By default it calculates p-values 
analytically using a hypergeometric test on the edges, where the arguments 
to the  R phyper function are as follows:
  q = number of edges between cell type A and B 
  m = number of edges between cell type B and any other cell type
  n = the number of edges between any cell type apart from cell type B
  k =  number of edges between cell type B and any other cell type

The purist may object to the use of the hypergeometric test here.
We may think of "edges out of a cell of type A" as being the random draw 
balls (here, edges) from the urn and "edges out of cells of type B" as being 
success. However, all edges out of a given cell of type A are in this 
"random draw". Clearly the edges in this draw are not independent. However, 
empirically we find that p-values computed using this method correspond very 
closely to those computed using permutation while the computation time is 
orders of magnitude faster.


For legacy purposes, and user flexibility, we allow for the calculation of 
p-values by comparison to randomised graphs. 
The function offers two methods of randomisation, controlled 
by the parameter randomiseBy.  With the value randomiseBy='cells', the
default, randomisaiton is carried out by fixing the underlying graph
(e.g., Delaunay neigbhours graph) and permuting the cell types.  For
counting purposes, we keep the actual cell type at the center of each
neighbourhood and count the permuted cell types surrounding it.  With
the value randomiseBy = 'graph' we use the function randomiseGraph()
to permute the underlying graph.  This uses a heuristic to produce a
random graph which preserves the degrees of the vertices.  When
applied to a Delaunay neighbours graph this can produce a small number
of self-edges and a small number of duplicated edges.  In our
experience, these are sufficiently few in number as to not interfere
with the use of this randomisation method for permutation testing.
randomiseBy='cells' is quicker.


Let's plot -log10(pvalue)
```{r [Sp16]}
cellTypesPerCellTypePValuesNegLog = -log10(cellTypesPerCellTypePValues + 0.001)
pheatmap(cellTypesPerCellTypePValuesNegLog)
```

## Analysis of neighbourhoods based on cell type composition

As mentioned above we can create Seurat objects of neighbourhoods
where the underlying counts matrix gives the the number of cells of
each type in a given neighbourhood.  We can now perform dimensionality
reduction and clustering based on neighbourhood composition. As the
dimensionality of the feature space is relatively low (number of cell
types) we calculate the UMAP using features rather than PCs.

```{r [Sp17]}
NBHDByCTSeurat = computeNBHDVsCTObject(NBHDByCTMatrix,verbose=FALSE)
```

Add cell type information to the neighbourhoodSeurat object.
```{r [Sp18]}
NBHDByCTSeurat$cellType = clusters
```

Visualise neighbourhood clusters.
```{r [Sp19]}
DimPlot(NBHDByCTSeurat, group.by = c("cellType"), cols = "polychrome", reduction = "umap")
DimPlot(NBHDByCTSeurat, group.by = c("neighbourhood_clusters"), cols = "polychrome", reduction = "umap")
```


We can now add information on neighbourhood clusters to our original
Xenium object and visualise these on the tissue.


```{r [Sp20]}
smallXenium$NBHDCluster = NBHDByCTSeurat@active.ident
ImageDimPlot(smallXenium, group.by = "NBHDCluster", size = 1, cols = "polychrome")
```

Let's try the same thing with our extended neighbourhoods up to degree 4.
```{r [Sp21]}
NBHDByCTSeuratExtended = computeNBHDVsCTObject(NBHDByCTMatrixExtended,
                                               verbose=FALSE)
```
Add cell type information to the NBHDByCTSeuratExtended object.
```{r [Sp22]}
NBHDByCTSeuratExtended$cellType = clusters
```

Visualise extended neighbourhood clusters.
```{r [Sp23]}
DimPlot(NBHDByCTSeuratExtended, group.by = c("cellType"), cols = "polychrome", reduction = "umap")
DimPlot(NBHDByCTSeuratExtended, group.by = c("neighbourhood_clusters"), cols = "polychrome", reduction = "umap")
```

We can now add information on extended neighbourhood clusters to our
original Xenium object and visualise these on the tissue.

```{r [Sp24]}
smallXenium$NBHDClusterExtended= 
  NBHDByCTSeuratExtended@active.ident
ImageDimPlot(smallXenium, group.by = c("NBHDClusterExtended"), 
             size = 1, cols = "polychrome")
```


Here we retrieve fewer clusters, and these describe tissue architecture 
rather than small variations in cellular niche.  

We leave it to the user to decide which approach is most applicable to the 
biological question at hand.


## Relating cell type clusters to neighbourhood clusters

We can now ask how clusters defined transcriptomically (cell type)
relate to those defined based on neighbourhoods (cell niche).  Put
differently, each cell has its (transcriptomic) cell type and sits at
the center of a neighbourhood with a given neighbourhood type.  For
each cell type A and each neighbourhood type B, we can ask what
percentage of the time a cell of type A sits at the center of a
neighbourhood of type B.

```{r [Sp25]}
CTByNBHDCluster = table(NBHDByCTSeurat$cellType,NBHDByCTSeurat@active.ident)
CTByNBHDCluster = CTByNBHDCluster/rowSums(CTByNBHDCluster)

rownames(CTByNBHDCluster) = paste0("CellType",rownames(CTByNBHDCluster))
colnames(CTByNBHDCluster) = paste0("NBHDCluster",colnames(CTByNBHDCluster))

pheatmap(CTByNBHDCluster,
      fontsize_row=8,
      fontsize_col=8,
      cellheight=10,
      cellwidth=10)

sankeyFromMatrix(CTByNBHDCluster)
```

This allows us to see which cell types share the same niche (neighbourhood 
clusters). Let's also perform this analysis using the extended neighbourhoods. 


```{r [Sp26]}
CTByNBHDClusterExtended = table(NBHDByCTSeuratExtended$cellType,NBHDByCTSeuratExtended@active.ident)
CTByNBHDClusterExtended = CTByNBHDClusterExtended/rowSums(CTByNBHDClusterExtended)

rownames(CTByNBHDClusterExtended) = paste0("CellType",rownames(CTByNBHDClusterExtended))
colnames(CTByNBHDClusterExtended) = paste0("NBHDCluster",colnames(CTByNBHDClusterExtended))

pheatmap(CTByNBHDClusterExtended,
      fontsize_row=8,
      fontsize_col=8,
      cellheight=10,
      cellwidth=10)

sankeyFromMatrix(CTByNBHDClusterExtended)
```

## Analysing cell types based on their neighbourhoods

Now we perform dimensionality reduction and clustering of cell types, based on the 
neighbourhoods they are found in. Note that this is based on the transpose of 
the matrix used in the previous section.

First we create a Seurat object for a cell type by neighbourhood matrix (the 
transposed NBHDByCTMatrix). Here we have a 
large feature space (number of cells) however a low number of observations 
(number of cell types). Therefore we compute the UMAP using PCs, however due to 
the low number of observations we need to set a lower value for n.neighbours.


```{r [Sp27]}
CTByNBHDSeurat = 
  computeNBHDVsCTObject(t(NBHDByCTMatrix), npcs = 10, 
                        transpose = TRUE, resolution = 1, n.neighbors = 5,
			verbose=FALSE)

CTByNBHDSeurat$cellType = colnames(CTByNBHDSeurat)

DimPlot(CTByNBHDSeurat, group.by = "cellType", cols = "polychrome", 
        reduction = "umap", label = TRUE)
```

Note how Pyramidal, Pvalb and Lamp5 neurons are located close together
in UMAP space indicating they are often found in the same
neighbourhoods.

We can also compute a force-directed graph embedding.

```{r [Sp28]}
CTByNBHDSeurat= computeGraphEmbedding(CTByNBHDSeurat)

DimPlot(CTByNBHDSeurat,group.by = "cellType", cols = "alphabet", reduction = "graph", label = TRUE)
```

From this  11_oligo and 5_Astro appear to be in neighbourhoods distinct from the 
other cell types. Looking at cell type localisation in the tissue shows that 
they colocalise.
```{r [Sp29]}
ImageDimPlot(smallXenium, cols = "polychrome", size = 1)
```


Clustering cell types using the Louvain algorithm acheives only one cluster. 
Let's try hierachical clustering.
```{r [Sp30]}
pca = Embeddings(CTByNBHDSeurat, reduction = "pca")
res = pheatmap(pca)
```

Here we decide to cut the tree at the correct level to form 11 clusters.
```{r [Sp31]}
CTClust = cutree(res$tree_row, k = 11)
CTByNBHDSeurat$neighbourhood_clusters = factor(CTClust)
```

Let's look at the celltype composition of these clusters.
```{r [Sp32]}
CTComposition = table(CTByNBHDSeurat$cellType, CTByNBHDSeurat$neighbourhood_clusters)
pheatmap(CTComposition)
```

Although most clusters here are formed of a single cell type, cluster 2
contains several cell types. We anticipate this analysis may be more informative 
when performed on a larger tissue area, as some cell types are poorly 
represented in this subset. To this end we demonstrate how to explore the 
relationship between neighbourhood clusters and cell type clusters.
```{r [Sp33]}
averageExpMatrix = getAverageExpressionMatrix(NBHDByCTSeurat,
                           CTByNBHDSeurat,
			    clusteringName='neighbourhood_clusters')
averageExpMatrix = tagRowAndColNames(averageExpMatrix,
                                     ccTag='neighbourhoodClusters_',
                                     gcTag='cellTypeClusters_')
pheatmap(averageExpMatrix,
      fontsize_row=8,
      fontsize_col=8,
      cellheight=10,
      cellwidth=10)


sankeyFromMatrix(averageExpMatrix)
```

## Detection of genes with spatially variable expression.

Given a neighbour graph describing the spatial relationships between
cells, we can compute spatial autocorrelation (here we use Moran's
I). This describes how clustered gene expression values are in
space. We derive an upper tail p value based on permutation testing
where expression values for genes are permuted. N.B in this
implementation equal weights are given to all neighbours of a cell.
Minimum values are limited by nSim, the number of permutations used.

```{r [Sp34]}
moransI = runMoransI(smallXenium, delaunayNeighbours, assay = "SCT", 
                     layer = "data", nSim = 20, verbose = FALSE)
```
Look at most spatially autocorrelated genes.
```{r [Sp35]}
head(moransI)
```

Look at least spatially autocorrelated genes.
```{r [Sp36]}
tail(moransI)
```

Here we visualise the most spatially autocorrelated gene.
```{r [Sp37]}
ImageFeaturePlot(smallXenium, "Nwd2")
```

Here we visualise the least spatially autocorrelated gene.
```{r [Sp38]}
ImageFeaturePlot(smallXenium, "Trbc2")
```

## Ligand receptor analysis

We can perform an analysis of ligand receptor interactions in order to
infer communication between cells.  Here we calculate whether
interactions between ligand receptor pairs occur on edges of the
spatial graph, i.e. between neighbouring cells, where cell A expresses
a ligand and cell B expresses a receptor. Note that the distance
between cell A and cell B will vary depending on how the graph has
been constructed, larger distances may be desired for analyses
involving diffusable ligands. We also analyse the number
of ligand receptor interactions that occur between cells from the same
cluster and between cells from different clusters. Note that clusters
may represent cell type, or another property of the cells such as
their neighbourhood type. We leave this for the user to decide. 

We then calculate an analytical p-value for the enrichment of ligand-receptor 
interactions between clusters.  Specifically an upper 
tail p-value for observing a given number of A-B edges positive for a given 
interaction is calculated using a binomial test (R pbinom function) where:
  q = number of A-B edges positive for an interaction
  size = total number of A-B edges   
  prob = pL*pR
Where pL is the probability of a cell expressing a specific ligand (number 
of cells positive for a ligand/total cells), 
and pR is the probability of a cell expressing a specific receptor (number 
of cells positive for a receptor/total cells). 
If conditional = True  p-values will be calculated given the proportion of 
cells that express ligands and receptors in the specific clusters (pL = 
number of cells in cluster A positive for a ligand/number of cells in cluster
A, pR = number of cells in cluster B positive for a receptor/number of cells
in cluster B).

The analytical method to calculate p-values has a faster run time than the 
permutation-based method previously implemented in this package, however the 
user can optionally choose to calculate permutation-based p-values by using 
method = "permutation". Of note, the speed of this method has also increased due 
to the use of sparse matrices. 

```{r [Sp39]}
ligandReceptorResults = performLigandReceptorAnalysis(smallXenium, delaunayNeighbours, 
                                                "mouse", clusters, 
                                                method = "analytical", 
                                                conditional = FALSE,
                                                minEdgesPos = 10)
```


We look at interactions on edges.  For concision, we display the
first ten of our 28 ligand-receptor pairs.  Since ligand-receptor
interactions are directed, each undirected edge will show up both as
cell A - cell B and as cell B - cell A.
```{r [Sp40]}
head(as.matrix(ligandReceptorResults$interactionsOnEdges[,1:10]),10)
```

Look at total interactions between/within clusters:
```{r [Sp41]}
head(ligandReceptorResults$totalInteractionsByCluster[,1:10],10)
```

Look at mean interactions per edge between/within clusters (sum of ligand 
receptor interactions on A-B edges)/(total A-B edges):
```{r [Sp42]}
head(ligandReceptorResults$meanInteractionsByCluster[,1:10],10)
```


Look at p-values (upper tail) for the enrichment of interactions. Of note these 
are set to NA if less than 10 edges are positive for a ligand-receptor 
interaction. This threshold can be changed using the "minEdgesPos" argument of
performLigandReceptorAnalysis().

```{r [Sp43]}
head(ligandReceptorResults$pValues,10) 
```


Plot a heatmap showing -log10(pvalues) for the enrichment of ligand-receptor 
interactions between pairs of clusters. 

```{r [Sp44]}
ligRecMatrix = makeLRInteractionHeatmap(ligandReceptorResults, clusters, colours = colours, labelClusterPairs = FALSE)
```

Make a heatmap of total ligand-receptor interactions between clusters.
```{r [Sp45]}
cellTypePerCellTypeLigRecMatrix = makeSummedLRInteractionHeatmap(ligandReceptorResults, clusters, "total")
```

Make a heatmap of summed mean ligand/receptor interactions between
clusters i.e., (sum of all ligand receptor interactions on A-B edges)/(total
A-B edges)

```{r [Sp46]}
cellTypePerCellTypeLigRecMatrix = makeSummedLRInteractionHeatmap(ligandReceptorResults, clusters, "mean", logScale = TRUE)
```
Make a dotplot of ligand-receptor interactions.

```{r [Sp47]}
p = plotLRDotplot(ligandReceptorResults, padjCutoff = 0.05, pvalCutoff = F, splitBy = "sender")
```
Here we choose to plot interactions with a p-adj value < 0.05 and split the 
data by sender populations. It is also possible to create dotplots of ligand-receptor 
interactions for specific populations:

```{r [Sp48]}
plotLRDotplot(ligandReceptorResults, senderClusters = c("6_VLMC","7_Granule"), 
              padjCutoff = 0.05, pvalCutoff = F, splitBy = "sender")
```

Look at a histogram of values for summed mean ligand/receptor interactions between clusters.
```{r [Sp49]}
hist(cellTypePerCellTypeLigRecMatrix)
```

Visualise in graph format.
```{r [Sp50]}
cellTypesPerCellTypeGraphFromCellMatrix(cellTypePerCellTypeLigRecMatrix, 
                                    minWeight = 0.4, colours = colours)
```
Here the arrows are very thick - we can scale by a constant factor to improve the
presentation.

```{r [Sp51]}
scaleFactor = 3
cellTypesPerCellTypeGraphFromCellMatrix(cellTypePerCellTypeLigRecMatrix/scaleFactor, 
                                    minWeight = 0.4/scaleFactor, colours = colours)
```

### Visualising ligand-receptor interactions: Seurat objects of edges

We wish to create a Seurat object where the points are edges between
cells as a way of visualising ligand-receptor interactions.  Since
these interactions are asymmetric we need to consider directed edges.
We will place the spatial coordinates of the edges A-B and B-A
slightly separate but both between the centroids of cells A and B. The
"expression matrix" is the binarised presence/absence of an
interaction (ligand receptor pair) on an edge. This is useful for
visualising where ligand receptor interactions occur spatially.

```{r [Sp52]}
edgeSeurat = computeEdgeObject(ligandReceptorResults, centroids)
```


For example we can visualise the Pdyn-Npy2r interaction.
```{r [Sp53]}
ImageFeaturePlot(edgeSeurat, features = "Pdyn-Npy2r")
```

In principle, the edge Seurat object allows for the computation of an
edge UMAP and Louvain clustering of the edges based on ligand-receptor
interaction.  However some of these analyses are computationally
expensive due to large number of edges.


### Spatial autocorrelation of ligand-receptor interactions

We can compute a spatial graph where edges in the original delaunayNeighbours 
become nodes and A-B edges (in the original graph) become connected to all 
A- edges and all B- edges. This allows us to perfom graph-based
analyses associated with 
the spatial localisation of ligand receptor pairs on edges.  
```{r [Sp54]}
edgeNeighbours = computeEdgeGraph(delaunayNeighbours)
```

Compute Moran's I for the spatial autocorrelation of ligand-receptor interactions.
```{r [Sp55] , eval=FALSE}
moransILigandReceptor = runMoransI(edgeSeurat, edgeNeighbours, assay = "RNA", 
                     layer = "counts", nSim = 100)
```

View most spatially autocorrelated ligand-receptor interactions.
```{r [Sp56]}
moransILigandReceptor = getExample('moransILigandReceptor')
head(moransILigandReceptor)
```

View least spatially autocorrelated ligand-receptor interactions.
```{r [Sp57]}
tail(moransILigandReceptor)
```


View ligand-receptor interaction with the highest spatial autocorrelation (Moran's I).
```{r [Sp58]}
ImageFeaturePlot(edgeSeurat, "Penk-Htr1f")
```

View ligand-receptor interaction with the lowest spatial autocorrelation (Moran's I).
```{r [Sp59]}
ImageFeaturePlot(edgeSeurat, "Sst-Gpr17")
```

## Quality control of Delaunay neighbours

We use Delaunay triangulation to estimate neighbour to neighbour
contact between cells.  However, a cell which happens to sit next to a
tissue void is likely to have one or more Delaunay neighbours which
are cells that sit on the other side of this void.  We hope to detect
these spurious edges based on their lengths.  However, the distances
between neighbouring cells' centroids tend to depend on the types of
the neighbouring cells.  Accordingly, we have supplied functions to
annotate edges with their distances and cell types; compute estimates
for edge-length cutoffs based on cell type pair; plot the
distributions of lengths together with proposed cutoffs; and subset
the edges based on cutoffs.

### edgeLengthsAndCellTypePairs

This functions annotates edges with their lengths and the cell types
of the two cells in question.

```{r [Sp60]}
annEdges =
edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
head(annEdges)
```

### Visualising the edge lengths

Here we visualise the distribution of edge lengths together with
proposed cutoffs.  We will say more in a moment about how these
proposed cutoffs are calculated.

```{r [Sp61]}
cutoffDF = edgeCutoffsByPercentile(annEdges,percentileCutof=95)
g = edgeLengthPlot(annEdges,cutoffDF,whichPairs=60)
print(g)
```

In this case, there are 111 types of edges based on the cell types of
the cells they connect.  By setting whichPairs=60, we are restricting
to those types for which there are at least 60 such edges.  Here we
are also using a default xlim of 100.  Passing the value NULL for
cutoffDF will produce a plot without these cutoffs.

### Estimating edge length cutoffs

We offer several methods for estimating edge length cutoffs.  We
recommend visual inspection before settling on any specific set of
results. Each of these returns a data frame with two columns,
cellTypePair and cutoff.  As we have just seen this can be used for
the plotting function and it can also be used for culling the edges.
Since these results are returned as a data frame, they can be tuned
"by hand" by modifying the cutoff value for any individual cell type
pair.

#### edgeCutoffsByClustering

```{r [Sp62]}
cutoffDF = edgeCutoffsByClustering(annEdges)
```


This method works well in a small percentage of cases.  For each cell
type pair it performs k-means clustering of the lengths with k=2 and
produces a cutoff halfway between the two clusters.  This works well
when there really are two clusters of edges, those between truly
neighbouring cells and those across tissue voids.  However, in the
case where all the edges are between neighbouring cells, this will
produce a spurious cutoff.

#### edgeCutoffsByPercentile


```{r [Sp63]}
cutoffDF = edgeCutoffsByPercentile(annEdges,percentileCutoff=95)
```


This method produces cutoffs based on a chosen percentile applied
across all cell type pairs.  It has the advantages that its results
are easily comprehensible in terms of the chosen parameter.  The user
my want to "mix and match" using different percentiles for different
cell type pairs.

#### edgeCutoffsByZScore


```{r [Sp65]}
cutoffDF = edgeCutoffsByZScore(annEdges,zCutoff=1.5)
```


Very similar to the previous method, using z-score instead of
percentile.

#### edgeCutoffsByWatershed


```{r [Sp66]}
cutoffDF = edgeCutoffsByWatershed(annEdges,nbins=15,tolerance=10)
```

For each cell type pair, this function first computes a histogram of
the distances.  It then uses watershed segmentation to discover the
"humps" of this histogram.  The most important of these will be the
one containing the median of the distances.  This reports as a cutoff
the midpoint of the first histogram bin which is past this main hump.
The parameter nbins specifies the number of bins for the histogram.
The parameter tolerance controls the sensitivity of the watershed
algorithm.  

#### cullEdges

```{r [Sp67]}
cutoffDF = edgeCutoffsByWatershed(annEdges,nbins=15,tolerance=10)
culledEdges = cullEdges(annEdges,cutoffDF)
nrow(annEdges)
nrow(culledEdges)
```

This subsets the edges according to the parameter cutoffSpec.  This
can be a cutoff data frame as produced by any of the edgeCutoff
functions or can be a single number in which case all the cell type
pairs will be trimmed to the same cutoff.

```{r [Sp68]}
sessionInfo()
```

---
title: "CatsCradle Example Data"
author: "Anna Laddach and Michael Shapiro"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{CatsCradle Example Data}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE, warning = FALSE}
knitr::opts_chunk$set(
    collapse = TRUE,
    fig.dim = c(6,6),
    comment = "#>"
)
```
`r BiocStyle::Biocpkg("BiocStyle")`

![](CatsCradleLogo.png){width=2in}


## CatsCradle Example Data

CatsCradle has a slightly unusual way of delivering example data.  In
order to save space we have tried to minimise the number and size of
the .rda data.  We have limited these to objects which are necessary
to compute other objects (e.g., certain Seurat objects), objects which
are necessary to the functioning of the package (e.g., human and mouse
ligand receptor networks), and objects which take a long time to
compute (e.g., ligandReceptorResults).  These package variables are
documented in the usual way and are loaded when invoked with
data(variable name).

Other variables are computed from these as needed.  The function which
computes these additional objects also stores them internally for
quick retrieval when they are subsequently requested.  The retrieval
function must be created before it can be used.  It only needs to be
created once in each session.


On the first invocation this computes STranspose:

```{r [Ex1], message=FALSE}
library(CatsCradle,quietly=TRUE)
library(tictoc)
getExample = make.getExample()
tic()
STranspose = getExample('STranspose')
toc()
```

On subsequent invocations it retrieves it:

```{r [Ex2]}
tic()
STranspose = getExample('STranspose')
toc()
```

getExample() also retrieves package variables.

```{r [Ex3], message=FALSE}
smallXenium = getExample('smallXenium')
```

getExample() also supports the option toy, which is FALSE by default.
Setting toy=TRUE will result in the initial Seurat objects being
subset to smaller copies of themselves and all further computations
being performed using these smaller objects.  This is done to speed up
the running of the code examples.

```{r [Ex4], message=FALSE}
exSeuratObj = getExample('exSeuratObj')
toySeurat = getExample('exSeuratObj',toy=TRUE)
dim(exSeuratObj)
dim(toySeurat)
toyXenium = getExample('smallXenium',toy=TRUE)
dim(smallXenium)
dim(toyXenium)
```

A complete list of the example objects available via the function
exampleObjects().  Here we document some of the more prominent of
these.

* __exSeuratObj__ - A Seurat object of cells.  It includes a UMAP of the
cells and annotated clustering into cell types.  This is a package
variable.
* __STranpose__ - A Seurat object of genes.  This is created from S by
transposeObject().
* __S_sce__ - S as a SingleCellExperiment
* __STranspose_sce__ - STranspose as a SingleCellExperiment
* __NN__ - A nearest neighbours graph from STranspose.  This is a data
frame with columns nodeA, nodeB, and weight, where nodeA and nodeB are
genes.
* __smallXenium__ - A spatial Seurat object subset from the Xenium
object used in 
https://satijalab.org/seurat/articles/seurat5_spatial_vignette_2.
This is a package variable.
* __clusters__ - Annotated cell type clustering for the cells of
smallXenium.
* __centroids__ - The centroids of the cells of smallXenium
* __delaunayNeighbours__  - A data frame giving neighbour-neighbour
relations for the cells of smallXenium as estimated by Delaunay
triangulation with columns nodeA and nodeB.
* __extendedNeighours__ - A data frame giving extended
neighbour-neighbour relations for the cells of smallXenium.  Here
cells are "neighbours" if they are within four edges of each other in
the Delaunay triangulation.
* __euclideanNeighbours__ - Neighbour-neighbour relations between the
cells of smallXenium based on the Euclidean distance of their
centroids.
* __NBHDByCTMatrix__ - For each neighbourhood, we count the number of
cells it contains of each cell type. Neighbourhoods "express" cell
types.
* __NBHDByCTSeurat__ - A Seurat object whose underlying counts are
NBHDByCTMatrix. 
* __cellTypesPerCellTypeMatrix__ - A matrix which reports on the
Delaunay neighbourhoods.  For each cell type, it examines the
neighbourhoods around cells of that type and reports their average
composition as to the cell types they contain.
* __cellTypesPerCellTypePValues__ This gives the statistical
significance for the enrichment of cell types in the neighbourhoods of
each cell type, computed using randomiseBy = 'cells'.


```{r [Ex5]}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoods.R
\name{collapseExtendedNBHDs}
\alias{collapseExtendedNBHDs}
\title{This function takes an expanded neighbourhood list and collapses it to a
nearest neighbourhood graph where all neighbours of degree <= n in the
original graph are considered first neighbours.}
\usage{
collapseExtendedNBHDs(
  extendedNeighboursList,
  n = length(extendedNeighboursList)
)
}
\arguments{
\item{extendedNeighboursList}{\itemize{
\item the results of getExtendedNBHDs()
}}

\item{n}{\itemize{
\item the maximum degree to connect neighbours. Defaults to the maximum
degree neighbourhoods were expanded to in the results of getExtendedNBHDs().
}}
}
\value{
a graph in neighbour format, i.e., a data frame with
columns nodeA and nodeB, where nodes that were originally of degree <= n
are connected.
}
\description{
This function takes an expanded neighbourhood list and collapses it to a
nearest neighbourhood graph where all neighbours of degree <= n in the
original graph are considered first neighbours.
}
\examples{
extendedNeighboursList = make.getExample()('extendedNeighboursList',toy=TRUE)
extendedNeighbours = collapseExtendedNBHDs(extendedNeighboursList, 4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{orderGeneSetPValues}
\alias{orderGeneSetPValues}
\title{This orders the gene set p-values (or -log10 p-values) and
applies a cutoff (if given) to show only the significant
gene sets for each gene cluster}
\usage{
orderGeneSetPValues(M, ascending = TRUE, cutoff = NULL, nameTag = "")
}
\arguments{
\item{M}{\itemize{
\item A matrix of gene set p-values (or their logs)
to be ordered by their significance
}}

\item{ascending}{\itemize{
\item Direction in which to order the columns.
Defaults to TRUE, so that p-values will be ordered according
to decreasing significance, should be set to FALSE if ordering
-log p-value
}}

\item{cutoff}{\itemize{
\item if non-null this is used to extract only
significant cases
}}

\item{nameTag}{\itemize{
\item can be used to modify the names of the list.
}}
}
\value{
This returns a list of whose entries are data frames,
one for each gene cluster, each giving the significant gene sets
for that cluster and their significance.
}
\description{
This orders the gene set p-values (or -log10 p-values) and
applies a cutoff (if given) to show only the significant
gene sets for each gene cluster
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{moransILigandReceptor}
\alias{moransILigandReceptor}
\title{moransILigandReceptor}
\format{
A data frame showing the spatial autocorrelation of the
28 ligand receptor pairs

\describe{
A data frame with rownames giving the  28 ligand-receptor pairs and columns moransI
and pValues
}
}
\source{
Computed using the function runMoransI on the object edgeSeurat and
neighbours edgeNeighbours = computeEdgeGraph(delaunayNeighbours) with 100 trials.
For more informations see the CatsCradleSpatial vignette.
}
\usage{
moransILigandReceptor
}
\description{
Moran's I for the ligand receptor pairs
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{performLigandReceptorAnalysisPermutation}
\alias{performLigandReceptorAnalysisPermutation}
\title{Given a seurat object, a spatial graph, clusters and
species this function identifies ligand-receptor
interactions between neighbouring cells, identifies
ligand-receptor interactions within and between clusters
and calculates whether these are observed more frequently
than expected by chance using a permutation-based approach.}
\usage{
performLigandReceptorAnalysisPermutation(
  obj,
  spatialGraph,
  species,
  clusters,
  nSim = 1000,
  lrn = getLigandReceptorNetwork(species),
  minEdgesPos = 10,
  verbose = TRUE
)
}
\arguments{
\item{obj}{\itemize{
\item a Seurat object
}}

\item{spatialGraph}{\itemize{
\item a data frame of neighbouring
cell pairs.
}}

\item{species}{\itemize{
\item either 'human' or 'mouse'
}}

\item{clusters}{\itemize{
\item named vector of clusters where names are each cell and
clusters are a factor
}}

\item{nSim}{\itemize{
\item number of simulations to perform for p value calculation.
}}

\item{lrn}{\itemize{
\item a ligand-receptor network, i.e., a
data frame with columns from and to.  By default, it
retrieves the nichenetr ligand receptor network
}}

\item{minEdgesPos}{\itemize{
\item the minimum edges that need to be positive for a
ligand-receptor interaction between two clusters for a p-value to be
calculated. Only taken into consideration when the analytical method is
selected.
}}

\item{verbose}{\itemize{
\item whether to print trace, defaults to TRUE
}}
}
\value{
A list containing:
interactionsOnEdges - a sparse matrix where the rownames give pairs of
neighbouring cells and column names give ligand-receptor pairs.
Entries are TRUE if the ligand is expressed in the first cell and the receptor is expressed in
the second cell and FALSE if not.
interactionsOnEdgesMeta - a dataframe where the first two columns are the
cells that comprise the edges in interactionsOnEdges, and the next two
columns are their clusters.
totalInteractionsByCluster - a dataframe where the rownames are
sender-receiver cluster pairs and column names are ligand receptor pairs.
Entries are total numbers of edges on which particular ligand receptor
interactions are present.
meanInteractionsByCluster - a dataframe where the rownames are
sender-receiver cluster pairs and column names are ligand receptor pairs.
Entries are total numbers of edges on which particular ligand receptor i
nteractions are present (for that cluster pair) divided by the total number
of edges between those clusters.
simResults - a dataframe where the rownames are sender-receiver cluster
pairs and column names are ligand receptor pairs.
Values give the number of simulations for which observed values are greater
than simulated values.
pValues - a dataframe where the rownames are sender-receiver cluster pairs
and column names are ligand receptor pairs. Entries are uppertail p-values
describing whether a particular ligand receptor interaction is observed more
frequently between 2 clusters than expected.
totalEdges - a vector of total edges between cluster pairs.
}
\description{
Given a seurat object, a spatial graph, clusters and
species this function identifies ligand-receptor
interactions between neighbouring cells, identifies
ligand-receptor interactions within and between clusters
and calculates whether these are observed more frequently
than expected by chance using a permutation-based approach.
}
\examples{
getExample = make.getExample()
smallXenium = getExample('smallXenium')
delaunayNeighbours = getExample('delaunayNeighbours')
clusters = getExample('clusters')
performLigandReceptorAnalysis(smallXenium, delaunayNeighbours, 
                                      "mouse", clusters,  minEdgesPos = 10, nSim = 10,
                                       verbose=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{geneSetsVsGeneClustersPValueMatrix}
\alias{geneSetsVsGeneClustersPValueMatrix}
\title{This compares the gene clusters to other gene sets
e.g., GO, Hallmark, and determines the p-value for
their overlaps when compared to a set of background
genes.}
\usage{
geneSetsVsGeneClustersPValueMatrix(
  geneSets,
  clusterDF,
  backgroundGenes,
  adjust = FALSE
)
}
\arguments{
\item{geneSets}{\itemize{
\item a named list of gene sets
}}

\item{clusterDF}{\itemize{
\item a data frame giving the cluster
membership of each gene with columns gene and geneCluster
}}

\item{backgroundGenes}{\itemize{
\item a character vector of genes
}}

\item{adjust}{\itemize{
\item a logical deciding whether to adjust
p values.  Defaults to FALSE.
}}
}
\value{
a matrix of p-values rows correspond to the gene
sets and the columns correspond the the CatsCradle gene
clusters
}
\description{
This compares the gene clusters to other gene sets
e.g., GO, Hallmark, and determines the p-value for
their overlaps when compared to a set of background
genes.
}
\examples{
getExample = make.getExample()
STranspose = getExample('STranspose',toy=TRUE)
clusterDF = data.frame(gene=colnames(STranspose),
                       geneCluster=STranspose$seurat_clusters)
hallmark = getExample('hallmark')
geneSet = intersect(hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]],colnames(STranspose))
pvalueMatrix = geneSetsVsGeneClustersPValueMatrix(geneSet,
                                              clusterDF,
                                              colnames(STranspose))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNNGraphs.R
\name{symmetryCheckNN}
\alias{symmetryCheckNN}
\title{Tests whether a nearest neighbor graph is symmetric}
\usage{
symmetryCheckNN(NN)
}
\arguments{
\item{NN}{\itemize{
\item a nearest neighbor graph.  This is in the form
of a data frame  as returned by getNearestNeighbourLists.
Its coloumns include nodeA and nodeB.
}}
}
\value{
TRUE or FALSE
}
\description{
The nearest neighbor relationship is not inherently
symmetric.  This tests whether the nearest neighbor graph
retrieved from a Seurat object is.
}
\examples{
 NN = make.getExample()('NN',toy=TRUE)
symmetryTest = symmetryCheckNN(NN)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{computeGraphEmbedding}
\alias{computeGraphEmbedding}
\title{This function adds a force directed graph embedding to a seurat object}
\usage{
computeGraphEmbedding(
  seuratObj,
  graph = defaultGraph(seuratObj),
  returnType = "Seurat"
)
}
\arguments{
\item{seuratObj}{\itemize{
\item a seurat object of SingleCellExperiment to be
turned into a Seurat object
}}

\item{graph}{\itemize{
\item which graph to extract.  Defaults to
paste0(f@active.assay,'_snn')
}}

\item{returnType}{\itemize{
\item Will return a SingleCellExperiment if this is either
of SCE, SingleCellExperiment or their lower-case equivalents.  Otherwise,
returns a Seurat object
}}
}
\value{
a seurat object with a "graph" dimensionality reduction. Can also
be a SingleCellExperiment depending on parameter returnType.
}
\description{
This function adds a force directed graph embedding to a seurat object
}
\examples{
NBHDByCTSeurat = make.getExample()('NBHDByCTSeurat',toy=TRUE)
objWithEmbedding = computeGraphEmbedding(NBHDByCTSeurat)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{convertToLong}
\alias{convertToLong}
\title{This is a utility function for converting ligandReceptor cluster-level
results to long format and calculates adjusted p-values.}
\usage{
convertToLong(ligandReceptorResults)
}
\arguments{
\item{ligandReceptorResults}{\itemize{
\item ligandReceptorReults calculated using
performLigandReceptorAnalysis()
}}
}
\value{
ligand receptor results in long format
}
\description{
This is a utility function for converting ligandReceptor cluster-level
results to long format and calculates adjusted p-values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{getClusterOrder}
\alias{getClusterOrder}
\title{This gets the clusters in their cannonical order}
\usage{
getClusterOrder(f)
}
\arguments{
\item{f}{\itemize{
\item a Seurat object with meta.data column seurat_clusters
or SingleCellExperiment to be turned into a Seurat object
}}
}
\value{
A vector of these unique values in order
}
\description{
This deals with skullduggery in which seurat_clusters
has been converted from a factor to a character or a
numeric.
}
\examples{
STranspose = make.getExample()('STranspose',toy=TRUE)
geneClusters = getClusterOrder(STranspose)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGenesVsCells.R
\name{meanGeneClusterOnCellUMAP}
\alias{meanGeneClusterOnCellUMAP}
\title{Mean gene cluster on cell umap}
\usage{
meanGeneClusterOnCellUMAP(f, fPrime, geneCluster)
}
\arguments{
\item{f}{\itemize{
\item a Seurat object of cells or SingleCellExperiment to
be converted to a Seurat object
}}

\item{fPrime}{\itemize{
\item the corresponding Seurat object of genes
SingleCellExperiment to be converted to a Seurat object
}}

\item{geneCluster}{\itemize{
\item a gene cluster of fPrime
}}
}
\value{
This returns a ggplot object
}
\description{
This function paints gene expression for a
given gene cluster on cell umap.
}
\examples{
getExample = make.getExample()
exSeuratObj = getExample('exSeuratObj',toy=TRUE)
STranspose = getExample('STranspose',toy=TRUE)
g = meanGeneClusterOnCellUMAP(exSeuratObj,STranspose,geneCluster=0)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{runMoransI}
\alias{runMoransI}
\title{This function takes a matrix where rows are features and columns are cells,
and a neighbourhood list, and computes Moran's I.}
\usage{
runMoransI(
  obj,
  spatialGraph,
  assay = "RNA",
  layer = "data",
  nSim = 100,
  verbose = TRUE
)
}
\arguments{
\item{obj}{\itemize{
\item a Seurat object
}}

\item{spatialGraph}{\itemize{
\item a data frame of neighbouring
cell pairs.
}}

\item{assay}{\itemize{
\item assay to pull data from, defaults to RNA.
}}

\item{layer}{\itemize{
\item layer to pull data from, defaults to data.
}}

\item{nSim}{\itemize{
\item number of simulations to perform for p value calculation.
Defaults to 100.
}}

\item{verbose}{\itemize{
\item whether to print trace, defaults to TRUE
}}
}
\value{
a dataframe containing Moran's I and p values for each feature.
}
\description{
This function takes a matrix where rows are features and columns are cells,
and a neighbourhood list, and computes Moran's I.
}
\examples{
getExample = make.getExample()
smallXenium = getExample('smallXenium',toy=TRUE)
delaunayNeighbours = getExample('delaunayNeighbours',toy=TRUE)
moransI = runMoransI(smallXenium, delaunayNeighbours, assay = "SCT", 
layer = "data", nSim = 10, verbose = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{predictGeneAnnotationImpl}
\alias{predictGeneAnnotationImpl}
\title{This function is the implementation for predicting
the functions of a gene based on the functions of its
neighbours.}
\usage{
predictGeneAnnotationImpl(
  gene,
  fPrime,
  genesAnno,
  radius,
  metric,
  numPCs = NULL,
  normaliseByDistance = TRUE
)
}
\arguments{
\item{gene}{\itemize{
\item gene to annotate
}}

\item{fPrime}{\itemize{
\item a Seurat object of genes or
SingleCellExperiment to be converted to a Seurat object
}}

\item{genesAnno}{\itemize{
\item genes annotated with gene sets
}}

\item{radius}{\itemize{
\item radius of neighbours to consider
}}

\item{metric}{\itemize{
\item which metric to use to discover
neighbours, can be one of 'umap', 'tsne', 'pca', 'NN',
defaults to umap
}}

\item{numPCs}{\itemize{
\item used only if metric is pca. Defaults to NULL
}}

\item{normaliseByDistance}{\itemize{
\item choose whether to normalise
contributions of neighbors by their distance, defaults to
TRUE
}}
}
\value{
This returns a named list.  The names are
the anotations that apply to the neighbour genes, the values
are the relative wieghts of the contributions.
}
\description{
This function is the implementation for predicting
the functions of a gene based on the functions of its
neighbours.
}
\examples{
getExample = make.getExample()
STranspose = getExample('STranspose',toy=TRUE)
hallmark = getExample('hallmark',toy=TRUE)
genesAnno = annotateGenesByGeneSet(hallmark)
predictions = predictGeneAnnotationImpl('Myc',STranspose,genesAnno,
radius=.5,metric='umap')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleEdgeQualityControl.R
\name{edgeLengthsAndCellTypePairs}
\alias{edgeLengthsAndCellTypePairs}
\title{This function annotates edges with their distance and
the types of cells they connect}
\usage{
edgeLengthsAndCellTypePairs(edges, clusters, centroids)
}
\arguments{
\item{edges}{\itemize{
\item A data frame with columns nodeA and nodeB giving the
cells of each edge
}}

\item{clusters}{\itemize{
\item the clusters of each cell
}}

\item{centroids}{\itemize{
\item the centroids of each cell
}}
}
\value{
a data frame giving the edges (as nodeA and nodeB), their
lengths and the cell type pair.
}
\description{
This function annotates edges with their distance and
the types of cells they connect
}
\examples{
getExample = make.getExample()
centroids = getExample('centroids')
clusters = getExample('clusters')
delaunayNeighbours = getExample('delaunayNeighbours')
annEdges = edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGenesVsCells.R
\name{tagRowAndColNames}
\alias{tagRowAndColNames}
\title{This gussies up the rownames and colnames of M}
\usage{
tagRowAndColNames(M, ccTag = "CC_", gcTag = "GC_")
}
\arguments{
\item{M}{\itemize{
\item a matrix, typically the average expression matrix
}}

\item{ccTag}{\itemize{
\item a prefix for the row (cell cluster) names
}}

\item{gcTag}{\itemize{
\item a prefix for the column (gene cluster) names
}}
}
\value{
The same matrix with fancier row and col names
}
\description{
This gussies up the rownames and colnames of M
}
\examples{
getExample = make.getExample()
averageExpMatrix = getExample('averageExpMatrix',toy=TRUE)
averageExpMatrix = tagRowAndColNames(averageExpMatrix,'cellCluster_','geneCluster_')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{xeniumCells}
\alias{xeniumCells}
\title{xeniumCells}
\format{
A vector of cells

\describe{
A vector of cells consisting of approximately one
quarter of the cells in smallXenium used to subset
this object to give toy examples.
}
}
\source{
We extracted a rectangle whose width and
height were one half the width and height of smallXenium
and which was centered in the field of view of
smallXenium
}
\usage{
xeniumCells
}
\description{
A vector of cells used for subsetting exSeuratObj
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleExampleServer.R
\name{make.getExample}
\alias{make.getExample}
\title{This function makes the function whichretrieves and
makes example data objects.}
\usage{
make.getExample()
}
\value{
This returns the function which retrieves
and makes example data objects. The latter saves any
object it has found for quicker return.  Using the value 'list'
causes it to return the list of all objects found so far.
}
\description{
This function makes the function whichretrieves and
makes example data objects.
}
\examples{
getExample = make.getExample()
## Provided:
smallXenium = getExample('smallXenium')
## Computed:
delaunayNeighbours = getExample('delaunayNeighbours')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGenesVsCells.R
\name{getGeneClusterAveragesPerCell}
\alias{getGeneClusterAveragesPerCell}
\title{This produces a matrix giving the average expression of gene
clusters in cells.  By default, it uses all cells and all gene
clusters.}
\usage{
getGeneClusterAveragesPerCell(
  f,
  fPrime,
  cells = colnames(f),
  geneClusters = getClusterOrder(fPrime),
  layer = "data"
)
}
\arguments{
\item{f}{\itemize{
\item the cell Seurat object or SingleCellExperiment to be
turned into a Seurat object
}}

\item{fPrime}{\itemize{
\item the genes Seurat object or SingleCellExperiment to be
turned into a Seurat object
}}

\item{cells}{\itemize{
\item the cells to compute this for
}}

\item{geneClusters}{\itemize{
\item the geneClusters to compute average
expression for
}}

\item{layer}{\itemize{
\item the data layer to use, defaults to 'data'
}}
}
\value{
A matrix where the rows correspond to cells, the columns
correspond to geneClusters and the entries give average expression
for each cluster in each cell
}
\description{
This produces a matrix giving the average expression of gene
clusters in cells.  By default, it uses all cells and all gene
clusters.
}
\examples{
getExample = make.getExample()
exSeuratObj = getExample('exSeuratObj',toy=TRUE)
STranspose = getExample('STranspose',toy=TRUE)
clusterExpression = getGeneClusterAveragesPerCell(exSeuratObj,STranspose)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{makeSummedLRInteractionHeatmap}
\alias{makeSummedLRInteractionHeatmap}
\title{This function takes ligandReceptorResults and plots a heatmap of the total
number of ligand receptor interactions between clusters.}
\usage{
makeSummedLRInteractionHeatmap(
  ligandReceptorResults,
  clusters,
  type,
  logScale = TRUE
)
}
\arguments{
\item{ligandReceptorResults}{\itemize{
\item as returned by performLigandReceptorAnalysis()
}}

\item{clusters}{\itemize{
\item named vector of cell types where names are each cell and
clusters are a factor
}}

\item{type}{\itemize{
\item "total" or "mean" to plot raw total interactions or mean
interactions per edge.
}}

\item{logScale}{\itemize{
\item plot heatmap using log scale (defaults to TRUE)
}}
}
\value{
matrix of total ligand receptor interactions that underlies t
he heatmap.
}
\description{
This function takes ligandReceptorResults and plots a heatmap of the total
number of ligand receptor interactions between clusters.
}
\examples{
getExample = make.getExample()
clusters = getExample('clusters')
ligandReceptorResults = getExample('ligandReceptorResults')
cellTypePerCellTypeLigRecMatrix = 
makeSummedLRInteractionHeatmap(ligandReceptorResults, clusters, "mean")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGenesVsCells.R
\name{meanZPerClusterOnUMAP}
\alias{meanZPerClusterOnUMAP}
\title{This collects together mean z-score data together with
UMAP coordinates from the gene seurat object for plotting.}
\usage{
meanZPerClusterOnUMAP(f, fPrime, clusterBy = "seurat_clusters", layer = "data")
}
\arguments{
\item{f}{\itemize{
\item a Seurat object of cells or SingleCellExperiment to
be converted to a Seurat object
}}

\item{fPrime}{\itemize{
\item the corresponding Seurat object of genes
SingleCellExperiment to be converted to a Seurat object
}}

\item{clusterBy}{\itemize{
\item the name of the column of f@meta.data to
be used to subset the cells
}}

\item{layer}{\itemize{
\item the data layer to be used for z-scores
}}
}
\value{
This returns a data frame with the UMAP coordinates
of the gene Seurat object and the average z-score for each
gene within each of the cell clusters defined by the clusterBy
column of the meta.data of f.
}
\description{
This collects together mean z-score data together with
UMAP coordinates from the gene seurat object for plotting.
}
\examples{
getExample = make.getExample()
exSeuratObj = getExample('exSeuratObj',toy=TRUE)
STranspose = getExample('STranspose',toy=TRUE)
df = meanZPerClusterOnUMAP(exSeuratObj,STranspose,clusterBy='shortName')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleEdgeQualityControl.R
\name{edgeCutoffsByClustering}
\alias{edgeCutoffsByClustering}
\title{This finds proposed cutoffs for edge lengths by clustering
the lengths of the edges for each cell type pair using k-means
clustering with k  = 2}
\usage{
edgeCutoffsByClustering(annEdges)
}
\arguments{
\item{annEdges}{\itemize{
\item a data frame with columns nodeA, nodeB, length
and cellTypePair as produced by edgeLengthsAndCellTypePairs.
}}
}
\value{
This returns a data frame with columns cellTypePair and
cutoff.
}
\description{
This finds proposed cutoffs for edge lengths by clustering
the lengths of the edges for each cell type pair using k-means
clustering with k  = 2
}
\examples{
getExample = make.getExample()
centroids = getExample('centroids')
clusters = getExample('clusters')
delaunayNeighbours = getExample('delaunayNeighbours')
annEdges = 
    edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
cutoffDF = edgeCutoffsByClustering(annEdges)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{predictAnnotationAllGenes}
\alias{predictAnnotationAllGenes}
\title{This function predicts the functions of all genes based
on the functions of their neighbours.}
\usage{
predictAnnotationAllGenes(
  geneSets,
  fPrime,
  radius,
  metric = "umap",
  normaliseByGeneSet = TRUE,
  normaliseByDistance = TRUE,
  normaliseToUnitVector = TRUE
)
}
\arguments{
\item{geneSets}{\itemize{
\item a set of gene sets, e.g., hallmark
}}

\item{fPrime}{\itemize{
\item a transposed Seurat object (generated with
transposeObject()) or SingleCellExperiment to
be converted to a Seurat object
}}

\item{radius}{\itemize{
\item radius of the region to use for prediction
}}

\item{metric}{\itemize{
\item reduction or NN, defaults to umap
}}

\item{normaliseByGeneSet}{\itemize{
\item normalise by size of each gene set,
defaults to TRUE
}}

\item{normaliseByDistance}{\itemize{
\item attenutate neighbour contributions
based on distance, defaults to TRUE
}}

\item{normaliseToUnitVector}{\itemize{
\item return results as unit
vectors, defaults to TRUE
}}
}
\value{
\itemize{
\item A list where names are genes and values are vectors
of gene annotations whose entries correspond to the geneSets
}
}
\description{
This function predicts the functions of all genes based
on the functions of their neighbours.
}
\examples{
getExample = make.getExample()
STranspose = getExample('STranspose',toy=TRUE)
hallmark = getExample('hallmark',toy=TRUE)
predictions = predictAnnotationAllGenes(hallmark,STranspose,radius=.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{cellTypesPerCellTypeGraphFromNbhdMatrix}
\alias{cellTypesPerCellTypeGraphFromNbhdMatrix}
\title{This function takes a neighbourhood-by-cell type
matrix and produces a directed igraph showing the
fractions of cells of each type in the neighbourhoods
around cells of each type.}
\usage{
cellTypesPerCellTypeGraphFromNbhdMatrix(
  nbhdByCellType,
  clusters,
  colours = NULL,
  selfEdges = FALSE,
  minWeight = 0,
  edgeWeighting = 20,
  edgeCurved = 0.2,
  arrowSize = 4,
  arrowWidth = 4,
  plotGraph = TRUE
)
}
\arguments{
\item{nbhdByCellType}{\itemize{
\item A matrix whose rows are neighbourhoods
each denoted by the cell at their center, whose columns are
cell types, and whose entries are counts.
}}

\item{clusters}{\itemize{
\item a named vector whose names are the cells
and whose entries are their seurat_clusters.
}}

\item{colours}{\itemize{
\item a named vector of colours used to colour the
vertices of the graph.  The names are the seurat_clusters
as character strings.
}}

\item{selfEdges}{\itemize{
\item a logical which determines whether to include
self edges.  Defaults to FALSE
}}

\item{minWeight}{\itemize{
\item Allows one to exclude edges of low weight.
Defaults to 0, thus including all edges.
}}

\item{edgeWeighting}{\itemize{
\item a parameter used to thicken the edges
in the display.  Defaults to 20.
}}

\item{edgeCurved}{\itemize{
\item a parameter to set curvature of the edges.
Defaults to 0.2
}}

\item{arrowSize}{\itemize{
\item a parameter to set arrow size. Defaults to 4.
}}

\item{arrowWidth}{\itemize{
\item a parameter to set arrow width. Defaults to 4.
}}

\item{plotGraph}{\itemize{
\item a logical which determines whether to
plot the graph.  Defaults to TRUE.
}}
}
\value{
This returns a directed igraph whose vertices are
the cell types and whose arrows indicate "ownership" of
cells of the target type by neighbourhoods of cells of the
source type.  Layout is done witht the FR algorithm and
coordinates are found in the coords attribute of G.  If colours
were supplied these are found in the color attribute of V(G).
Edge weights and widths are found in the weight and width
attributes of E(G).
}
\description{
This function takes a neighbourhood-by-cell type
matrix and produces a directed igraph showing the
fractions of cells of each type in the neighbourhoods
around cells of each type.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{makeLRInteractionHeatmap}
\alias{makeLRInteractionHeatmap}
\title{This function takes ligandReceptorResults and plots a heatmap of -log10(pvalues).
If the minimum p-value is 0 a pseudocount of 0.001 will be added before log
transformation.}
\usage{
makeLRInteractionHeatmap(
  ligandReceptorResults,
  clusters,
  colours = c(),
  pValCutoffClusterPair = 0.05,
  pValCutoffLigRec = 0.05,
  labelClusterPairs = TRUE
)
}
\arguments{
\item{ligandReceptorResults}{\itemize{
\item as returned by performLigandReceptorAnalysis()
}}

\item{clusters}{\itemize{
\item named vector of cell types where names are each cell and
clusters are a factor
}}

\item{colours}{\itemize{
\item a named list of colours where names are clusters. If not
specified the default pheatmap colour scheme will be used.
}}

\item{pValCutoffClusterPair}{\itemize{
\item a cutoff for showing interactions between two
clusters. A cluster pair must have at least one ligand-receptor interaction
pvalue <  pValCutoffClusterPair. Defaults to 0.05.
}}

\item{pValCutoffLigRec}{\itemize{
\item a cutoff for showing interactions between a
ligand and receptor. At least one cluster pair must have
pvalue <  pValCutoffLigRec for ligand-receptor pair. Defaults to 0.05.
}}

\item{labelClusterPairs}{\itemize{
\item show labels for cluster pairs. Defaults to TRUE.
}}
}
\value{
matrix of -log10(pvalues) that underlies the heatmap.
}
\description{
This function takes ligandReceptorResults and plots a heatmap of -log10(pvalues).
If the minimum p-value is 0 a pseudocount of 0.001 will be added before log
transformation.
}
\examples{
getExample = make.getExample()
clusters = getExample('clusters')
ligandReceptorResults = getExample('ligandReceptorResults')
cellTypePerCellTypeLigRecMatrix = 
makeSummedLRInteractionHeatmap(ligandReceptorResults, clusters, "mean")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{computeNeighbourEnrichment}
\alias{computeNeighbourEnrichment}
\title{This function calculates P values for whether cell types are more frequently
neighbours than expected by chance. By default it calculates P values
analytically using a hypergeometric test on the edges, where the arguments
to the  R phyper function are as follows:
q = number of edges between cell type A and B
m = number of edges between cell type B and any other cell type
n = the number of edges between any cell type apart from cell type B
k =  number of edges between cell type B and any other cell type
The purist may object to the use of the hypergeometric test here.
We may think of "edges out of a cell of type A" as being the random draw
balls (here, edges) from the urn and "edges out of cells of type B" as being
success. However, all edges out of a given cell of type A are in this
"random draw". Clearly the edges in this draw are not independent. However,
empirically we find that p-values computed using this method correspond very
closely to those computed using permutation while the computation time is
orders of magnitude faster.}
\usage{
computeNeighbourEnrichment(
  spatialGraph,
  cellTypes,
  method = "analytical",
  nSim = 1000,
  maxTries = 1000,
  randomiseBy = "cells",
  verbose = TRUE
)
}
\arguments{
\item{spatialGraph}{\itemize{
\item a spatial graph in neighbour list format.
}}

\item{cellTypes}{\itemize{
\item named vector of cell types where names are each cell and
cell types are a factor.
}}

\item{method}{\itemize{
\item method for computing p-values. Defaults to "analytical", in
which case an edge-based hypergeometric test is performed. If "permutation"
is selected p-values are calculated by comparison to randomised graphs (note
this is slower than the analytical approach).
}}

\item{nSim}{\itemize{
\item the number of randomised graphs to create for pvalue
calculation, if the method is set to permutation.
}}

\item{maxTries}{\itemize{
\item the maximum number of tries to remove self edges during
graph randomisation. If self edges are remeining this will be reported.
}}

\item{randomiseBy}{\itemize{
\item This takes either the value 'cells' (the default)
or 'graph'.  In the former case randomisation is carried out by permuting
the cell types on the existing graph.  In the latter case, the graph is
permuted using the function randomiseGraph() which is a heuristic
algorithm to preserve the distribution of vertex degrees.
}}

\item{verbose}{\itemize{
\item whether to print trace.  Defaults to TRUE
}}
}
\value{
A square matrix containing upper tail p values describing
whether two  cell types are more frequently found together than
expected by chance.
}
\description{
For legacy purposes, and for user flexibility, it allows for the
calculation of P values by comparison to randomised graphs.
It offers two distinct randomisations.
One is by permuting the cell types on the neighbour (e.g., delaunay)
graph.  The other is by comparison to randomised  neighbour graphs
where edges are randomised but the degree of each node is
preserved.
}
\examples{
getExample = make.getExample()
delaunayNeighbours = getExample('delaunayNeighbours')
clusters = getExample('clusters')
cellTypesPerCellTypePValues = computeNeighbourEnrichment(delaunayNeighbours, 
                                        clusters, verbose = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeometricClustering.R
\name{runGeometricClusteringTrials}
\alias{runGeometricClusteringTrials}
\title{This runs random trials to determine the statistical
significance of the clustering of a set of points
within a larger set.}
\usage{
runGeometricClusteringTrials(S, geneSubset, numTrials)
}
\arguments{
\item{S}{\itemize{
\item a set of points given as a matrix. The rows
are the coordinates of these points
}}

\item{geneSubset}{\itemize{
\item this is either a subset of the rownames of
S or a logical whose length is nrow(S)
}}

\item{numTrials}{\itemize{
\item the number or random trials to perform
}}
}
\value{
This returns a list. subsetDistance gives the
median complement distance for the actual set,
randomSubsetDistance gives the complement distances for
the numTrials random sets, pValue gives a p-value based
on the rank of the actual distance among the random
distances and zScore gives its z-score.
}
\description{
This function takes a matrix whose rows are geometric
coordinates and a subset of these points either given
as a character vector which is a subset of the rownames
or as a logical vector.  It returns statistics on the
mean distance of the complement to the subset.
}
\examples{
library(Seurat)
getExample = make.getExample()
STranspose = getExample('STranspose',toy=TRUE)
hallmark = getExample('hallmark')
S = data.matrix(FetchData(STranspose,c('umap_1','umap_2')))
geneSubset = rownames(S) \%in\% hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]]
geneClustering = runGeometricClusteringTrials(S,geneSubset,100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{aggregateFeatureMatrix}
\alias{aggregateFeatureMatrix}
\title{This function takes a matrix where rows are features and columns are cells,
and a neighbourhood list, and creates an matrix where columns are the
neighbourhoods, the rows are are the features and the values are aggregated
expression values for cells in each neighbourhood.}
\usage{
aggregateFeatureMatrix(M, nbhdList, aggregateFunction)
}
\arguments{
\item{M}{\itemize{
\item a matrix where column names are cells and row names are
features.
}}

\item{nbhdList}{\itemize{
\item a named list with memberships of the neighbourhoods
of cells
}}

\item{aggregateFunction}{\itemize{
\item a function to aggregate expression (e.g. rowSums,
rowMeans)
}}
}
\value{
a matrix giving aggregated gene expression for a cell's neighbourhood.
}
\description{
This function takes a matrix where rows are features and columns are cells,
and a neighbourhood list, and creates an matrix where columns are the
neighbourhoods, the rows are are the features and the values are aggregated
expression values for cells in each neighbourhood.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{computeNBHDByCTMatrix}
\alias{computeNBHDByCTMatrix}
\title{This function computes a matrix where neighbourhoods are rows and
cell types are columns. The values in the matrix indicate the
number of cells of a given type within a neighbourhood.}
\usage{
computeNBHDByCTMatrix(spatialGraph, cellTypes)
}
\arguments{
\item{spatialGraph}{\itemize{
\item a spatial graph in neighbour list format.
}}

\item{cellTypes}{\itemize{
\item named vector of cell types where names are each cell and
cell types are a factor
}}
}
\value{
a matrix of neighbourhoods by cell types
}
\description{
This function computes a matrix where neighbourhoods are rows and
cell types are columns. The values in the matrix indicate the
number of cells of a given type within a neighbourhood.
}
\examples{
getExample = make.getExample()
clusters = getExample('clusters')
delaunayNeighbours = getExample('delaunayNeighbours')
NBHDByCTMatrix = computeNBHDByCTMatrix(delaunayNeighbours,clusters)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleEdgeQualityControl.R
\name{edgeCutoffsByZScore}
\alias{edgeCutoffsByZScore}
\title{This finds edge cutoffs by z-score}
\usage{
edgeCutoffsByZScore(annEdges, zCutoff)
}
\arguments{
\item{annEdges}{\itemize{
\item a data frame with columns nodeA, nodeB, length
and cellTypePair as produced by edgeLengthsAndCellTypePairs.
}}

\item{zCutoff}{\itemize{
\item a numeric
}}
}
\value{
This returns a data frame with columns cellTypePair and
cutoff.
}
\description{
This finds edge cutoffs by z-score
}
\examples{
getExample = make.getExample()
centroids = getExample('centroids')
clusters = getExample('clusters')
delaunayNeighbours = getExample('delaunayNeighbours') 
annEdges =
    edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
cutoffDF = edgeCutoffsByZScore(annEdges,zCutoff=1.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGenesVsCells.R
\name{getAverageExpressionMatrix}
\alias{getAverageExpressionMatrix}
\title{This computes average expression of each gene cluster in
each cell cluster and returns the result as a matrix}
\usage{
getAverageExpressionMatrix(
  f,
  fPrime,
  clusteringName = "seurat_clusters",
  layer = "scale.data"
)
}
\arguments{
\item{f}{\itemize{
\item The Seurat object of cells, or SingleCellExperiment
to be turned into a Seurat object
}}

\item{fPrime}{\itemize{
\item The Seurat object of genes,  or SingleCellExperiment
to be turned into a Seurat object
}}

\item{clusteringName}{In many cases, this will be the cell
clustering, i.e., seurat_clusters, which is the default, but
for neighbourhood Seurat objects, this can be
neighbourhood_clusters.}

\item{layer}{\itemize{
\item layer to use for expression values
}}
}
\value{
A matrix of the average expression where the rows
correspond to cell clusters and the columns correspond to
gene clusters.
}
\description{
This computes average expression of each gene cluster in
each cell cluster and returns the result as a matrix
}
\examples{
getExample = make.getExample()
STranspose = getExample('STranspose',toy=TRUE)
exSeuratObj = getExample('exSeuratObj',toy=TRUE)
M = getAverageExpressionMatrix(exSeuratObj,STranspose,layer='data')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGenesVsCells.R
\name{sankeyFromMatrix}
\alias{sankeyFromMatrix}
\title{This makes a sankey graph from a matrix of average
expression.  Our "Cat's Cradle".}
\usage{
sankeyFromMatrix(
  M,
  disambiguation = c("R_", "C_"),
  fontSize = 20,
  minus = "red",
  plus = "blue",
  height = 1200,
  width = 900
)
}
\arguments{
\item{M}{\itemize{
\item a matrix of gene expression
}}

\item{disambiguation}{\itemize{
\item used to distinguish between
the row names and the column names if these overlap
}}

\item{fontSize}{\itemize{
\item defaults to 20
}}

\item{minus}{\itemize{
\item colour to use for links with negative
values
}}

\item{plus}{\itemize{
\item colour for positive values
}}

\item{height}{\itemize{
\item height in pixels, defaults to 1200
}}

\item{width}{\itemize{
\item width in pixels, defaults to 900
}}
}
\value{
A sankey graph
}
\description{
This makes a sankey graph from a matrix of average
expression.  Our "Cat's Cradle".
}
\examples{
set.seed(100)
M = matrix(runif(12)-.3,nrow=3)
rownames(M) = as.character(seq_len(3))
colnames(M) = as.character(seq_len(4))
sankey = sankeyFromMatrix(M)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeometricClustering.R
\name{getSubsetComponents}
\alias{getSubsetComponents}
\title{This is designed to dectect the components of a
gene subset in the case where median complement
distance detects clustering.}
\usage{
getSubsetComponents(fPrime, theSubset, alpha = 0.5, edgeCut = NA)
}
\arguments{
\item{fPrime}{\itemize{
\item a gene Seurat object or
SingleCellExperiment
}}

\item{theSubset}{\itemize{
\item a subset of the genes
}}

\item{alpha}{\itemize{
\item a parameter typically less than
one controling the granularity of the components.
Defaults to .5
}}

\item{edgeCut}{\itemize{
\item the maximum length of edges included in
the subgraph whose components are returned.  If it
is NA (the default) it is computed using alpha.
Otherwise, it can be supplied directly.
}}
}
\value{
A list of the components of the subset
treated as a graph whose edges are determined by
their distance in UMAP coordinates.
}
\description{
This is designed to dectect the components of a
gene subset in the case where median complement
distance detects clustering.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGenesVsCells.R
\name{meanZPerCluster}
\alias{meanZPerCluster}
\title{This finds the mean z-score for features in subsets
of cells e.g., in each of the seurat_clusters}
\usage{
meanZPerCluster(f, features, clusterBy = "seurat_clusters", layer = "data")
}
\arguments{
\item{f}{\itemize{
\item a Seurat object of cells or SingleCellExperiment to
be converted to a Seurat object
}}

\item{features}{\itemize{
\item a set of features of f
}}

\item{clusterBy}{\itemize{
\item the name of the column of f@meta.data to
be used to subset the cells
}}

\item{layer}{\itemize{
\item the data layer to be used for z-scores
}}
}
\value{
This returns a data frame each of whose columns
corresponds to a value of the clusterBy data.  In the case
where the clusterBy data is a factor or numeric, it prepends
cluster_ to the column name.
}
\description{
This finds the mean z-score for features in subsets
of cells e.g., in each of the seurat_clusters
}
\examples{
getExample = make.getExample()
exSeuratObj = getExample('exSeuratObj',toy=TRUE)
STranspose = getExample('STranspose',toy=TRUE)
df = meanZPerCluster(exSeuratObj,features=colnames(STranspose),
                     clusterBy='shortName')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{seuratGenes}
\alias{seuratGenes}
\title{seuratGenes}
\format{
A vector of genes

\describe{
A vector of the top 100 most variable genes in exSeuratObj
used to subset this object to give toy examples.
}
}
\source{
Computed by retrieving the data layer from exSeuratObj and
subsetting to the 100 genes with the highest standard deviation.
}
\usage{
seuratGenes
}
\description{
A vector of genes used for subsetting exSeuratObj
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{performLigandReceptorAnalysis}
\alias{performLigandReceptorAnalysis}
\title{Given a seurat object, a spatial graph, clusters and
species this function identifies ligand-receptor
interactions between neighbouring cells, identifies
ligand-receptor interactions within and between clusters
and calculates whether these are observed more frequently
than expected by chance. If the "analytical" method is selected, an upper
tail p-value for observing a given number of A-B edges positive for a given
interaction is calculated using a binomial test (R pbinom function) where:
q = number of A-B edges positive for an interaction
size = total number of A-B edges
prob = pL*pR
Where pL is the probability of a cell expressing a specific ligand (number
of cells positive for a ligand/total cells),
and pR is the probability of a cell expressing a specific receptor (number
of cells positive for a receptor/total cells).
If conditional = True  p-values will be calculated given the proportion of
cells that express ligands and receptors in the specific clusters (pL =
number of cells in cluster A positive for a ligand/number of cells in cluster
A, pR = number of cells in cluster B positive for a receptor/number of cells
in cluster B).

We recommend to use the analytical method, which has a much faster runtime
than the permutation-based method, however for legacy purposes and user
flexibility we retain the permutation-based method.}
\usage{
performLigandReceptorAnalysis(
  obj,
  spatialGraph,
  species,
  clusters,
  method = "analytical",
  conditional = FALSE,
  minEdgesPos = 10,
  nSim = 1000,
  lrn = getLigandReceptorNetwork(species),
  verbose = TRUE
)
}
\arguments{
\item{obj}{\itemize{
\item a Seurat object
}}

\item{spatialGraph}{\itemize{
\item a data frame of neighbouring
cell pairs.
}}

\item{species}{\itemize{
\item either 'human' or 'mouse'
}}

\item{clusters}{\itemize{
\item named vector of clusters where names are each cell and
clusters are a factor
}}

\item{method}{\itemize{
\item method for computing p-values. Defaults to "analytical".
If "permutation" is selected p-values are calculated by comparison to
randomised graphs (note this is slower than the analytical approach).
}}

\item{conditional}{\itemize{
\item if method is "analytical" and conditional is true,
p-values will be calculated given the proportion of cells that express
ligands and receptors in the specific clusters. Otherwise global proportions
of ligand and receptor expression are used. Defaults to FALSE.
}}

\item{minEdgesPos}{\itemize{
\item the minimum edges that need to be positive for a
ligand-receptor interaction between two clusters for a p-value to be
calculated. Only taken into consideration when the analytical method is
selected.
}}

\item{nSim}{\itemize{
\item number of simulations to perform for pvalue calculation.
}}

\item{lrn}{\itemize{
\item a ligand-receptor network, i.e., a
data frame with columns from and to.  By default, it
retrieves the nichenetr ligand receptor network
}}

\item{verbose}{\itemize{
\item whether to print trace, defaults to TRUE
}}
}
\value{
A list containing:
interactionsOnEdges - a sparse matrix where the rownames give pairs of
neighbouring cells and column names give ligand-receptor pairs.
Entries are TRUE if the ligand is expressed in the first cell and the receptor is expressed in
the second cell and FALSE if not.
interactionsOnEdgesMeta - a dataframe where the first two columns are the
cells that comprise the edges in interactionsOnEdges, and the next two
columns are their clusters.
totalInteractionsByCluster - a dataframe where the rownames are
sender-receiver cluster pairs and column names are ligand receptor pairs.
Entries are total numbers of edges on which particular ligand receptor
interactions are present.
meanInteractionsByCluster - a dataframe where the rownames are
sender-receiver cluster pairs and column names are ligand receptor pairs.
Entries are total numbers of edges on which particular ligand receptor i
nteractions are present (for that cluster pair) divided by the total number
of edges between those clusters.
simResults - a dataframe where the rownames are sender-receiver cluster
pairs and column names are ligand receptor pairs.
Values give the number of simulations for which observed values are greater
than simulated values. Only returned if method = "permutation".
pValues - a dataframe where the rownames are sender-receiver cluster pairs
and column names are ligand receptor pairs. Entries are uppertail p-values
describing whether a particular ligand receptor interaction is observed more
frequently between 2 clusters than expected.
totalEdges - a vector of total edges between cluster pairs.
}
\description{
Given a seurat object, a spatial graph, clusters and
species this function identifies ligand-receptor
interactions between neighbouring cells, identifies
ligand-receptor interactions within and between clusters
and calculates whether these are observed more frequently
than expected by chance. If the "analytical" method is selected, an upper
tail p-value for observing a given number of A-B edges positive for a given
interaction is calculated using a binomial test (R pbinom function) where:
q = number of A-B edges positive for an interaction
size = total number of A-B edges
prob = pL*pR
Where pL is the probability of a cell expressing a specific ligand (number
of cells positive for a ligand/total cells),
and pR is the probability of a cell expressing a specific receptor (number
of cells positive for a receptor/total cells).
If conditional = True  p-values will be calculated given the proportion of
cells that express ligands and receptors in the specific clusters (pL =
number of cells in cluster A positive for a ligand/number of cells in cluster
A, pR = number of cells in cluster B positive for a receptor/number of cells
in cluster B).

We recommend to use the analytical method, which has a much faster runtime
than the permutation-based method, however for legacy purposes and user
flexibility we retain the permutation-based method.
}
\examples{
getExample = make.getExample()
smallXenium = getExample('smallXenium')
delaunayNeighbours = getExample('delaunayNeighbours')
clusters = getExample('clusters')
performLigandReceptorAnalysis(smallXenium, delaunayNeighbours, 
                                      "mouse", clusters)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeometricClustering.R
\name{getObjectSubsetClusteringStatistics}
\alias{getObjectSubsetClusteringStatistics}
\title{This function computes statistics for the geometric
clustering of a gene set (in UMAP or PCA reduction)
based on the median distance from its complement to
the set.}
\usage{
getObjectSubsetClusteringStatistics(
  fPrime,
  geneSubset,
  numTrials = 1000,
  reduction = "UMAP",
  numPCs = 10
)
}
\arguments{
\item{fPrime}{\itemize{
\item a transposed Seurat object, i.e. a
Seurat object of genes or SingleCellExperiment to
be converted to a Seurat object
}}

\item{geneSubset}{\itemize{
\item a subset of the genes which can
be given as a character vector or as a logical vector
}}

\item{numTrials}{\itemize{
\item the number of random trials to be
carried out for randomised testing. Defaults to 1000.
}}

\item{reduction}{\itemize{
\item can be 'UMAP' or 'PCA', defaults
to 'UMAP'
}}

\item{numPCs}{\itemize{
\item number of PCs to use if reduction is
'PCA'
}}
}
\value{
A list of statistics resulting from the
testing of randomised subsets of the same size as the
given gene subset.  These include subsetDistance, the
actual median complement distance; randomSubsetDistance,
the median complement distances for randomised subsets;
pValue, computed by comparing the real and randomised
distances; and zScore, the z-distance of the actual
median distance from the mean of the randomised distances.
}
\description{
This function computes statistics for the geometric
clustering of a gene set (in UMAP or PCA reduction)
based on the median distance from its complement to
the set.
}
\examples{
getExample = make.getExample()
STranspose = getExample('STranspose',toy=TRUE)
hallmark = getExample('hallmark')
geneSubset = intersect(colnames(STranspose),hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]])
stats = getObjectSubsetClusteringStatistics(STranspose,geneSubset,100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeometricClustering.R
\name{directedHausdorfDistance}
\alias{directedHausdorfDistance}
\title{This finds the directed Hausdorf distance from A to B}
\usage{
directedHausdorfDistance(A, B)
}
\arguments{
\item{A}{\itemize{
\item an m x d matrix representing m points in
dimension d
}}

\item{B}{\itemize{
\item an n x d matrix representing n points in
dimension d
}}
}
\value{
This returns the distance of the furthest point
in A from its nearest point in B.
}
\description{
This finds the directed Hausdorf distance from A to B
}
\examples{
A = matrix(seq_len(8),ncol=2)
B = matrix(seq(from=3,to=16),ncol=2)
d_hausdorf = directedHausdorfDistance(A,B)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{getLigandReceptorPairsInPanel}
\alias{getLigandReceptorPairsInPanel}
\title{This functions takes an Seurat object, its species
and a ligand receptor network and subsets the ligand
receptor network to those pairs that occur in the
panel}
\usage{
getLigandReceptorPairsInPanel(
  obj,
  species,
  lrn = getLigandReceptorNetwork(species)
)
}
\arguments{
\item{obj}{\itemize{
\item a Seurat object or SingleCellExperiment to
be converted to a Seurat object
}}

\item{species}{\itemize{
\item either 'human' or 'mouse'
}}

\item{lrn}{\itemize{
\item a ligand-receptor network, i.e., a
data frame with columns from and to.  By default, it
retrieves the nichenetr ligand receptor network
}}
}
\value{
This returns a data frame with columns ligand and
receptor
}
\description{
This functions takes an Seurat object, its species
and a ligand receptor network and subsets the ligand
receptor network to those pairs that occur in the
panel
}
\examples{
smallXenium = make.getExample()('smallXenium')
lrPairs = getLigandReceptorPairsInPanel(smallXenium, "mouse")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{moransI}
\alias{moransI}
\title{moransI}
\format{
A data fame containing Moran's I and related pvalues.

\describe{
Moran's I values calculated for the genes in smallXenium (using the SCT
assay). Pvalues derived using 100 permutations.
}
}
\source{
Created from smallXenium and delaunayNeighbours by using
runMoransI()
}
\usage{
moransI
}
\description{
A data fame containing Moran's I and related pvalues.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGenesVsCells.R
\name{getAverageExpressionDF}
\alias{getAverageExpressionDF}
\title{This converts an average gene expression matrix to a
data frame.}
\usage{
getAverageExpressionDF(M)
}
\arguments{
\item{M}{\itemize{
\item An average gene expression matrix.
}}
}
\value{
A data frame with columns cellCluster, geneCluster
and average expression
}
\description{
This converts an average gene expression matrix to a
data frame.
}
\examples{
getExample = make.getExample()
averageExpMatrix = getExample('averageExpMatrix',toy=TRUE)
averageExpDF = getAverageExpressionDF(averageExpMatrix)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{formatData}
\alias{formatData}
\title{This is a utility function for converting entries in ligandReceptorResults
to long format.}
\usage{
formatData(data, name)
}
\arguments{
\item{data}{\itemize{
\item item from ligandReceptorResults
}}

\item{name}{\itemize{
\item name to give column of returned data
}}
}
\value{
dataframe with item from ligandReceptorResults in long format
}
\description{
This is a utility function for converting entries in ligandReceptorResults
to long format.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{permuteMatrix}
\alias{permuteMatrix}
\title{This function permutes the rows of a matrix.}
\usage{
permuteMatrix(M)
}
\arguments{
\item{M}{\itemize{
\item a binarised expression matrix where rows are genes and columns
}}
}
\value{
This returns a matrix in which the values have been permuted within
rows.
}
\description{
This function permutes the rows of a matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{computeCellTypesPerCellTypeMatrix}
\alias{computeCellTypesPerCellTypeMatrix}
\title{For each cell type, this function looks at the neighbourhoods
around cells of that type and discovers the fractions (or numbers if
normalise = F) of those cells of each type.}
\usage{
computeCellTypesPerCellTypeMatrix(nbhdByCellType, cellTypes, normalise = TRUE)
}
\arguments{
\item{nbhdByCellType}{\itemize{
\item A matrix whose rows are neighbourhoods
each denoted by the cell at their center, whose columns are
cell types, and whose entries are counts.
}}

\item{cellTypes}{\itemize{
\item named vector of cell types where names are each cell and
cell types are a factor
}}

\item{normalise}{\itemize{
\item boolean, defaults to TRUE
}}
}
\value{
A square matrix whose rownames and colnames are the
seurat_clusters as character strings.  Each row corresponds
to neighbourhoods around all cells of that type and the entries
give the fractions of those neighbourhoods occupied by cells
of each type.
}
\description{
For each cell type, this function looks at the neighbourhoods
around cells of that type and discovers the fractions (or numbers if
normalise = F) of those cells of each type.
}
\examples{
getExample = make.getExample()
NBHDByCTMatrix = getExample('NBHDByCTMatrix')
clusters = getExample('clusters')
cellTypesPerCellType = computeCellTypesPerCellTypeMatrix(NBHDByCTMatrix,clusters)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{humanLRN}
\alias{humanLRN}
\title{humanLRN}
\format{
a data frame with two columns, 'from' and 'to'
\describe{A data frame with two columns, 'from' and 'to'.
Each row represents a human ligand - receptor pair.
}
}
\source{
This is taken from the nichenetr package,
url = {https://www.nature.com/articles/s41592-019-0667-5}.
Specifically we use the human ligand - receptor network.
}
\usage{
humanLRN
}
\description{
A data frame giving 12019 human ligand receptor pairs
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{performLigandReceptorAnalysisAnalytical}
\alias{performLigandReceptorAnalysisAnalytical}
\title{Given a seurat object, a spatial graph, clusters and
species this function identifies ligand-receptor
interactions between neighbouring cells, identifies
ligand-receptor interactions within and between clusters
and calculates whether these are observed more frequently
than expected by chance using an analytical approach.}
\usage{
performLigandReceptorAnalysisAnalytical(
  obj,
  spatialGraph,
  species,
  clusters,
  conditional = FALSE,
  lrn = getLigandReceptorNetwork(species),
  minEdgesPos = 10
)
}
\arguments{
\item{obj}{\itemize{
\item a Seurat object
}}

\item{spatialGraph}{\itemize{
\item a data frame of neighbouring
cell pairs.
}}

\item{species}{\itemize{
\item either 'human' or 'mouse'
}}

\item{clusters}{\itemize{
\item named vector of clusters where names are each cell and
clusters are a factor
}}

\item{conditional}{\itemize{
\item if method is "analytical" and conditional is true,
p-values will be calculated given the proportion of cells that express
ligands and receptors in the specific clusters. Otherwise global proportions
of ligand and receptor expression are used. Defaults to FALSE.
}}

\item{lrn}{\itemize{
\item a ligand-receptor network, i.e., a
data frame with columns from and to.  By default, it
retrieves the nichenetr ligand receptor network
}}

\item{minEdgesPos}{\itemize{
\item the minimum edges that need to be positive for a
ligand-receptor interaction between two clusters for a p-value to be
calculated.
}}
}
\value{
A list containing:
interactionsOnEdges - a sparse matrix where the rownames give pairs of
neighbouring cells and column names give ligand-receptor pairs.
Entries are TRUE if the ligand is expressed in the first cell and the receptor is expressed in
the second cell and FALSE if not.
interactionsOnEdgesMeta - a dataframe where the first two columns are the
cells that comprise the edges in interactionsOnEdges, and the next two
columns are their clusters.
totalInteractionsByCluster - a dataframe where the rownames are
sender-receiver cluster pairs and column names are ligand receptor pairs.
Entries are total numbers of edges on which particular ligand receptor
interactions are present.
meanInteractionsByCluster - a dataframe where the rownames are
sender-receiver cluster pairs and column names are ligand receptor pairs.
Entries are total numbers of edges on which particular ligand receptor
interactions are present (for that cluster pair) divided by the total number
of edges between those clusters.
pValues - a dataframe where the rownames are sender-receiver cluster pairs
and column names are ligand receptor pairs. Entries are uppertail p-values
describing whether a particular ligand receptor interaction is observed more
frequently between 2 clusters than expected.
totalEdges - a vector of total edges between cluster pairs.
}
\description{
Given a seurat object, a spatial graph, clusters and
species this function identifies ligand-receptor
interactions between neighbouring cells, identifies
ligand-receptor interactions within and between clusters
and calculates whether these are observed more frequently
than expected by chance using an analytical approach.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoods.R
\name{getExtendedNBHDs}
\alias{getExtendedNBHDs}
\title{This function takes a nearest neighbour graph and a radius
and calculates nth degree neighbour graphs where max(n) == radius}
\usage{
getExtendedNBHDs(spatialGraph, n)
}
\arguments{
\item{spatialGraph}{\itemize{
\item a nearest neighbour graph
}}

\item{n}{\itemize{
\item the maximum degree to calculate a neighbour graph with edges
connecting vertices of degree n for.
}}
}
\value{
A named list of neighbour graphs, where each graph contains edges
connecting vertices of degree n. Each graph is named according to degree n.
}
\description{
This function takes a nearest neighbour graph and a radius
and calculates nth degree neighbour graphs where max(n) == radius
}
\examples{
delaunayNeighbours = make.getExample()('delaunayNeighbours')
extendedNeighboursList = getExtendedNBHDs(delaunayNeighbours, 4)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{smallXenium}
\alias{smallXenium}
\title{smallXenium}
\format{
A Seurat object

\describe{
A spatial Seurat object subset from the Xenium object used in
https://satijalab.org/seurat/articles/seurat5_spatial_vignette_2.
}
}
\source{
This is subset from the Xenium spatial Seurat object
https://cf.10xgenomics.com/samples/xenium/1.0.2/Xenium_V1_FF_Mouse_Brain_Coronal_Subset_CTX_HP/Xenium_V1_FF_Mouse_Brain_Coronal_Subset_CTX_HP_outs.zip
to include a small region of the field of view surrounding the dentate gyrus.
}
\usage{
smallXenium
}
\description{
A spatial Seurat object of 4261 cells and 248 genes
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeometricClustering.R
\name{getObjectSubsetClusteringPValue}
\alias{getObjectSubsetClusteringPValue}
\title{This function computes a p-value for the geometric
clustering of a gene set (in UMAP or PCA reduction)
based on the median distance from its complement to
the set.}
\usage{
getObjectSubsetClusteringPValue(
  fPrime,
  geneSubset,
  numTrials = 1000,
  reduction = "UMAP",
  numPCs = 10
)
}
\arguments{
\item{fPrime}{\itemize{
\item a transposed Seurat object, i.e. a
Seurat object of genes or SingleCellExperiment to
be converted to a Seurat object
}}

\item{geneSubset}{\itemize{
\item a subset of the genes which can
be given as a character vector as a logical vector
}}

\item{numTrials}{\itemize{
\item the number of random trials to be
carried out for randomised testing. Defaults to 1000.
}}

\item{reduction}{\itemize{
\item can be 'UMAP' or 'PCA', defaults
to 'UMAP'
}}

\item{numPCs}{\itemize{
\item number of PCs to use if reduction is
'PCA'
}}
}
\value{
A p-value reporting how often a random subset
of the same size is sufficiently clustered to produce
an equally large distance from its complement.
}
\description{
This function computes a p-value for the geometric
clustering of a gene set (in UMAP or PCA reduction)
based on the median distance from its complement to
the set.
}
\examples{
getExample = make.getExample()
STranspose = getExample('STranspose')
hallmark = getExample('hallmark',toy=TRUE)
geneSubset = intersect(colnames(STranspose),hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]])
p = getObjectSubsetClusteringPValue(STranspose,geneSubset,100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ligandReceptorResults}
\alias{ligandReceptorResults}
\title{ligandReceptorResults}
\format{
A list of data frames.

\describe{
A list containing:
interactionsOnEdges - a data frame whose first two columns give
the neighbouring cells and next two columns give their corresponding
clusters. Each of the remaining columns is a logical
corresponding to a ligand-receptor pair telling whether the ligand
is expressed in the first cell and the receptor is expressed in the
second cell.
totalInteractionsByCluster - a dataframe where the first column gives a
directed (sender-receiver) pair of clusters. The second column gives the
total number of edges between those clusters. The remaining columns give the
total numbers of edges on which particular ligand receptor interactions are
present.
meanInteractionsByCluster - a dataframe where the first column gives a
directed (sender-receiver) pair of clusters. The second column gives the
total number of edges between those clusters. The remaining columns give the
total numbers of edges on which particular ligand receptor interactions are
present (for that cluster pair) divided by the total number of edges between
those clusters.
simResults - a dataframe where the rownames are sender-receiver cluster pairs
and column names are ligand receptor pairs. Values give the number of
simulations for which observed values are greater than simulated values.
pValues - a dataframe where the rownames are sender-receiver cluster pairs
and column names are ligand receptor pairs. Entries are uppertail pvalues
describing whether a particular ligand receptor interaction is observed more
frequently between 2 clusters than expected.
}
}
\source{
Created from smallXenium and delaunayNeighbours by using
performLigandReceptorAnalysis(()
}
\usage{
ligandReceptorResults
}
\description{
The result of performLigandReceptorAnalysis(smallXenium, delaunayNeighbours,
"mouse", clusters,verbose=FALSE)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleEdgeQualityControl.R
\name{edgeCutoffsByPercentile}
\alias{edgeCutoffsByPercentile}
\title{This finds edge cutoffs by percentile}
\usage{
edgeCutoffsByPercentile(annEdges, percentileCutoff)
}
\arguments{
\item{annEdges}{\itemize{
\item a data frame with columns nodeA, nodeB, length
and cellTypePair as produced by edgeLengthsAndCellTypePairs.
}}

\item{percentileCutoff}{\itemize{
\item a numeric
}}
}
\value{
This returns a data frame with columns cellTypePair and
cutoff.
}
\description{
This finds edge cutoffs by percentile
}
\examples{
getExample = make.getExample()
centroids = getExample('centroids')
clusters = getExample('clusters')
delaunayNeighbours = getExample('delaunayNeighbours')
annEdges =
    edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
cutoffDF = edgeCutoffsByPercentile(annEdges,percentileCutoff=95)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{predictAnnotation}
\alias{predictAnnotation}
\title{This function makes annotation predictions for a
set of genes based on gene sets (e.g., hallmark)
and a CatsCradle object by considering the annotations of
its neighboring genes.}
\usage{
predictAnnotation(
  genes,
  geneSets,
  fPrime,
  radius,
  metric = "umap",
  numPCs = NULL,
  normaliseByGeneSet = TRUE,
  normaliseByDistance = TRUE,
  normaliseToUnitVector = TRUE
)
}
\arguments{
\item{genes}{\itemize{
\item a character vector of genes
}}

\item{geneSets}{\itemize{
\item a set of annotations, e.g., hallmark
or GO
}}

\item{fPrime}{\itemize{
\item a Seurat object of genes SingleCellExperiment
to be converted to a Seurat object
}}

\item{radius}{\itemize{
\item radius for prediction neighborhood
}}

\item{metric}{\itemize{
\item reduction or NN, defaults to umap
}}

\item{numPCs}{\itemize{
\item used only if reduction is pca, defaults to NULL
}}

\item{normaliseByGeneSet}{\itemize{
\item determines whether vector annotations
are normalised by gene set size.  Defaults to TRUE
}}

\item{normaliseByDistance}{\itemize{
\item determines whether neighbor
contributions are normalised by edge weight.  Defaults to
TRUE.
}}

\item{normaliseToUnitVector}{\itemize{
\item determines whether to normalise
returned values to unit length.  Defaults to TRUE
}}
}
\value{
This returns a list of prediction vectors, one vector
for each gene in genes, each vector corresponding to the sets
in geneSets
}
\description{
This function makes annotation predictions for a
set of genes based on gene sets (e.g., hallmark)
and a CatsCradle object by considering the annotations of
its neighboring genes.
}
\examples{
getExample = make.getExample()
STranspose = getExample('STranspose',toy=TRUE)
STranspose_sce = getExample('STranspose_sce',toy=TRUE)
hallmark = getExample('hallmark',toy=TRUE)
set.seed(100)
genes = sample(colnames(STranspose),5)
predictions = predictAnnotation(genes,hallmark,STranspose,radius=.5)
predictions_sce = predictAnnotation(genes,hallmark,STranspose_sce,radius=.5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{computeEdgeGraph}
\alias{computeEdgeGraph}
\title{This function takes a spatial graph and computes a new spatial graph where
edges become nodes and A-B edges (in the original graph) become connected to
all A- edges and all B- edges.}
\usage{
computeEdgeGraph(spatialGraph, selfEdges = FALSE)
}
\arguments{
\item{spatialGraph}{\itemize{
\item a data frame of neighbouring edge pairs.
}}

\item{selfEdges}{\itemize{
\item a logical determining whether to include self edges.
Defaults to False.
}}
}
\value{
a graph in neighbour format  where edges in the original graph
become nodes and A-B edges (in the original graph) become connected to
all A- edges and all B- edges.
}
\description{
This function takes a spatial graph and computes a new spatial graph where
edges become nodes and A-B edges (in the original graph) become connected to
all A- edges and all B- edges.
}
\examples{
delaunayNeighbours = make.getExample()('delaunayNeighbours')
edgeNeighbours = computeEdgeGraph(delaunayNeighbours)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeometricClustering.R
\name{medianComplementPValue}
\alias{medianComplementPValue}
\title{This takes a set S of n points in dimension d and a subset
A and computes a p-value for the co-localization of the subset
by comparing the median complement distance for the given set
to values of the median complement distance computed for random
subsets of the same size.}
\usage{
medianComplementPValue(S, idx, numTrials = 1000, returnTrials = FALSE)
}
\arguments{
\item{S}{\itemize{
\item an n x d matrix representing a set of n points
in dimension d
}}

\item{idx}{\itemize{
\item a logical of length n representing a subset of
S.  This should not be the empty set or all of S.
}}

\item{numTrials}{\itemize{
\item the number of random trials to perform,
defaults to 1000
}}

\item{returnTrials}{\itemize{
\item whether to report the real and random median
complement distances.
}}
}
\value{
By default this reports a p-value.  If returnTrials is set,
this returns a list giving the p-value, the actual complement distance
and the random complement distances.
}
\description{
This takes a set S of n points in dimension d and a subset
A and computes a p-value for the co-localization of the subset
by comparing the median complement distance for the given set
to values of the median complement distance computed for random
subsets of the same size.
}
\examples{
library(Seurat)
getExample = make.getExample()
STranspose = getExample('STranspose',toy=TRUE)
hallmark = getExample('hallmark')
S = data.matrix(FetchData(STranspose,c('umap_1','umap_2')))
idx = colnames(STranspose) \%in\% hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]]
mcpv = medianComplementPValue(S,idx,numTrials=100)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{readGmt}
\alias{readGmt}
\title{This function reads in gene sets in .gmt format}
\usage{
readGmt(gmtFile, addDescr = FALSE)
}
\arguments{
\item{gmtFile}{\itemize{
\item a .gmt file containing gene sets, e.g., Hallmark of GO
}}

\item{addDescr}{\itemize{
\item include gene set description (2nd column in .gmt file) in
gene set name
}}
}
\value{
\itemize{
\item A named list of gene sets
}
}
\description{
This function reads in gene sets in .gmt format
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleExampleServer.R
\name{exampleObjects}
\alias{exampleObjects}
\title{This returns the names of available example objects.}
\usage{
exampleObjects()
}
\value{
A character vector of the names of available
example data objects
}
\description{
This returns the names of available example objects.
}
\examples{
availableObjects = exampleObjects()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{seuratCells}
\alias{seuratCells}
\title{seuratCells}
\format{
A vector of cells

\describe{
A vector of cells consisting of half the cells from
each seurat_cluster in exSeuratObj used to subset this
object to give toy examples.
}
}
\source{
Computed by retrieving half the cells from each
cluster in exSeuratObj
}
\usage{
seuratCells
}
\description{
A vector of cells used for subsetting exSeuratObj
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{countLRInteractionsPerCell}
\alias{countLRInteractionsPerCell}
\title{This function takes a listing of the neighbouring
cells together with the presence or absence of each
ligand-receptor pair on each edge and produces a count
showing for each cell, how many neighbours it has with
that interaction either as source or as target}
\usage{
countLRInteractionsPerCell(edges, sourceOrTarget)
}
\arguments{
\item{edges}{\itemize{
\item A data frame of neighbouring cells
together with their interactions as produced by
getInteractionsOnEdges()
}}

\item{sourceOrTarget}{\itemize{
\item a character, either 'source' or
'target' telling which direction of interaction to count
}}
}
\value{
This returns a data frame with one row for each
cell and a column giving the name of that cell and the
other columns giving the counts of interactions that it
has with its neighbours.
}
\description{
This function takes a listing of the neighbouring
cells together with the presence or absence of each
ligand-receptor pair on each edge and produces a count
showing for each cell, how many neighbours it has with
that interaction either as source or as target
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{getGeneNeighbors}
\alias{getGeneNeighbors}
\title{This function gets the neighbors of a given gene
using either the gene Seurat object or its nearest
neighbor graph returned from getNearestNeighbourLists}
\usage{
getGeneNeighbors(gene, NN)
}
\arguments{
\item{gene}{\itemize{
\item the gene in question
}}

\item{NN}{\itemize{
\item either the gene Seurat object or its nearest
neighbor graph as found by getNearestNeighbourLists.
This can also be a SingleCellExperiment which will be converted
to a Seurat object
}}
}
\value{
the neighboring genes
}
\description{
This function gets the neighbors of a given gene
using either the gene Seurat object or its nearest
neighbor graph returned from getNearestNeighbourLists
}
\examples{
library(Seurat)
getExample = make.getExample()
STranspose = getExample('STranspose',toy=TRUE)
NN = getExample('NN',toy=TRUE)
neighbors = getGeneNeighbors("Ccl6",STranspose)
neighborsAgain = getGeneNeighbors("Ccl6",NN)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleEdgeQualityControl.R
\name{edgeLengthPlot}
\alias{edgeLengthPlot}
\title{edgeLengthPlot}
\usage{
edgeLengthPlot(annEdges, cutoffDF, whichPairs, xLim = 100, legend = FALSE)
}
\arguments{
\item{annEdges}{\itemize{
\item A data frame as produced by
edgeLengthsAndCellTypePairs
}}

\item{cutoffDF}{\itemize{
\item A data frame with columns cellTypePair and
cutoff. This defaults to NULL in which case no cutoffs will be
plotted.
}}

\item{whichPairs}{\itemize{
\item Which cellTypePairs to plot.  If this is NULL,
we plot all pairs.  If this is a numeric, we plot only pairs
that have at least this many edges.  If this is a character
vector, we plot the pairs in this list.
}}

\item{xLim}{\itemize{
\item limits the extent of the plots. Defaults to 100.  Can
be set to NULL.
}}

\item{legend}{\itemize{
\item Show legend, defaults to FALSE
}}
}
\value{
This returns a ggplot object
}
\description{
This plots histograms of the edge lengths broken out
by the cell types of the cells they connect.  It
optionally plots a cutoff for each pair of types.
}
\examples{
getExample = make.getExample()
centroids = getExample('centroids')
clusters = getExample('clusters')
delaunayNeighbours = getExample('delaunayNeighbours') 
annEdges =
   edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
cutoffDF = edgeCutoffsByPercentile(annEdges,95)
g = edgeLengthPlot(annEdges,cutoffDF,whichPairs=60)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{permuteColumns}
\alias{permuteColumns}
\title{This function permutes the columns of a sparse dcG matrix.}
\usage{
permuteColumns(M)
}
\arguments{
\item{M}{\itemize{
\item a binarised expression matrix in sparse format where rows are cells and columns
}}
}
\value{
This returns a matrix in which the values have been permuted within
columns.
}
\description{
This function permutes the columns of a sparse dcG matrix.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{exSeuratObj}
\alias{exSeuratObj}
\title{exSeuratObj}
\format{
A Seurat object

\describe{
A Seurat object of cells.  It includes a UMAP of the
cells and annotated clustering into cell types. It has
been severely reduced in size to accommodate Bioconductor
size restrictions.
}
}
\source{
This is subset from the data associated with
https://www.nature.com/articles/s41586-021-04006-z
}
\usage{
exSeuratObj
}
\description{
A Seurat object of 2000 genes by 540 cells.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{computeMoransI}
\alias{computeMoransI}
\title{This function takes a matrix where rows are features and columns are cells,
and a neighbourhood list, and computes Moran's I.}
\usage{
computeMoransI(M, nbhdList)
}
\arguments{
\item{M}{\itemize{
\item a matrix where column names are cells and row names are features.
}}

\item{nbhdList}{\itemize{
\item a named list with memberships of the neighbourhoods
of cells
}}
}
\value{
a matrix giving aggregated gene expression for a cell's neighbourhood.
}
\description{
This function takes a matrix where rows are features and columns are cells,
and a neighbourhood list, and computes Moran's I.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGenesVsCells.R
\name{getFeatureZScores}
\alias{getFeatureZScores}
\title{This gets z-scores for the values of features}
\usage{
getFeatureZScores(f, features = rownames(f), layer = "data")
}
\arguments{
\item{f}{\itemize{
\item a Seurat object of cells or SingleCellExperiment to
be converted to a Seurat object
}}

\item{features}{\itemize{
\item a set of features to retrieve z-scores for,
defaults to rownames(f)
}}

\item{layer}{\itemize{
\item the data layer to retrieve
}}
}
\value{
This returns a data frame with a column for each
feature and a row for each cell
}
\description{
This gets z-scores for the values of features
}
\examples{
getExample = make.getExample()
exSeuratObj = getExample('exSeuratObj',toy=TRUE)
df = getFeatureZScores(exSeuratObj)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNNGraphs.R
\name{randomiseNodeIndices}
\alias{randomiseNodeIndices}
\title{This function generates random indices for node B}
\usage{
randomiseNodeIndices(neighborListDf, n = 100, useWeights = FALSE)
}
\arguments{
\item{neighborListDf}{\itemize{
\item a dataframe containing the neighborlist
}}

\item{n}{\itemize{
\item the number of times to randomise indices
}}

\item{useWeights}{\itemize{
\item whether to preserve edgeweights.
}}
}
\value{
\itemize{
\item a matrix with randomised indices for node B
}
}
\description{
This function generates random indices for node B
}
\examples{
NN = make.getExample()('NN')
NN = desymmetriseNN(NN)
randomIndices = randomiseNodeIndices(NN,10,TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoods.R
\name{neighbourhoodDiameter}
\alias{neighbourhoodDiameter}
\title{neighbourhoodDiameter}
\usage{
neighbourhoodDiameter(neighbourhoods, centroids)
}
\arguments{
\item{neighbourhoods}{\itemize{
\item a list of neighbourhoods as
returned by nbhdsAsEdgesToNbhdsAsList
}}

\item{centroids}{\itemize{
\item the centroids of the cells
}}
}
\value{
a named numeric.  The names are the names
of the list neighbourhoods and the values are the
maximum distance within each neighbourhood
}
\description{
This function takes a list of neighbourhoods and and the
centroids of the cells and finds their diameters, i.e.,
for each neighbourhood, the maximum distance between.
}
\examples{
getExample = make.getExample()
centroids = getExample('centroids')
delaunayNeighbours = getExample('delaunayNeighbours')
cells = unique(c(delaunayNeighbours[,'nodeA'],delaunayNeighbours[,'nodeB']))
nbhds = nbhdsAsEdgesToNbhdsAsList(cells,delaunayNeighbours)
diameters = neighbourhoodDiameter(nbhds[seq_len(100)],centroids)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{combinatorialSpheres}
\alias{combinatorialSpheres}
\title{Discovers the combinatorial ball of a given radius
around a fixed set of genes in the nearest neighbor
graph of a Seurat object.}
\usage{
combinatorialSpheres(NN, origin, radius)
}
\arguments{
\item{NN}{\itemize{
\item a nearest neighbors graph
}}

\item{origin}{\itemize{
\item a gene or list of genes
}}

\item{radius}{\itemize{
\item the radius of the combinatorial ball
to be found.
}}
}
\value{
This returns a data frame whose columns are the
gene name, the radius from the origin at which it is found
}
\description{
Discovers the combinatorial ball of a given radius
around a fixed set of genes in the nearest neighbor
graph of a Seurat object.
}
\examples{
getExample = make.getExample()
NN = getExample('NN',toy=TRUE)
STranspose = getExample('STranspose',toy=TRUE)
spheres = combinatorialSpheres(NN,'Ccl6',3)
hallmark = getExample('hallmark')
geneSet = intersect(hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]],colnames(STranspose))
sphereAroundSet = combinatorialSpheres(NN,geneSet,1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{getBinarisedMatrix}
\alias{getBinarisedMatrix}
\title{This functions retrieves an expression matrix from a
seurat object or SingleCellExperiment and binarises it.}
\usage{
getBinarisedMatrix(obj, cutoff = 0, layer = "counts")
}
\arguments{
\item{obj}{\itemize{
\item a Seurat object or SingleCellExperiment to be
turned into a Seurat object
}}

\item{cutoff}{\itemize{
\item a cutoff for binarisation. Defaults to 0.
}}

\item{layer}{\itemize{
\item layer to fetch data from. Defaults to count.
}}
}
\value{
A binarised sparse expression matrix where rows are genes and
columns are cells.
}
\description{
This functions retrieves an expression matrix from a
seurat object or SingleCellExperiment and binarises it.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{getLigandReceptorNetwork}
\alias{getLigandReceptorNetwork}
\title{This function retrieves the Nichenetr ligand-
receptor network for mouse or human.}
\usage{
getLigandReceptorNetwork(species)
}
\arguments{
\item{species}{\itemize{
\item either 'human' or 'mouse'
}}
}
\value{
This returns a data frame whose first two
columns are from and to, i.e., ligand and receptor.
These are derived from the nichenetr ligand receptor
networks.
}
\description{
This function retrieves the Nichenetr ligand-
receptor network for mouse or human.
}
\examples{
lrn = getLigandReceptorNetwork('human')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNNGraphs.R
\name{getNearestNeighbourLists}
\alias{getNearestNeighbourLists}
\title{This function extracts a shared nearest neighbor network
from a Seurat object}
\usage{
getNearestNeighbourLists(f, graph = defaultGraph(f))
}
\arguments{
\item{f}{\itemize{
\item a Seurat object or SingleCellExperiment to
be converted to a Seurat object
}}

\item{graph}{\itemize{
\item which graph to extract.  Defaults to
paste0(f@active.assay,'_snn')
}}
}
\value{
\itemize{
\item This returns dataframe of neighbors:
nodeA - node names for node A
nodeB - node names for node B
weight - edge weight
}
}
\description{
This function extracts a shared nearest neighbor network
from a Seurat object
}
\examples{
STranspose = make.getExample()('STranspose',toy=TRUE)
NN = getNearestNeighbourLists(STranspose)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{computeEdgeObject}
\alias{computeEdgeObject}
\title{This function takes interactionResults and creates a seurat object where
each point represents an edge between cells, and spatial coordinates are the
centroids of edges between cells. The "expression matrix" is the
binarised presence/absence of an interaction (ligand receptor pair) on an edge.}
\usage{
computeEdgeObject(
  ligandReceptorResults,
  centroids,
  npcs = 10,
  returnType = "Seurat"
)
}
\arguments{
\item{ligandReceptorResults}{\itemize{
\item as returned by performLigandReceptorResultsAnalysis()
}}

\item{centroids}{\itemize{
\item a dataframe containing centroids
where rownames are cellnames and the first two columns
contain x and y coordinates respectively.
}}

\item{npcs}{\itemize{
\item number of pcs used for PCA, defaults to 10
}}

\item{returnType}{Determines whether to return a Seurat object or a
SpatialExperiment.  Will do the later if this is set to either SCE,
SingleCellExperiment or lower case versions of either.}
}
\value{
This returns a seurat object where
each point represents an edge between cells, and spatial coordinates are the
centroids of edges between cells. The "expression matrix" is the
binarised presence/absence of an interaction (ligand receptor pair) on an edge.
Depending on the parameter returnType, this can alternatively be returned as
a SpatialExperiment.
}
\description{
This function takes interactionResults and creates a seurat object where
each point represents an edge between cells, and spatial coordinates are the
centroids of edges between cells. The "expression matrix" is the
binarised presence/absence of an interaction (ligand receptor pair) on an edge.
}
\examples{
getExample = make.getExample()
centroids = getExample('centroids')
ligandReceptorResults = getExample('ligandReceptorResults')
edgeSeurat = computeEdgeObject(ligandReceptorResults, centroids)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoods.R
\name{nbhdsAsEdgesToNbhdsAsList}
\alias{nbhdsAsEdgesToNbhdsAsList}
\title{nbhdsAsEdgesToNbhdsAsList}
\usage{
nbhdsAsEdgesToNbhdsAsList(cells, neighbourhoods, self = FALSE)
}
\arguments{
\item{cells}{\itemize{
\item The cells whose neighbourhoods to
extract.
}}

\item{neighbourhoods}{\itemize{
\item neighbourhoods given as a
data frame with columns nodeA and nodeB, for example
the output of collapseNeighbourhoods
}}

\item{self}{\itemize{
\item include cell in its neighbourhood, defaults to FALSE
}}
}
\value{
a named list with memberships of the neighbourhoods
of cells
}
\description{
This function takes a set of neighbourhoods given
by edges and turns it into a named list giving the
memberships of each neighbourhood
}
\examples{
delaunayNeighbours = make.getExample()('delaunayNeighbours')
cells = unique(c(delaunayNeighbours[,'nodeA'],delaunayNeighbours[,'nodeB']))
nbhdsList = nbhdsAsEdgesToNbhdsAsList(cells,delaunayNeighbours)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{stripGeneSet}
\alias{stripGeneSet}
\title{This function strips out non-gene information from
the beginning of GO sets, etc.}
\usage{
stripGeneSet(geneSet)
}
\arguments{
\item{geneSet}{\itemize{
\item a list of gene sets
}}
}
\value{
a named list of gene sets
}
\description{
This function strips out non-gene information from
the beginning of GO sets, etc.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{transposeObject}
\alias{transposeObject}
\title{Create the transpose of a Seurat object}
\usage{
transposeObject(
  f,
  active.assay = "RNA",
  npcs = 30,
  dims = seq_len(20),
  res = 1,
  returnType = "Seurat",
  verbose = FALSE
)
}
\arguments{
\item{f}{\itemize{
\item a Seurat object
}}

\item{active.assay}{\itemize{
\item the assay to use.  Defaults to 'RNA'
}}

\item{npcs}{\itemize{
\item number of principal components, defaults to 30
}}

\item{dims}{\itemize{
\item dimensions to use for umap and nearest neighbors,
defaults to 1:20
}}

\item{res}{\itemize{
\item the clustering resolution, defaults to 1
}}

\item{returnType}{\itemize{
\item Will return a SingleCellExperiment if this is either
of SCE, SingleCellExperiment or their lower-case equivalents.  Otherwise,
returns a Seurat object
}}

\item{verbose}{\itemize{
\item Controls whether to display trace from the Seurat
functions. Defaults to FALSE
}}
}
\value{
A Seurat object or SingleCellExperiment
}
\description{
This takes a Seurat object f and creates a new Seurat object whose
expression matrix is the transpose of that of f.  This can
also be a SingleCellExperiment which will be converted to a
Seurat object
}
\examples{
exSeuratObj = make.getExample()('exSeuratObj',toy=TRUE)
STranspose = transposeObject(exSeuratObj)
STransposeAsSCE = transposeObject(exSeuratObj,returnType='SCE')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{aggregateGeneExpression}
\alias{aggregateGeneExpression}
\title{This function takes a Seurat object and a list of
neighbourhoods and creates a Seurat object where the
columns are the neighbourhoods, the rows are are the
genes and the values are gene expression totals for
the cells in each neighbourhood}
\usage{
aggregateGeneExpression(
  f,
  neighbourhoods,
  self = FALSE,
  verbose = TRUE,
  returnType = "Seurat"
)
}
\arguments{
\item{f}{\itemize{
\item a Seurat object with layer counts or a SingleCellExperiment
to be turned into a Seurat object
}}

\item{neighbourhoods}{\itemize{
\item Neighbourhoods as given by a
collapsed expanded edge graph, as produced by
collapseNeighbourhoods. In particular, each cell should
appear as nodeA.
}}

\item{self}{\itemize{
\item include cell in its neighbourhood, defaults to FALSE
}}

\item{verbose}{\itemize{
\item used to control trace, defaults to TRUE
}}

\item{returnType}{\itemize{
\item Will return a SingleCellExperiment if this is either
of SCE, SingleCellExperiment or their lower-case equivalents.  Otherwise,
returns a Seurat object or SingleCellExperiment, depending on the
parameter returnType.
}}
}
\value{
a Seurat object giving total gene expression
in each neighbourhood or SingleCellExperiment
}
\description{
This function takes a Seurat object and a list of
neighbourhoods and creates a Seurat object where the
columns are the neighbourhoods, the rows are are the
genes and the values are gene expression totals for
the cells in each neighbourhood
}
\examples{
getExample = make.getExample()
smallXenium = getExample('smallXenium',toy=TRUE)
extendedNeighbours = getExample('extendedNeighbours',toy=TRUE)
agg = aggregateGeneExpression(smallXenium,extendedNeighbours,verbose=FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNNGraphs.R
\name{randomiseGraph}
\alias{randomiseGraph}
\title{This function performs degree-preserving randomisation of neighbour graphs.}
\usage{
randomiseGraph(spatialGraph, maxTries = 1000)
}
\arguments{
\item{spatialGraph}{\itemize{
\item a spatial graph in neighbour list format.
}}

\item{maxTries}{\itemize{
\item the maximum number of tries to remove self edges during
graph randomisation. If self edges are remaining this will be reported.
}}
}
\value{
A randomised graph where degree from the original graph is preserved.
We also report any duplicated edges.
}
\description{
This function performs degree-preserving randomisation of neighbour graphs.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{annotateLRInteractionCounts}
\alias{annotateLRInteractionCounts}
\title{This takes a data frame of interaction counts as found
by countLRInteractionsPerCell(), the underlying Seurat object
and the neighbourhood Seurat object and annotates the counts
with the cell type and the neighbourhood type corresponding
to the cells of the interaction counts.}
\usage{
annotateLRInteractionCounts(interactionCounts, obj, nbhdObj)
}
\arguments{
\item{interactionCounts}{\itemize{
\item as found by countLRInteractionsPerCell()
}}

\item{obj}{\itemize{
\item a Seurat object, or SingleCellExperiment to be turned
into a Seurat object
}}

\item{nbhdObj}{\itemize{
\item a neighbourhood x cell type Seurat object or a
SingleCellExperiment to be turned into a Seurat object
}}
}
\value{
This returns the interaction counts annotated with the
cell type and neighbourhood type of each cell.
}
\description{
This takes a data frame of interaction counts as found
by countLRInteractionsPerCell(), the underlying Seurat object
and the neighbourhood Seurat object and annotates the counts
with the cell type and the neighbourhood type corresponding
to the cells of the interaction counts.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{mouseLRN}
\alias{mouseLRN}
\title{mouseLRN}
\format{
a data frame with two columns, 'from' and 'to'
\describe{A data frame with two columns, 'from' and 'to'.
Each row represents a mouse ligand - receptor pair.
}
}
\source{
This is taken from the nichenetr package,
url = {https://www.nature.com/articles/s41592-019-0667-5}.
Specifically, we use the mouse ligand - receptor network.
}
\usage{
mouseLRN
}
\description{
A data frame giving 11592 mouse ligand receptor pairs
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{cellTypesPerCellTypeGraphFromCellMatrix}
\alias{cellTypesPerCellTypeGraphFromCellMatrix}
\title{This function converts a matrix as found by
cellTypesPerCellTypeMatrix into a directed igraph
whose vertices correspond to seurat_clusters and whose
edge correspond to occupancy fraction.}
\usage{
cellTypesPerCellTypeGraphFromCellMatrix(
  M,
  colours = NULL,
  selfEdges = FALSE,
  minWeight = 0,
  edgeWeighting = 20,
  edgeCurved = 0.2,
  arrowSize = 4,
  arrowWidth = 4,
  plotGraph = TRUE
)
}
\arguments{
\item{M}{\itemize{
\item a matrix as found by cellTypesPerCellTypeMatrix. Note,
however, that this matrix may need to be reduced to a square matrix
as the matrix produced from a subset object may be missing certain
cell types as rows.
}}

\item{colours}{\itemize{
\item a named vector of colours used to colour the
vertices of the graph.  The names are the seurat_clusters
as character strings.
}}

\item{selfEdges}{\itemize{
\item a logical which determines whether to include
self edges.  Defaults to FALSE
}}

\item{minWeight}{\itemize{
\item Allows one to exclude edges of low weight.
Defaults to 0, thus including all edges.
}}

\item{edgeWeighting}{\itemize{
\item a parameter used to thicken the edges
in the display.  Defaults to 20.
}}

\item{edgeCurved}{\itemize{
\item a parameter to set curvature of the edges.
Defaults to 0.2
}}

\item{arrowSize}{\itemize{
\item a parameter to set arrow size. Defaults to 4.
}}

\item{arrowWidth}{\itemize{
\item a parameter to set arrow width. Defaults to 4.
}}

\item{plotGraph}{\itemize{
\item a logical which determines whether to
plot the graph.  Defaults to TRUE.
}}
}
\value{
This returns a directed igraph whose vertices are
the cell types and whose arrows indicate "ownership" of
cells of the target type by neighbourhoods of cells of the
source type.  Layout is done witht the FR algorithm and
coordinates are found in the coords attribute of G.  If colours
were supplied these are found in color attribute of V(G).  Edge
weights and widths are found in the weight and width attributes
of E(G).
}
\description{
This function converts a matrix as found by
cellTypesPerCellTypeMatrix into a directed igraph
whose vertices correspond to seurat_clusters and whose
edge correspond to occupancy fraction.
}
\examples{
getExample = make.getExample()
cellTypesPerCellTypeMatrix = getExample('cellTypesPerCellTypeMatrix')
colours = getExample('colours')
G = cellTypesPerCellTypeGraphFromCellMatrix(cellTypesPerCellTypeMatrix, 
                                   minWeight = 0.05, colours = colours)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNNGraphs.R
\name{desymmetriseNN}
\alias{desymmetriseNN}
\title{This function takes the data frame of neighbor genes
and reduces it so that each undirected edge is
represented by only one directed edge.  This ensures
that randomisation does not magically split undirected
edges into two edges.}
\usage{
desymmetriseNN(NN)
}
\arguments{
\item{NN}{\itemize{
\item a dataframe containing the neighborlist
}}
}
\value{
\itemize{
\item a neighborListDF with only one directed edge per
undirected edge.
}
}
\description{
This function takes the data frame of neighbor genes
and reduces it so that each undirected edge is
represented by only one directed edge.  This ensures
that randomisation does not magically split undirected
edges into two edges.
}
\examples{
NN = make.getExample()('NN',toy=TRUE)
print(dim(NN))
NNN = desymmetriseNN(NN)
print(dim(NNN))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoods.R
\name{computeNeighboursDelaunay}
\alias{computeNeighboursDelaunay}
\title{This function computes a spatial graph where
neighbors are identified based on Delaunay triangulation.}
\usage{
computeNeighboursDelaunay(centroids)
}
\arguments{
\item{centroids}{\itemize{
\item a dataframe containing centroids
where rownames are cellnames and the first two columns
contain x and y coordinates respectively.
}}
}
\value{
a graph in neighbour format, i.e., a data frame with
columns nodeA and nodeB.
}
\description{
This function computes a spatial graph where
neighbors are identified based on Delaunay triangulation.
}
\examples{
centroids = make.getExample()('centroids')
delaunayNeighbours = computeNeighboursDelaunay(centroids)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoods.R
\name{computeNeighboursEuclidean}
\alias{computeNeighboursEuclidean}
\title{This function computes a spatial graph where
neighbors are identified based on euclidean distance and a
user defined threshold.}
\usage{
computeNeighboursEuclidean(centroids, threshold)
}
\arguments{
\item{centroids}{\itemize{
\item a dataframe containing centroids
where rownames are cellnames and columns contain x
and y coordinates respectively.
}}

\item{threshold}{\itemize{
\item a distance cut off to compute neighbours.
}}
}
\value{
a graph in neighbour format, i.e., a data frame with
columns nodeA and nodeB.
}
\description{
This function computes a spatial graph where
neighbors are identified based on euclidean distance and a
user defined threshold.
}
\examples{
centroids = make.getExample()('centroids')
euclideanNeighbours = computeNeighboursEuclidean(centroids,20)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{getNearbyGenes}
\alias{getNearbyGenes}
\title{Nearby genes}
\usage{
getNearbyGenes(
  fPrime,
  geneSet,
  radius,
  metric = "umap",
  numPCs = NULL,
  weights = FALSE
)
}
\arguments{
\item{fPrime}{\itemize{
\item a Seurat object of genes or
SingleCellExperiment to be converted to a Seurat object
}}

\item{geneSet}{\itemize{
\item set of genes
}}

\item{radius}{\itemize{
\item the distance around the given set
}}

\item{metric}{\itemize{
\item the metric to use, one of umap, tsne,
pca or nearest neighbor
}}

\item{numPCs}{\itemize{
\item used only if the metric is pca
}}

\item{weights}{\itemize{
\item whether to use edge weights in the NN case
}}
}
\value{
This returns a named vector whose values are distance
from geneSet and whose names are the nearby genes.
}
\description{
This finds the genes near a give subset using either
a dimensional reduction or the nearest neighbor graph
}
\examples{
getExample = make.getExample()
STranspose = getExample('STranspose',toy=TRUE)
hallmark = getExample('hallmark')
geneSet = intersect(colnames(STranspose),hallmark[["HALLMARK_TNFA_SIGNALING_VIA_NFKB"]])
geometricallyNearby = getNearbyGenes(STranspose,geneSet,radius=0.2,metric='umap')
combinatoriallyNearby = getNearbyGenes(STranspose,geneSet,radius=1,metric='NN')
weightedNearby = getNearbyGenes(STranspose,'Myc',radius=1,metric='NN',weights=TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{annotateGeneAsVector}
\alias{annotateGeneAsVector}
\title{This function returns a numeric indicating which gene
sets it does and does not belong to.  This vector can
be normalised to account for the sizes of the sets.}
\usage{
annotateGeneAsVector(gene, geneSets, normalise = FALSE)
}
\arguments{
\item{gene}{\itemize{
\item the gene to annotate
}}

\item{geneSets}{\itemize{
\item a list of gene sets
}}

\item{normalise}{\itemize{
\item whether to normalise by set size
}}
}
\value{
a numeric
}
\description{
This function returns a numeric indicating which gene
sets it does and does not belong to.  This vector can
be normalised to account for the sizes of the sets.
}
\examples{
hallmark = make.getExample()('hallmark')
Myc = annotateGeneAsVector('Myc',hallmark)
MycNormalised = annotateGeneAsVector('Myc',hallmark,TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{getInteractionsOnEdges}
\alias{getInteractionsOnEdges}
\title{This function takes a binarised expression matrix, a set of ligand receptor
pairs and a set of edges denoting neighbouring cells and
annotates these with the ligand receptor interactions taking
place on those edges in each direction.}
\usage{
getInteractionsOnEdges(M, pairDF, spatialGraph)
}
\arguments{
\item{M}{\itemize{
\item a binarised expression matrix where rows are genes and columns
are cells.
}}

\item{pairDF}{\itemize{
\item a data frame giving the ligand-receptor pairs
}}

\item{spatialGraph}{\itemize{
\item a data frame of neighbouring
cell pairs.  Note that each row is a directed edge (A,B) so
that this data frame should have both the edge (A,B) and the
edge (B,A)
}}
}
\value{
This returns a data frame whose first two columns give
the neighbouring cells.  Each of the remaining columns is a logical
corresponding to a ligand-receptor pair telling whether the ligand
is expressed in the first cell and the receptor is expressed in the
second cell.
}
\description{
This function takes a binarised expression matrix, a set of ligand receptor
pairs and a set of edges denoting neighbouring cells and
annotates these with the ligand receptor interactions taking
place on those edges in each direction.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleLigandReceptorAnalysis.R
\name{plotLRDotplot}
\alias{plotLRDotplot}
\title{This is a function to create a dotplot using the ligand receptor results}
\usage{
plotLRDotplot(
  ligandReceptorResults,
  senderClusters = unique(ligandReceptorResults$interactionsOnEdgesMeta$cellTypeA),
  receiverClusters = unique(ligandReceptorResults$interactionsOnEdgesMeta$cellTypeB),
  padjCutoff = 0.05,
  pvalCutoff = F,
  splitBy = "sender"
)
}
\arguments{
\item{ligandReceptorResults}{\itemize{
\item ligandReceptorResults calculated using
performLigandReceptorAnalysis().
}}

\item{senderClusters}{\itemize{
\item sender clusters to plot (defaults to all).
}}

\item{receiverClusters}{\itemize{
\item receiver clusters to plot (defaults to all).
}}

\item{padjCutoff}{\itemize{
\item only plot results with p-adj < padjCutoff (defaults to
0.05).
}}

\item{pvalCutoff}{\itemize{
\item only plot results with p-value < pvalCutoff (defaults to
False in which case padjCutoff is used).
}}

\item{splitBy}{\itemize{
\item split plots by "sender" or "receiver" cell types (defaults
to sender).
}}
}
\value{
matrix of total ligand receptor interactions that underlies the heatmap.
}
\description{
This is a function to create a dotplot using the ligand receptor results
}
\examples{
getExample = make.getExample()
centroids = getExample('centroids')
ligandReceptorResults = getExample('ligandReceptorResults')
p = plotLRDotplot(ligandReceptorResults)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeneAnnotation.R
\name{annotateGenesByGeneSet}
\alias{annotateGenesByGeneSet}
\title{This function annotates genes with terms}
\usage{
annotateGenesByGeneSet(geneSets)
}
\arguments{
\item{geneSets}{\itemize{
\item a list of gene sets, e.g., as produced by
readGmt
}}
}
\value{
\itemize{
\item A list where names are genes and values are lists of terms
}
}
\description{
This essentially inverts a list of gene sets.  It takes
a list (e.g., Hallmark or GO) where each list item is a
name of a gene set and gives the genes in that set and
returns a list where each item is a gene and gives the
gene sets that gene is in.
}
\examples{
hallmark = make.getExample()('hallmark')
annotatedGenes = annotateGenesByGeneSet(hallmark)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleGeometricClustering.R
\name{medianComplementDistance}
\alias{medianComplementDistance}
\title{This takes a set S of n points in dimension d given
by an n x d matrix and a subset A given by a logical
and returns the median distance from the complement to
the given subset.}
\usage{
medianComplementDistance(S, idx)
}
\arguments{
\item{S}{\itemize{
\item an n x d matrix representing a set of n points
in dimension d
}}

\item{idx}{\itemize{
\item a logical of length n representing a subset of
S.  This should not be the empty set or all of S.
}}
}
\value{
This returns the median distance from the complement
to the subset
}
\description{
This takes a set S of n points in dimension d given
by an n x d matrix and a subset A given by a logical
and returns the median distance from the complement to
the given subset.
}
\examples{
S = matrix(seq_len(12),ncol=2)
idx = c(rep(FALSE,3),rep(TRUE,3))
compDist = medianComplementDistance(S,idx)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleEdgeQualityControl.R
\name{edgeCutoffsByWatershed}
\alias{edgeCutoffsByWatershed}
\title{This finds proposed cutoffs for edge lengths by computing
the histogram of edge lengths for each cell type pair and
then using the watershed algorithm to find the hump of the
histogram containing the median.}
\usage{
edgeCutoffsByWatershed(annEdges, nbins = 15, tolerance = 10)
}
\arguments{
\item{annEdges}{\itemize{
\item a data frame with columns nodeA, nodeB, length
and cellTypePair as produced by edgeLengthsAndCellTypePairs.
}}

\item{nbins}{\itemize{
\item the number of bins for the histogram
}}

\item{tolerance}{\itemize{
\item the tolerance parameter for the watershed
algorithm.
}}
}
\value{
This returns a data frame with columns cellTypePair and
cutoff.
}
\description{
This finds proposed cutoffs for edge lengths by computing
the histogram of edge lengths for each cell type pair and
then using the watershed algorithm to find the hump of the
histogram containing the median.
}
\examples{
getExample = make.getExample()
centroids = getExample('centroids')
clusters = getExample('clusters')
delaunayNeighbours = getExample('delaunayNeighbours') 
annEdges =
    edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
cutoffDF = edgeCutoffsByWatershed(annEdges)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNNGraphs.R
\name{symmetriseNN}
\alias{symmetriseNN}
\title{This symmetrises a nearest neighbors graph.}
\usage{
symmetriseNN(NN)
}
\arguments{
\item{NN}{\itemize{
\item a nearest neighbors graph as returned
by getNearestNeighbourLists
}}
}
\value{
a nearest neighbors graph
}
\description{
This first checks to see if the NN graph is symmetric
and if not symmetrises it.
}
\examples{
NN = make.getExample()('NN',toy=TRUE)
NNStar = symmetriseNN(NN)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleEdgeQualityControl.R
\name{cullEdges}
\alias{cullEdges}
\title{This subsets edges by our chosen critera}
\usage{
cullEdges(annEdges, cutoffSpec)
}
\arguments{
\item{annEdges}{\itemize{
\item a data frame with columns nodeA, nodeB, length
and cellTypePair as produced by edgeLengthsAndCellTypePairs.
}}

\item{cutoffSpec}{\itemize{
\item This can be either a numeric value which will
be applied across all edges as an upper limit or a data frame
with columns cellTypePair and cutoff as produced by any of the
edgeCutoffsBy functions
}}
}
\value{
This returns a subset of the annotated edges
}
\description{
This subsets edges by our chosen critera
}
\examples{
getExample = make.getExample()
centroids = getExample('centroids')
clusters = getExample('clusters')
delaunayNeighbours = getExample('delaunayNeighbours') 
annEdges =
    edgeLengthsAndCellTypePairs(delaunayNeighbours,clusters,centroids)
tolerance = 5
nbins = 15
cutoffDFWater = edgeCutoffsByWatershed(annEdges,
                                      tolerance=tolerance,
                                      nbins=nbins)
culledEdges = cullEdges(annEdges,cutoffDFWater)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CatsCradleNeighbourhoodAnalysis.R
\name{computeNBHDVsCTObject}
\alias{computeNBHDVsCTObject}
\title{This function creates a seurat object using a neighbourhood by cell type
matrix}
\usage{
computeNBHDVsCTObject(
  dataMatrix,
  resolution = 0.1,
  npcs = 10,
  n.neighbors = 30L,
  transpose = FALSE,
  verbose = TRUE,
  returnType = "Seurat"
)
}
\arguments{
\item{dataMatrix}{\itemize{
\item a matrix of neighbourhoods by cell types or its
transpose.
}}

\item{resolution}{\itemize{
\item resolution for clustering (default 0.1).
}}

\item{npcs}{\itemize{
\item number of pcs used for PCA, defaults to 10.
}}

\item{n.neighbors}{\itemize{
\item number of neighbors used by UMAP, defaults to 30.
}}

\item{transpose}{\itemize{
\item defaults to FALSE.
}}

\item{verbose}{\itemize{
\item defaults to TRUE, used to limit trace if FALSE
}}

\item{returnType}{\itemize{
\item Will return a SingleCellExperiment if this is either
of SCE, SingleCellExperiment or their lower-case equivalents.  Otherwise,
returns a Seurat object
}}
}
\value{
a seurat object based on a neighbourhood by cell type matrix or its
transpose, containing clusters and UMAP. This can also be a
SingleCellExperiment depending on the parameter returnType.
}
\description{
This function creates a seurat object using a neighbourhood by cell type
matrix
}
\examples{
NBHDByCTMatrix = make.getExample()('NBHDByCTMatrix',toy=TRUE)
NBHDByCTSeurat = computeNBHDVsCTObject(NBHDByCTMatrix)
NBHDByCTSingleCell_sce = computeNBHDVsCTObject(NBHDByCTMatrix,returnType='SCE')
}
